﻿using System;
using System.Collections.Generic;
using MinerWars.CommonLIB.AppCode.Utils;
using MinerWars.AppCode.Game.Utils;
using MinerWars.AppCode.Game.Entities;
using SysUtils.Utils;
using SysUtils;
using SharpDX.XACT3;

namespace MinerWars.AppCode.Game.Audio
{
    enum MyMusicTransitionEnum
    {
        MainMenu,
        Horror,
        Mystery,
        LightFight,
        TensionBeforeAnAction,
        CalmAtmosphere,
        HeavyFight,
        Victory,
        SadnessOrDesperation,
        StressOrTimeRush,
        StealthAction,
        DesperateWithStress,
        Special,
    }

    //  Add here every sound/cue we use in the game. You don't need to add it to other places, that is done automaticaly by LoadContent
    //  IMPORTANT: Name of enum must match cube name from XACT project    
    //  IMPORTANT: DO NOT USE MULTILINE COMMETS OR REGIONS!
    //  IMPORTANT: Ambient 2d sounds are stored as part of influence spheres, so they need to have their ids predfined
    //  IMPORTANT: If you add new cue add it to: m_cueLoopability, m_3dSoundsTo2d
    internal enum MySoundCuesEnum : short
    {
        GuiMouseClick = 0,
        GuiMouseOver = 1,
        GuiEditorFlyOutsideBorder = 2,
        GuiEditorObjectAttach = 3,
        GuiEditorObjectDelete = 4,
        GuiEditorObjectDetach = 5,
        GuiEditorObjectMoveInvalid = 6,
        GuiEditorObjectMoveStep = 7,
        GuiEditorObjectRotateStep = 8,
        GuiEditorObjectSelect = 9,
        GuiEditorPrefabCommit = 10,
        GuiEditorPrefabEnter = 11,
        GuiEditorPrefabExit = 12,
        GuiEditorVoxelHandAdd = 13,
        GuiEditorVoxelHandRemove = 14,
        GuiEditorVoxelHandSoften = 15,
        GuiEditorVoxelHandMaterial = 16,
        GuiEditorVoxelHandSwitch = 17,
        GuiWheelControlOpen = 18,
        GuiWheelControlClose = 19,
        ImpRockCollideMetal = 20,
        ImpRockCollideRock = 21,
        ImpBulletHitRock = 22,
        ImpBulletHitMetal = 23,
        ImpBulletHitGlass = 24,
        ImpBulletHitShip = 25,
        ImpExpHitGlass = 26,
        ImpExpHitMetal = 27,
        ImpExpHitShip = 28,
        ImpExpHitRock = 29,
        ImpShipCollideMetal = 30,
        ImpShipCollideRock = 31,
        ImpPlayerShipCollideMetal = 32,
        ImpPlayerShipCollideRock = 33,
        ImpPlayerShipCollideShip = 34,
        ImpPlayerShipScrapeShipLoop = 35,
        ImpPlayerShipScrapeShipRelease = 36,
        VehShipaEngineIdle2d = 37,
        VehShipaEngineIdle3d = 38,
        VehShipaEngineHigh2d = 39,
        VehShipaEngineHigh3d = 40,
        VehShipaThrust2d = 41,
        VehShipaThrust3d = 42,
        VehShipaLightsOn = 43,
        VehShipaLightsOff = 44,
        VehShipaEngineOn = 45,
        VehShipaEngineOff = 46,
        VehHarvesterTubeRelease2d = 47,
        VehHarvesterTubeRelease3d = 48,
        VehHarvesterTubeMovingLoop2d = 49,
        VehHarvesterTubeMovingLoop3d = 50,
        VehHarvesterTubeColliding2d = 51,
        VehHarvesterTubeColliding3d = 52,
        VehHarvesterTubeCollision2d = 53,
        VehHarvesterTubeCollision3d = 54,
        VehHarvesterTubeImplode2d = 55,
        VehHarvesterTubeImplode3d = 56,
        VehToolCrusherDrillLoop2d = 57,
        VehToolCrusherDrillLoop3d = 58,
        VehToolCrusherDrillRelease2d = 59,
        VehToolCrusherDrillRelease3d = 60,
        VehToolCrusherDrillColliding2d = 61,
        VehToolCrusherDrillColliding3d = 62,
        VehToolCrusherDrillCollidingRelease2d = 63,
        VehToolCrusherDrillCollidingRelease3d = 64,
        VehToolThermalDrillLoop2d = 65,
        VehToolThermalDrillLoop3d = 66,
        VehToolThermalDrillRelease2d = 67,
        VehToolThermalDrillRelease3d = 68,
        VehToolSawCut2d = 69,
        VehToolSawCut3d = 70,
        VehToolSawLoop2d = 71,
        VehToolSawLoop3d = 72,
        VehToolSawRelease2d = 73,
        VehToolSawRelease3d = 74,
        VehToolLaserDrillLoop2d = 75,
        VehToolLaserDrillLoop3d = 76,
        VehToolLaserDrillRelease2d = 77,
        VehToolLaserDrillRelease3d = 78,
        WepMissileLock = 79,
        WepMissileFly = 80,
        WepMissileExplosion = 81,
        WepMissileLaunch2d = 82,
        WepMissileLaunch3d = 83,
        WepAutocanonFire2d = 84,
        WepAutocanonFire3d = 85,
        WepAutocanonRel2d = 86,
        WepAutocanonRel3d = 87,
        WepSniperScopeZoomRel = 88,
        WepSniperScopeZoomALoop = 89,
        WepSniperNormFire2d = 90,
        WepSniperHighFire2d = 91,
        WepSniperNormFire3d = 92,
        WepSniperHighFire3d = 93,
        WepMineMoveALoop = 94,
        WepUnivLaunch2d = 95,
        WepUnivLaunch3d = 96,
        WepArsHighShot2d = 97,
        WepArsHighShot3d = 98,
        WepArsNormShot2d = 99,
        WepArsNormShot3d = 100,
        WepBombExplosion = 101,
        WepLargeShipAutocannonRotate = 102,
        WepMachineGunHighFire2d = 103,
        WepMachineGunHighFire3d = 104,
        WepMachineGunHighRel2d = 105,
        WepMachineGunHighRel3d = 106,
        WepMachineGunNormFire2d = 107,
        WepMachineGunNormFire3d = 108,
        WepMachineGunNormRel2d = 109,
        WepMachineGunNormRel3d = 110,
        WepShotgunNormShot2d = 111,
        WepShotgunNormShot3d = 112,
        WepShotgunHighShot2d = 113,
        WepShotgunHighShot3d = 114,
        WepRailNormShot3d = 115,
        WepRailNormShot2d = 116,
        WepRailHighShot3d = 117,
        WepRailHighShot2d = 118,
        WepBombSmartTimer = 119,
        WepBombSmartSmoke = 120,
        WepBombSmartPlant = 121,
        WepBombSmartDrone = 122,
        WepBombGravSuck = 123,
        WepBombFlash = 124,
        SfxSolarWind = 125,
        SfxHudBackcameraOn = 126,
        SfxHudBackcameraOff = 127,
        SfxPlayerBreath = 128,
        SfxShipSmallExplosion = 129,
        SfxHudReflectorRange = 130,
        SfxHudSlowMovementOff = 131,
        SfxHudSlowMovementOn = 132,
        SfxHudAutolevelingOn = 133,
        SfxHudAutolevelingOff = 134,
        SfxHudWeaponScroll = 135,
        SfxHudWeaponSelect = 136,
        SfxHudRadarMode = 137,
        SfxHudCockpitOn = 138,
        SfxHudCockpitOff = 139,
        SfxSpark = 140,
        MovDoor1AClose = 141,
        MovDoor1AOpen = 142,
        MovDoor1BClose = 143,
        MovDoor1BOpen = 144,
        MovDoor2AClose = 145,
        MovDoor2AOpen = 146,
        MovDoor2BClose = 147,
        MovDoor2BOpen = 148,
        MovDoor3AClose = 149,
        MovDoor3AOpen = 150,
        HudFuelLowWarning = 151,
        HudFuelCriticalWarning = 152,
        HudOxygenLowWarning = 153,
        HudOxygenCriticalWarning = 154,
        HudOxygenNoWarning = 155,
        HudDamageAlertWarning = 156,
        HudDamageCriticalWarning = 157,
        HudAmmoCriticalWarning = 158,
        HudAmmoLowWarning = 159,
        HudArmorCriticalWarning = 160,
        HudArmorLowWarning = 161,
        HudDestinationReached = 162,
        HudRadarJammedWarning = 163,
        HudTargetDestroyed = 164,
        HudRadiationWarning = 165,
        SfxGeigerCounterHeavyLoop = 166,
        HudEnemyAlertWarning = 167,
        HudFriendlyFireWarning = 168,
        HudHarvestingComplete = 169,
        HudSolarFlareWarning = 170,
        HudInventoryComplete = 171,
        HudInventoryFullWarning = 172,
        HudInventoryTransfer = 173,
        HudObjectiveComplete = 174,
        MusSpecial_LostInTheDistance = 175,
        HudHealthLowWarning = 176,
        HudHealthCriticalWarning = 177,
        SfxClaxonAlert = 178,
        VehToolCrusherDrillIdle2d = 179,
        VehToolCrusherDrillIdle3d = 180,
        VehToolLaserDrillIdle2d = 181,
        VehToolLaserDrillIdle3d = 182,
        VehToolSawIdle2d = 183,
        VehToolSawIdle3d = 184,
        VehToolThermalDrillIdle2d = 185,
        VehToolThermalDrillIdle3d = 186,
        SfxGps = 187,
        SfxGpsFail = 188,
        MenuWelcome = 189,
        MovDock1Start = 190,
        MovDock1Loop = 191,
        MovDock1End = 192,
        MovDock2Start = 193,
        MovDock2Loop = 194,
        MovDock2End = 195,
        MovDock3Start = 196,
        MovDock3Loop = 197,
        MovDock3End = 198,
        VehCH1EngineHigh2d = 199,
        VehCH1EngineHigh3d = 200,
        VehCH1EngineIdle2d = 201,
        VehCH1EngineIdle3d = 202,
        VehEL1EngineHigh2d = 203,
        VehEL1EngineHigh3d = 204,
        VehEL1EngineIdle2d = 205,
        VehEL1EngineIdle3d = 206,
        VehEL1EngineOff = 207,
        VehEL1EngineOn = 208,
        VehNU1EngineHigh2d = 209,
        VehNU1EngineHigh3d = 210,
        VehNU1EngineIdle2d = 211,
        VehNU1EngineIdle3d = 212,
        ImpShipQuake = 213,
        Amb3D_Electrical01 = 214,
        Amb3D_Electrical02 = 215,
        Amb3D_FanLargeDamaged = 216,
        Amb3D_FanLargeDestroyed = 217,
        Amb3D_FanLargeNormal = 218,
        Amb3D_FanMediumDamaged = 219,
        Amb3D_FanMediumDestroyed = 220,
        Amb3D_FanMediumNormal = 221,
        Amb3D_FanSmallDamaged = 222,
        Amb3D_FanSmallDestroyed = 223,
        Amb3D_FanSmallNormal = 224,
        Amb3D_GenLargeDamaged = 225,
        Amb3D_GenLargeDestroyed = 226,
        Amb3D_GenLargeNormal = 227,
        Amb3D_GenMediumDamaged = 228,
        Amb3D_GenMediumDestroyed = 229,
        Amb3D_GenMediumNormal = 230,
        Amb2D_RoomLarge01 = 231,
        Amb2D_RoomLarge02 = 232,
        Amb2D_RoomLarge03 = 233,
        Amb2D_RoomLarge04 = 234,
        Amb2D_RoomLarge05 = 235,
        Amb2D_RoomMed01 = 236,
        Amb2D_RoomMed02 = 237,
        Amb2D_RoomMed03 = 238,
        Amb2D_RoomMed04 = 239,
        Amb2D_RoomMed05 = 240,
        Amb2D_RoomSmall01 = 241,
        Amb2D_RoomSmall02 = 242,
        Amb2D_RoomSmall03 = 243,
        Amb2D_RoomSmall04 = 244,
        Amb2D_StressLoop = 245,
        Amb2D_TunnelLarge01 = 246,
        Amb2D_TunnelMedium01 = 247,
        Amb2D_TunnelSmall01 = 248,
        VehToolLaserDrillColliding2d = 249,
        VehToolLaserDrillColliding3d = 250,
        VehToolLaserDrillCollidingRelease2d = 251,
        VehToolLaserDrillCollidingRelease3d = 252,
        VehToolThermalDrillColliding2d = 253,
        VehToolThermalDrillColliding3d = 254,
        VehToolThermalDrillCollidingRelease2d = 255,
        VehToolThermalDrillCollidingRelease3d = 256,
        SfxProgressHack = 257,
        SfxCancelHack = 258,
        SfxProgressRepair = 259,
        SfxProgressBuild = 260,
        SfxCancelRepair = 261,
        SfxCancelBuild = 262,
        Amb3D_GenXstart = 263,
        Amb3D_GenXloop = 264,
        SfxAlertVoc = 265,
        Amb3D_GenXend = 266,
        Amb3D_Temple1 = 267,
        Amb3D_Temple2 = 268,
        Amb3D_Temple3 = 269,
        SfxShipLargeExplosion = 270,
        MusCalmAtmosphere_MM01 = 271,
        MusCalmAtmosphere_MM02 = 272,
        MusDesperateWithStress_MM01 = 273,
        MusDesperateWithStress_KA01 = 274,
        MusCalmAtmosphere_KA01 = 275,
        MusHeavyFight_MM01 = 276,
        MusHeavyFight_MM02 = 277,
        MusCalmAtmosphere_KA02 = 278,
        MusLightFight_MM01 = 279,
        MusLightFight_MM02 = 280,
        MusSadnessOrDesperation_MM01 = 281,
        MusSadnessOrDesperation_MM02 = 282,
        MusSadnessOrDesperation_KA02 = 283,
        MusSadnessOrDesperation_KA01 = 284,
        MusStealthAction_MM01 = 285,
        MusStealthAction_MM02 = 286,
        MusStealthAction_KA01 = 287,
        MusStressOrTimeRush_MM01 = 288,
        MusStressOrTimeRush_MM02 = 289,
        MusStressOrTimeRush_KA01 = 290,
        MusStressOrTimeRush_KA02 = 291,
        MusTensionBeforeAnAction_MM01 = 292,
        MusTensionBeforeAnAction_MM02 = 293,
        MusTensionBeforeAnAction_KA01 = 294,
        MusTensionBeforeAnAction_KA02 = 295,
        MusSpecial_MM01 = 296,
        MusVictory_MM01 = 297,
        MusVictory_MM02 = 298,
        MusVictory_KA01 = 299,
        MusVictory_KA02 = 300,
        Mus_victory_KA_6 = 301,
        MusHeavyFight_KA01 = 302,
        MusHeavyFight_KA02 = 303,
        MusCalmAtmosphere_KA03 = 304,
        MusDesperateWithStress_KA02 = 305,
        MusDesperateWithStress_KA03 = 306,
        MusDesperateWithStress_KA04 = 307,
        Mus_desperatestress_KA_2_longer_2b = 308,
        MusHorror_KA01 = 309,
        MusHorror_MM01 = 310,
        MusMystery_MM01 = 311,
        MusMystery_KA02 = 312,
        MusMystery_KA01 = 313,
        MusMystery_MM02 = 314,
        MusHorror_KA02 = 315,
        MusHorror_MM02 = 316,
        MusHeavyFight_KA19 = 317,
        MusHeavyFight_KA17 = 318,
        MusHeavyFight_KA16 = 319,
        MusHeavyFight_KA06 = 320,
        MusStealthAction_KA03 = 321,
        MusSpecial_KA06 = 322,
        MusCalmAtmosphere_MM_b = 323,
        SfxHudAlarmIncoming = 324,
        SfxHudAlarmDamageA = 325,
        SfxHudAlarmDamageB = 326,
        SfxHudAlarmDamageC = 327,
        SfxHudAlarmDamageD = 328,
        SfxHudAlarmDamageE = 329,
        WepNoAmmo = 330,
        WepCannon2d = 331,
        WepCannon3d = 332,
        VehCH1EngineOff = 333,
        VehCH1EngineOn = 334,
        VehNU1EngineOff = 335,
        VehNU1EngineOn = 336,
        VehToolPressureDrillBlast2d = 337,
        VehToolPressureDrillIdle2d = 338,
        VehToolPressureDrillRecharge2d = 339,
        VehToolPressureDrillBlastRock2d = 340,
        Amb3D_PrefabFire = 341,
        SfxProgressActivation = 342,
        SfxCancelActivation = 343,
        Dlg_EACSurveySite_0015 = 344,
        Dlg_EACSurveySite_0016 = 345,
        Dlg_EACSurveySite_0017_01 = 346,
        Dlg_EACSurveySite_0017_02 = 347,
        Dlg_EACSurveySite_0018_01 = 348,
        Dlg_EACSurveySite_0018_02 = 349,
        Dlg_EACSurveySite_0018_03 = 350,
        Dlg_EACSurveySite_0019 = 351,
        Dlg_EACSurveySite_0020 = 352,
        Dlg_EACSurveySite_0021 = 353,
        Dlg_EACSurveySite_0022 = 354,
        Dlg_EACSurveySite_0023 = 355,
        Dlg_EACSurveySite_0024_01 = 356,
        Dlg_EACSurveySite_0024_02 = 357,
        Dlg_EACSurveySite_0025_01 = 358,
        Dlg_EACSurveySite_0025_02 = 359,
        Dlg_EACSurveySite_0026 = 360,
        Dlg_EACSurveySite_0027_01 = 361,
        Dlg_EACSurveySite_0027_02 = 362,
        Dlg_EACSurveySite_0028 = 363,
        Dlg_EACSurveySite_0029 = 364,
        Dlg_EACSurveySite_0030 = 365,
        Dlg_EACSurveySite_0031 = 366,
        Dlg_EACSurveySite_0032_01 = 367,
        Dlg_EACSurveySite_0032_02 = 368,
        Dlg_EACSurveySite_0033 = 369,
        Dlg_EACSurveySite_0034 = 370,
        Dlg_EACSurveySite_0035 = 371,
        Dlg_EACSurveySite_0036 = 372,
        Dlg_EACSurveySite_0037 = 373,
        Dlg_EACSurveySite_0038 = 374,
        Dlg_EACSurveySite_0039 = 375,
        Dlg_EACSurveySite_0040_01 = 376,
        Dlg_EACSurveySite_0040_02 = 377,
        Dlg_EACSurveySite_0041 = 378,
        Dlg_EACSurveySite_0042 = 379,
        Dlg_EACSurveySite_0043 = 380,
        Dlg_EACSurveySite_0044 = 381,
        Dlg_EACSurveySite_0045 = 382,
        Dlg_EACSurveySite_0046 = 383,
        Dlg_EACSurveySite_0047 = 384,
        Dlg_EACSurveySite_0048 = 385,
        Dlg_EACSurveySite_0049_01 = 386,
        Dlg_EACSurveySite_0049_02 = 387,
        Dlg_EACSurveySite_0050 = 388,
        Dlg_EACSurveySite_0051 = 389,
        Dlg_EACSurveySite_0052 = 390,
        Dlg_EACSurveySite_0053_01 = 391,
        Dlg_EACSurveySite_0053_02 = 392,
        Dlg_EACSurveySite_0053_03 = 393,
        Dlg_EACSurveySite_0054 = 394,
        Dlg_EACSurveySite_0055_01 = 395,
        Dlg_EACSurveySite_0055_02 = 396,
        Dlg_EACSurveySite_0056 = 397,
        Dlg_EACSurveySite_0057 = 398,
        Dlg_EACSurveySite_0058 = 399,
        Dlg_EACSurveySite_0059 = 400,
        Dlg_EACSurveySite_0060_01 = 401,
        Dlg_EACSurveySite_0060_02 = 402,
        Dlg_EACSurveySite_0061 = 403,
        Dlg_EACSurveySite_0062 = 404,
        Dlg_EACSurveySite_0063 = 405,
        Dlg_EACSurveySite_0064 = 406,
        Dlg_EACSurveySite_0065 = 407,
        Dlg_EACSurveySite_0066 = 408,
        Dlg_EACSurveySite_0067 = 409,
        Dlg_EACSurveySite_0068 = 410,
        Dlg_EACSurveySite_0069 = 411,
        Dlg_EACSurveySite_0070 = 412,
        Dlg_EACSurveySite_0071 = 413,
        Dlg_EACSurveySite_0072 = 414,
        Dlg_EACSurveySite_0073 = 415,
        Dlg_EACSurveySite_0074 = 416,
        Dlg_EACSurveySite_0075 = 417,
        Dlg_EACSurveySite_0076 = 418,
        Dlg_EACSurveySite_0077 = 419,
        Dlg_EACSurveySite_0078 = 420,
        Dlg_ChineseTransport_1000 = 421,
        Dlg_ChineseTransport_1001 = 422,
        Dlg_ChineseTransport_1002 = 423,
        Dlg_ChineseTransport_1003 = 424,
        Dlg_ChineseTransport_1004 = 425,
        Dlg_ChineseTransport_1005 = 426,
        Dlg_ChineseTransport_1006 = 427,
        Dlg_ChineseTransport_1007 = 428,
        Dlg_ChineseTransport_1008 = 429,
        Dlg_ChineseTransport_1009 = 430,
        Dlg_ChineseTransport_1010 = 431,
        Dlg_ChineseTransport_1011 = 432,
        Dlg_ChineseTransport_1012 = 433,
        Dlg_ChineseTransport_1013 = 434,
        Dlg_ChineseTransport_1014 = 435,
        Dlg_ChineseTransport_1015 = 436,
        Dlg_ChineseTransport_1016 = 437,
        Dlg_ChineseTransport_1017 = 438,
        Dlg_ChineseTransport_1018 = 439,
        Dlg_ChineseTransport_1019 = 440,
        Dlg_ChineseTransport_1020 = 441,
        Dlg_ChineseTransport_1021 = 442,
        Dlg_ChineseTransport_1022 = 443,
        Dlg_ChineseTransport_1023 = 444,
        Dlg_ChineseTransport_1024 = 445,
        Dlg_ChineseTransport_1025 = 446,
        Dlg_ChineseTransport_1026 = 447,
        Dlg_ChineseTransport_1027 = 448,
        Dlg_ChineseTransport_1028 = 449,
        Dlg_ChineseTransport_1029 = 450,
        Dlg_ChineseTransport_1030 = 451,
        Dlg_ChineseTransport_1031 = 452,
        Dlg_ChineseTransport_1032 = 453,
        Dlg_ChineseTransport_1033 = 454,
        Dlg_ChineseTransport_1034 = 455,
        Dlg_ChineseTransport_1035 = 456,
        Dlg_ChineseTransport_1036 = 457,
        Dlg_ChineseTransport_1037 = 458,
        Dlg_ChineseTransport_1038 = 459,
        Dlg_ChineseTransport_1039 = 460,
        Dlg_ChineseTransport_1040 = 461,
        Dlg_ChineseTransport_1041 = 462,
        Dlg_ChineseTransport_1042 = 463,
        Dlg_ChineseTransport_1043 = 464,
        Dlg_ChineseTransport_1044 = 465,
        Dlg_ChineseTransport_1045 = 466,
        Dlg_ChineseRafinery_1000 = 467,
        Dlg_ChineseRafinery_1001 = 468,
        Dlg_ChineseRafinery_1002 = 469,
        Dlg_ChineseRafinery_1003 = 470,
        Dlg_ChineseRafinery_1004 = 471,
        Dlg_ChineseRafinery_1005 = 472,
        Dlg_ChineseRafinery_1006 = 473,
        Dlg_ChineseRafinery_1007 = 474,
        Dlg_ChineseRafinery_1008 = 475,
        Dlg_ChineseRafinery_1009 = 476,
        Dlg_ChineseRafinery_1010 = 477,
        Dlg_ChineseRafinery_1011 = 478,
        Dlg_ChineseRafinery_1012 = 479,
        Dlg_ChineseRafinery_1013 = 480,
        Dlg_ChineseRafinery_1014 = 481,
        Dlg_ChineseRafinery_1015 = 482,
        Dlg_ChineseRafinery_1016 = 483,
        Dlg_ChineseRafinery_1017 = 484,
        Dlg_ChineseRafinery_1018 = 485,
        Dlg_ChineseRafinery_1019 = 486,
        Dlg_ChineseRafinery_1020 = 487,
        Dlg_ChineseRafinery_1021 = 488,
        Dlg_ChineseRafinery_1022 = 489,
        Dlg_ChineseRafinery_1023 = 490,
        Dlg_ChineseRafinery_1024 = 491,
        Dlg_ChineseRafinery_1025 = 492,
        Dlg_ChineseRafinery_1026 = 493,
        Dlg_ChineseRafinery_1027 = 494,
        Dlg_ChineseRafinery_1028 = 495,
        Dlg_ChineseRafinery_1029 = 496,
        Dlg_ChineseRafinery_1030 = 497,
        Dlg_ChineseRafinery_1031 = 498,
        Dlg_ChineseRafinery_1032 = 499,
        Dlg_ChineseRafinery_1033 = 500,
        Dlg_ChineseRafinery_1034 = 501,
        Dlg_ChineseRafinery_1035 = 502,
        Dlg_ChineseRafinery_1036 = 503,
        Dlg_ChineseRafinery_1037 = 504,
        Dlg_ChineseRafinery_1038 = 505,
        Dlg_ChineseRafinery_1039 = 506,
        Amb2D_City = 507,
        Amb2D_ComputerRoom = 508,
        Amb2D_Factory = 509,
        Amb2D_LostPlace = 510,
        Amb2D_RedHeat = 511,
        Amb2D_War = 512,
        Dlg_ChineseTransmitter_1004 = 513,
        Dlg_ChineseTransmitter_1005 = 514,
        Dlg_ChineseTransmitter_1006 = 515,
        Dlg_ChineseTransmitter_1007 = 516,
        Dlg_ChineseTransmitter_1008 = 517,
        Dlg_ChineseTransmitter_1009 = 518,
        Dlg_ChineseTransmitter_1010 = 519,
        Dlg_ChineseTransmitter_1011 = 520,
        Dlg_ChineseTransmitter_1012 = 521,
        Dlg_ChineseTransmitter_1013 = 522,
        Dlg_ChineseTransmitter_1014 = 523,
        Dlg_ChineseTransmitter_1015 = 524,
        Dlg_ChineseTransmitter_1016 = 525,
        Dlg_ChineseTransmitter_1017 = 526,
        Dlg_ChineseTransmitter_1018 = 527,
        Dlg_ChineseTransmitter_1019 = 528,
        Dlg_ChineseTransmitter_1020 = 529,
        Dlg_ChineseTransmitter_1021 = 530,
        Dlg_ChineseTransmitter_1022 = 531,
        Dlg_ChineseTransmitter_1023 = 532,
        Dlg_ChineseTransmitter_1024 = 533,
        Dlg_ChineseTransmitter_1025 = 534,
        Dlg_ChineseTransmitter_1026 = 535,
        Dlg_ChineseTransmitter_1027 = 536,
        Dlg_ChineseTransmitter_1028 = 537,
        Dlg_ChineseTransmitter_1029 = 538,
        Dlg_ChineseTransmitter_1030 = 539,
        Dlg_ChineseTransmitter_1031 = 540,
        Dlg_ChineseTransmitter_1032 = 541,
        Dlg_ChineseTransmitter_1033 = 542,
        Dlg_ChineseTransmitter_1034 = 543,
        Dlg_ChineseTransmitter_1035 = 544,
        Dlg_ChineseTransmitter_1036 = 545,
        Dlg_ChineseTransmitter_1037 = 546,
        Dlg_ChineseTransmitter_1038 = 547,
        Dlg_ChineseTransmitter_1039 = 548,
        Dlg_ChineseTransmitter_1040 = 549,
        Dlg_ChineseTransmitter_1041 = 550,
        Dlg_ChineseTransmitter_1042 = 551,
        Dlg_ChineseTransmitter_1043 = 552,
        Dlg_ChineseTransmitter_1044 = 553,
        Amb2D_City2 = 554,
        Amb2D_Factory2 = 555,
        Dlg_ChineseTransmitter_1047 = 556,
        Dlg_ChineseTransmitter_1048 = 557,
        Dlg_ChineseTransmitter_1049 = 558,
        Dlg_ChineseTransmitter_1050 = 559,
        Dlg_ChineseTransmitter_1051 = 560,
        Dlg_ChineseTransmitter_1052 = 561,
        Dlg_ChineseTransmitter_1053 = 562,
        Dlg_ChineseTransmitter_1054 = 563,
        Dlg_ChineseTransmitter_1055 = 564,
        Dlg_ChineseTransmitter_1056 = 565,
        Dlg_ChineseTransmitter_1057 = 566,
        Dlg_ChineseTransmitter_1058 = 567,
        Dlg_ChineseTransmitter_1059 = 568,
        Dlg_ChineseTransmitter_1060 = 569,
        Dlg_ChineseTransmitter_1061 = 570,
        Dlg_ChineseTransmitter_1062 = 571,
        Dlg_ChineseTransmitter_1063 = 572,
        Dlg_ChineseTransmitter_1064 = 573,
        Dlg_ChineseTransmitter_1065 = 574,
        Dlg_ChineseTransmitter_1066 = 575,
        Dlg_ChineseTransmitter_1067 = 576,
        Dlg_ChineseTransmitter_1068 = 577,
        Dlg_ChineseTransmitter_1069 = 578,
        Dlg_SlaverBaseWipeout_1000 = 579,
        Dlg_SlaverBaseWipeout_1001 = 580,
        Dlg_SlaverBaseWipeout_1002 = 581,
        Dlg_SlaverBaseWipeout_1003 = 582,
        Dlg_SlaverBaseWipeout_1004 = 583,
        Dlg_SlaverBaseWipeout_1005 = 584,
        Dlg_SlaverBaseWipeout_1006 = 585,
        Dlg_SlaverBaseWipeout_1007 = 586,
        Dlg_SlaverBaseWipeout_1008 = 587,
        Dlg_SlaverBaseWipeout_1009 = 588,
        Dlg_SlaverBaseWipeout_1010 = 589,
        Dlg_SlaverBaseWipeout_1011 = 590,
        Dlg_SlaverBaseWipeout_1012 = 591,
        Dlg_SlaverBaseWipeout_1013 = 592,
        Dlg_SlaverBaseWipeout_1014 = 593,
        Dlg_SlaverBaseWipeout_1015 = 594,
        Dlg_SlaverBaseWipeout_1016 = 595,
        Dlg_SlaverBaseWipeout_1017 = 596,
        Dlg_SlaverBaseWipeout_1018 = 597,
        Dlg_SlaverBaseWipeout_1019 = 598,
        Dlg_SlaverBaseWipeout_1020 = 599,
        Dlg_SlaverBaseWipeout_1021 = 600,
        Dlg_SlaverBaseWipeout_1022 = 601,
        Dlg_SlaverBaseWipeout_1023 = 602,
        Dlg_SlaverBaseWipeout_1024 = 603,
        Dlg_SlaverBaseWipeout_1025 = 604,
        Dlg_SlaverBaseWipeout_1026 = 605,
        Dlg_SlaverBaseWipeout_1027 = 606,
        Dlg_SlaverBaseWipeout_1028 = 607,
        Dlg_SlaverBaseWipeout_1029 = 608,
        Dlg_SlaverBaseWipeout_1030 = 609,
        Dlg_SlaverBaseWipeout_1031 = 610,
        Dlg_SlaverBaseWipeout_1032 = 611,
        Dlg_SlaverBaseWipeout_1033 = 612,
        Dlg_SlaverBaseWipeout_1034 = 613,
        Dlg_SlaverBaseWipeout_1035 = 614,
        Dlg_SlaverBaseWipeout_1036 = 615,
        Dlg_SlaverBaseWipeout_1037 = 616,
        Dlg_SlaverBaseWipeout_1038 = 617,
        Dlg_SlaverBaseWipeout_1039 = 618,
        Dlg_SlaverBaseWipeout_1040 = 619,
        Dlg_SlaverBaseWipeout_1041 = 620,
        Dlg_SlaverBaseWipeout_1042 = 621,
        Dlg_SlaverBaseWipeout_1043 = 622,
        Dlg_SlaverBaseWipeout_1044 = 623,
        Dlg_SlaverBaseWipeout_1045 = 624,
        Dlg_SlaverBaseWipeout_1046 = 625,
        Dlg_SlaverBaseWipeout_1047 = 626,
        Dlg_SlaverBaseWipeout_1048 = 627,
        Dlg_SlaverBaseWipeout_1049 = 628,
        Dlg_SlaverBaseWipeout_1050 = 629,
        Dlg_SlaverBaseWipeout_1051 = 630,
        Dlg_SlaverBaseWipeout_1052 = 631,
        Dlg_SlaverBase2_1000 = 632,
        Dlg_SlaverBase2_1001 = 633,
        Dlg_SlaverBase2_1002 = 634,
        Dlg_SlaverBase2_1003 = 635,
        Dlg_SlaverBase2_1004 = 636,
        Dlg_SlaverBase2_1005 = 637,
        Dlg_SlaverBase2_1006 = 638,
        Dlg_SlaverBase2_1007 = 639,
        Dlg_SlaverBase2_1008 = 640,
        Dlg_SlaverBase2_1009 = 641,
        Dlg_SlaverBase2_1010 = 642,
        Dlg_SlaverBase2_1011 = 643,
        Dlg_SlaverBase2_1012 = 644,
        Dlg_SlaverBase2_1013 = 645,
        Dlg_SlaverBase2_1014 = 646,
        Dlg_SlaverBase2_1015 = 647,
        Dlg_SlaverBase2_1016 = 648,
        Dlg_SlaverBase2_1017 = 649,
        Dlg_SlaverBase2_1018 = 650,
        Dlg_SlaverBase2_1019 = 651,
        Dlg_SlaverBase2_1020 = 652,
        Dlg_SlaverBase2_1021 = 653,
        Dlg_SlaverBase2_1022 = 654,
        Dlg_SlaverBase2_1023 = 655,
        Dlg_SlaverBase2_1024 = 656,
        Dlg_SlaverBase2_1025 = 657,
        Dlg_SlaverBase2_1026 = 658,
        Dlg_SlaverBase2_1027 = 659,
        Dlg_SlaverBase2_1028 = 660,
        Dlg_SlaverBase2_1029 = 661,
        Dlg_SlaverBase2_1030 = 662,
        Dlg_SlaverBase2_1031 = 663,
        Dlg_SlaverBase2_1032 = 664,
        Dlg_SlaverBase2_1033 = 665,
        Dlg_SlaverBase2_1034 = 666,
        Dlg_SlaverBase2_1035 = 667,
        Dlg_SlaverBase2_1036 = 668,
        Dlg_SlaverBase2_1037 = 669,
        Dlg_SlaverBase2_1038 = 670,
        Dlg_SlaverBase2_1039 = 671,
        Dlg_SlaverBase2_1040 = 672,
        Dlg_SlaverBase2_1041 = 673,
        Dlg_SlaverBase2_1042 = 674,
        Dlg_SlaverBase2_1043 = 675,
        Dlg_SlaverBase2_1044 = 676,
        Dlg_TwinTowers_1000 = 677,
        Dlg_TwinTowers_1001 = 678,
        Dlg_TwinTowers_1002 = 679,
        Dlg_TwinTowers_1003 = 680,
        Dlg_TwinTowers_1004 = 681,
        Dlg_TwinTowers_1005 = 682,
        Dlg_TwinTowers_1006 = 683,
        Dlg_TwinTowers_1007 = 684,
        Dlg_TwinTowers_1008 = 685,
        Dlg_TwinTowers_1009 = 686,
        Dlg_TwinTowers_1010 = 687,
        Dlg_TwinTowers_1011 = 688,
        Dlg_TwinTowers_1012 = 689,
        Dlg_TwinTowers_1013 = 690,
        Dlg_TwinTowers_1014 = 691,
        Dlg_TwinTowers_1015 = 692,
        Dlg_TwinTowers_1016 = 693,
        Dlg_TwinTowers_1017 = 694,
        Dlg_TwinTowers_1018 = 695,
        Dlg_TwinTowers_1019 = 696,
        Dlg_TwinTowers_1020 = 697,
        Dlg_TwinTowers_1021 = 698,
        Dlg_TwinTowers_1022 = 699,
        Dlg_TwinTowers_1023 = 700,
        Dlg_TwinTowers_1024 = 701,
        Dlg_TwinTowers_1025 = 702,
        Dlg_TwinTowers_1026 = 703,
        Dlg_TwinTowers_1027 = 704,
        Dlg_TwinTowers_1028 = 705,
        Dlg_TwinTowers_1029 = 706,
        Dlg_TwinTowers_1030 = 707,
        Dlg_TwinTowers_1031 = 708,
        Dlg_TwinTowers_1032 = 709,
        Dlg_TwinTowers_1033 = 710,
        Dlg_TwinTowers_1034 = 711,
        Dlg_TwinTowers_1035 = 712,
        Dlg_TwinTowers_1036 = 713,
        Dlg_TwinTowers_1037 = 714,
        Dlg_TwinTowers_1038 = 715,
        Dlg_TwinTowers_1039 = 716,
        Dlg_TwinTowers_1040 = 717,
        Dlg_TwinTowers_1041 = 718,
        Dlg_TwinTowers_1042 = 719,
        Dlg_TwinTowers_1043 = 720,
        Dlg_TwinTowers_1044 = 721,
        Dlg_TwinTowers_1045 = 722,
        Dlg_TwinTowers_1046 = 723,
        Dlg_TwinTowers_1047 = 724,
        Dlg_TwinTowers_1048 = 725,
        Dlg_TwinTowers_1049 = 726,
        Dlg_TwinTowers_1050 = 727,
        Dlg_TwinTowers_1051 = 728,
        Dlg_TwinTowers_1052 = 729,
        Dlg_TwinTowers_1053 = 730,
        Dlg_TwinTowers_1054 = 731,
        Dlg_TwinTowers_1055 = 732,
        Dlg_TwinTowers_1056 = 733,
        Dlg_TwinTowers_1057 = 734,
        Dlg_TwinTowers_1058 = 735,
        Dlg_TwinTowers_1059 = 736,
        Dlg_TwinTowers_1060 = 737,
        Dlg_TwinTowers_1061 = 738,
        Dlg_TwinTowers_1062 = 739,
        Dlg_TwinTowers_1063 = 740,
        Dlg_TwinTowers_1064 = 741,
        Dlg_TwinTowers_1065 = 742,
        Dlg_TwinTowers_1066 = 743,
        Dlg_TwinTowers_1067 = 744,
        Dlg_TwinTowers_1068 = 745,
        Dlg_TwinTowers_1069 = 746,
        Dlg_TwinTowers_1070 = 747,
        Dlg_TwinTowers_1071 = 748,
        Dlg_TwinTowers_1072 = 749,
        Dlg_TwinTowers_1073 = 750,
        Dlg_TwinTowers_1074 = 751,
        Dlg_TwinTowers_1075 = 752,
        Dlg_TwinTowers_1076 = 753,
        Dlg_TwinTowers_1077 = 754,
        Dlg_TwinTowers_1078 = 755,
        Dlg_TwinTowers_1079 = 756,
        Dlg_TwinTowers_1080 = 757,
        Dlg_TwinTowers_1081 = 758,
        Dlg_TwinTowers_1082 = 759,
        Dlg_TwinTowers_1083 = 760,
        Dlg_TwinTowers_1084 = 761,
        Dlg_TwinTowers_1085 = 762,
        Dlg_TwinTowers_1086 = 763,
        Dlg_TwinTowers_1087 = 764,
        Dlg_TwinTowers_1088 = 765,
        Dlg_TwinTowers_1089 = 766,
        Dlg_TwinTowers_1090 = 767,
        Dlg_TwinTowers_1091 = 768,
        Dlg_TwinTowers_1092 = 769,
        Dlg_TwinTowers_1093 = 770,
        Dlg_TwinTowers_1094 = 771,
        Dlg_TwinTowers_1095 = 772,
        Dlg_TwinTowers_1096 = 773,
        Dlg_TwinTowers_1097 = 774,
        Dlg_TwinTowers_1098 = 775,
        Dlg_TwinTowers_1099 = 776,
        Dlg_TwinTowers_1100 = 777,
        Dlg_TwinTowers_1101 = 778,
        Dlg_TwinTowers_1102 = 779,
        Dlg_TwinTowers_1103 = 780,
        Dlg_TwinTowers_1104 = 781,
        Dlg_TwinTowers_1105 = 782,
        Dlg_TwinTowers_1106 = 783,
        Dlg_TwinTowers_1107 = 784,
        Dlg_TwinTowers_1108 = 785,
        Dlg_TwinTowers_1109 = 786,
        Dlg_TwinTowers_1110 = 787,
        Dlg_TwinTowers_1111 = 788,
        Dlg_TwinTowers_1112 = 789,
        Dlg_TwinTowers_1113 = 790,
        Dlg_TwinTowers_1114 = 791,
        Dlg_TwinTowers_1115 = 792,
        Dlg_TwinTowers_1116 = 793,
        Dlg_TwinTowers_1117 = 794,
        Dlg_TwinTowers_1118 = 795,
        Dlg_TwinTowers_1119 = 796,
        Dlg_TwinTowers_1120 = 797,
        Dlg_TwinTowers_1121 = 798,
        Dlg_TwinTowers_1122 = 799,
        Dlg_TwinTowers_1123 = 800,
        Dlg_TwinTowers_1124 = 801,
        Dlg_TwinTowers_1125 = 802,
        Dlg_TwinTowers_1126 = 803,
        Dlg_TwinTowers_1127 = 804,
        Dlg_TwinTowers_1128 = 805,
        Dlg_TwinTowers_1129 = 806,
        Dlg_TwinTowers_1130 = 807,
        Dlg_TwinTowers_1131 = 808,
        Dlg_TwinTowers_1132 = 809,
        Dlg_WhiteWolvesBioResearch_1000 = 810,
        Dlg_WhiteWolvesBioResearch_1001 = 811,
        Dlg_WhiteWolvesBioResearch_1002 = 812,
        Dlg_WhiteWolvesBioResearch_1003 = 813,
        Dlg_WhiteWolvesBioResearch_1004 = 814,
        Dlg_WhiteWolvesBioResearch_1005 = 815,
        Dlg_WhiteWolvesBioResearch_1006 = 816,
        Dlg_WhiteWolvesBioResearch_1007 = 817,
        Dlg_WhiteWolvesBioResearch_1008 = 818,
        Dlg_WhiteWolvesBioResearch_1009 = 819,
        Dlg_WhiteWolvesBioResearch_1010 = 820,
        Dlg_WhiteWolvesBioResearch_1011 = 821,
        Dlg_WhiteWolvesBioResearch_1012 = 822,
        Dlg_WhiteWolvesBioResearch_1013 = 823,
        Dlg_WhiteWolvesBioResearch_1014 = 824,
        Dlg_WhiteWolvesBioResearch_1015 = 825,
        Dlg_WhiteWolvesBioResearch_1016 = 826,
        Dlg_WhiteWolvesBioResearch_1017 = 827,
        Dlg_WhiteWolvesBioResearch_1018 = 828,
        Dlg_WhiteWolvesBioResearch_1019 = 829,
        Dlg_WhiteWolvesBioResearch_1020 = 830,
        Dlg_WhiteWolvesBioResearch_1021 = 831,
        Dlg_WhiteWolvesBioResearch_1022 = 832,
        Dlg_WhiteWolvesBioResearch_1023 = 833,
        Dlg_WhiteWolvesBioResearch_1024 = 834,
        Dlg_WhiteWolvesBioResearch_1025 = 835,
        Dlg_WhiteWolvesBioResearch_1026 = 836,
        Dlg_WhiteWolvesBioResearch_1027 = 837,
        Dlg_WhiteWolvesBioResearch_1028 = 838,
        Dlg_WhiteWolvesBioResearch_1029 = 839,
        Dlg_WhiteWolvesBioResearch_1030 = 840,
        Dlg_WhiteWolvesBioResearch_1031 = 841,
        Dlg_WhiteWolvesBioResearch_1032 = 842,
        Dlg_WhiteWolvesBioResearch_1033 = 843,
        Dlg_WhiteWolvesBioResearch_1034 = 844,
        Dlg_WhiteWolvesBioResearch_1035 = 845,
        Dlg_WhiteWolvesBioResearch_1036 = 846,
        Dlg_WhiteWolvesBioResearch_1037 = 847,
        Dlg_WhiteWolvesBioResearch_1038 = 848,
        Dlg_WhiteWolvesBioResearch_1039 = 849,
        Dlg_WhiteWolvesBioResearch_1040 = 850,
        Dlg_WhiteWolvesBioResearch_1041 = 851,
        Dlg_WhiteWolvesBioResearch_1042 = 852,
        Dlg_WhiteWolvesBioResearch_1043 = 853,
        Dlg_WhiteWolvesBioResearch_1044 = 854,
        Dlg_RimeConvince_1000 = 855,
        Dlg_RimeConvince_1001 = 856,
        Dlg_RimeConvince_1002 = 857,
        Dlg_RimeConvince_1003 = 858,
        Dlg_RimeConvince_1004 = 859,
        Dlg_RimeConvince_1005 = 860,
        Dlg_RimeConvince_1006 = 861,
        Dlg_RimeConvince_1007 = 862,
        Dlg_RimeConvince_1008 = 863,
        Dlg_RimeConvince_1009 = 864,
        Dlg_RimeConvince_1010 = 865,
        Dlg_RimeConvince_1011 = 866,
        Dlg_RimeConvince_1012 = 867,
        Dlg_RimeConvince_1013 = 868,
        Dlg_RimeConvince_1014 = 869,
        Dlg_RimeConvince_1015 = 870,
        Dlg_RimeConvince_1016 = 871,
        Dlg_RimeConvince_1017 = 872,
        Dlg_RimeConvince_1018 = 873,
        Dlg_RimeConvince_1019 = 874,
        Dlg_RimeConvince_1020 = 875,
        Dlg_RimeConvince_1021 = 876,
        Dlg_RimeConvince_1022 = 877,
        Dlg_RimeConvince_1023 = 878,
        Dlg_RimeConvince_1024 = 879,
        Dlg_RimeConvince_1025 = 880,
        Dlg_RimeConvince_1026 = 881,
        Dlg_RimeConvince_1027 = 882,
        Dlg_RimeConvince_1028 = 883,
        Dlg_RimeConvince_1029 = 884,
        Dlg_RimeConvince_1030 = 885,
        Dlg_RimeConvince_1031 = 886,
        Dlg_RimeConvince_1032 = 887,
        Dlg_RimeConvince_1033 = 888,
        Dlg_RimeConvince_1034 = 889,
        Dlg_RimeConvince_1035 = 890,
        Dlg_RimeConvince_1036 = 891,
        Dlg_RimeConvince_1037 = 892,
        Dlg_RimeConvince_1038 = 893,
        Dlg_RimeConvince_1039 = 894,
        Dlg_RimeConvince_1040 = 895,
        Dlg_RimeConvince_1041 = 896,
        Dlg_RimeConvince_1042 = 897,
        Dlg_RimeConvince_1043 = 898,
        Dlg_RimeConvince_1044 = 899,
        Dlg_RimeConvince_1045 = 900,
        Dlg_RimeConvince_1046 = 901,
        Dlg_RimeConvince_1047 = 902,
        Dlg_RimeConvince_1048 = 903,
        Dlg_RimeConvince_1049 = 904,
        Dlg_RimeConvince_1050 = 905,
        Dlg_RimeConvince_1051 = 906,
        Dlg_RimeConvince_1052 = 907,
        Dlg_RimeConvince_1053 = 908,
        Dlg_RimeConvince_1054 = 909,
        Dlg_RimeConvince_1055 = 910,
        Dlg_RimeConvince_1056 = 911,
        Dlg_RimeConvince_1057 = 912,
        Dlg_RimeConvince_1058 = 913,
        Dlg_RimeConvince_1059 = 914,
        Dlg_RimeConvince_1060 = 915,
        Dlg_RimeConvince_1061 = 916,
        Dlg_RimeConvince_1062 = 917,
        Dlg_RimeConvince_1063 = 918,
        Dlg_RimeConvince_1064 = 919,
        Dlg_RimeConvince_1065 = 920,
        Dlg_RimeConvince_1066 = 921,
        Dlg_RimeConvince_1067 = 922,
        Dlg_RimeConvince_1068 = 923,
        Dlg_RimeConvince_1069 = 924,
        Dlg_RimeConvince_1070 = 925,
        Dlg_RimeConvince_1071 = 926,
        Dlg_RimeConvince_1072 = 927,
        Dlg_RimeConvince_1073 = 928,
        Dlg_RimeConvince_1074 = 929,
        Dlg_RimeConvince_1075 = 930,
        Dlg_RimeConvince_1076 = 931,
        Dlg_RimeConvince_1077 = 932,
        Dlg_RimeConvince_1078 = 933,
        Dlg_RimeConvince_1079 = 934,
        Dlg_RimeConvince_1080 = 935,
        Dlg_RimeConvince_1081 = 936,
        Dlg_RimeConvince_1082 = 937,
        Dlg_RimeConvince_1083 = 938,
        Dlg_RimeConvince_1084 = 939,
        Dlg_RimeConvince_1085 = 940,
        Dlg_RimeConvince_1086 = 941,
        Dlg_RimeConvince_1087 = 942,
        Dlg_RimeConvince_1088 = 943,
        Dlg_RimeConvince_1089 = 944,
        Dlg_RimeConvince_1090 = 945,
        Dlg_RimeConvince_1091 = 946,
        Dlg_RimeConvince_1092 = 947,
        Dlg_RimeConvince_1093 = 948,
        Dlg_RimeConvince_1094 = 949,
        Dlg_RimeConvince_1095 = 950,
        Dlg_RimeConvince_1096 = 951,
        Dlg_RimeConvince_1097 = 952,
        Dlg_RimeConvince_1098 = 953,
        Dlg_RimeConvince_1099 = 954,
        Dlg_RimeConvince_1100 = 955,
        Dlg_RimeConvince_1101 = 956,
        Dlg_RimeConvince_1102 = 957,
        Dlg_RimeConvince_1103 = 958,
        Dlg_RimeConvince_1104 = 959,
        Dlg_RimeConvince_1105 = 960,
        Dlg_RimeConvince_1106 = 961,
        Dlg_RimeConvince_1107 = 962,
        Dlg_RimeConvince_1108 = 963,
        Dlg_RimeConvince_1109 = 964,
        Dlg_RimeConvince_1110 = 965,
        Dlg_RimeConvince_1111 = 966,
        Dlg_RimeConvince_1112 = 967,
        Dlg_RimeConvince_1113 = 968,
        Dlg_RimeConvince_1114 = 969,
        Dlg_RimeConvince_1115 = 970,
        Dlg_RimeConvince_1116 = 971,
        Dlg_RimeConvince_1117 = 972,
        Dlg_RimeConvince_1118 = 973,
        Dlg_RimeConvince_1119 = 974,
        Dlg_RimeConvince_1120 = 975,
        Dlg_RimeConvince_1121 = 976,
        Dlg_RimeConvince_1122 = 977,
        Dlg_RimeConvince_1123 = 978,
        Dlg_RimeConvince_1124 = 979,
        Dlg_RimeConvince_1125 = 980,
        Dlg_RimeConvince_1126 = 981,
        Dlg_RimeConvince_1127 = 982,
        Dlg_RimeConvince_1128 = 983,
        Dlg_RimeConvince_1129 = 984,
        Dlg_RimeConvince_1130 = 985,
        Dlg_RimeConvince_1131 = 986,
        Dlg_RimeConvince_1132 = 987,
        Dlg_RimeConvince_1133 = 988,
        Dlg_RimeConvince_1134 = 989,
        Dlg_RimeConvince_1135 = 990,
        Dlg_RimeConvince_1136 = 991,
        Dlg_RimeConvince_1137 = 992,
        Dlg_RimeConvince_1138 = 993,
        Dlg_RimeConvince_1139 = 994,
        Dlg_RimeConvince_1140 = 995,
        Dlg_RimeConvince_1141 = 996,
        Dlg_RimeConvince_1142 = 997,
        Dlg_RimeConvince_1143 = 998,
        Dlg_RimeConvince_1144 = 999,
        Dlg_RimeConvince_1145 = 1000,
        Dlg_RimeConvince_1146 = 1001,
        Dlg_RimeConvince_1147 = 1002,
        Dlg_RimeConvince_1148 = 1003,
        Dlg_Rift_1000 = 1004,
        Dlg_Rift_1001 = 1005,
        Dlg_Rift_1002 = 1006,
        Dlg_Rift_1003 = 1007,
        Dlg_Rift_1004 = 1008,
        Dlg_Rift_1005 = 1009,
        Dlg_Rift_1006 = 1010,
        Dlg_Rift_1007 = 1011,
        Dlg_Rift_1008 = 1012,
        Dlg_Rift_1009 = 1013,
        Dlg_Rift_1010 = 1014,
        Dlg_Rift_1011 = 1015,
        Dlg_Rift_1012 = 1016,
        Dlg_Rift_1013 = 1017,
        Dlg_Rift_1014 = 1018,
        Dlg_Rift_1015 = 1019,
        Dlg_Rift_1016 = 1020,
        Dlg_Rift_1017 = 1021,
        Dlg_Rift_1018 = 1022,
        Dlg_Rift_1019 = 1023,
        Dlg_Rift_1020 = 1024,
        Dlg_Rift_1021 = 1025,
        Dlg_Rift_1022 = 1026,
        Dlg_Rift_1023 = 1027,
        Dlg_Rift_1024 = 1028,
        Dlg_Rift_1025 = 1029,
        Dlg_Rift_1026 = 1030,
        Dlg_Rift_1027 = 1031,
        Dlg_Rift_1028 = 1032,
        Dlg_Rift_1029 = 1033,
        Dlg_Rift_1030 = 1034,
        Dlg_Rift_1031 = 1035,
        Dlg_Rift_1032 = 1036,
        Dlg_Rift_1033 = 1037,
        Dlg_Rift_1034 = 1038,
        Dlg_Rift_1035 = 1039,
        Dlg_Rift_1036 = 1040,
        Dlg_Rift_1037 = 1041,
        Dlg_Rift_1038 = 1042,
        Dlg_Rift_1039 = 1043,
        Dlg_Rift_1040 = 1044,
        Dlg_Rift_1041 = 1045,
        Dlg_Rift_1042 = 1046,
        Dlg_Rift_1043 = 1047,
        Dlg_Rift_1044 = 1048,
        Dlg_Rift_1045 = 1049,
        Dlg_Rift_1046 = 1050,
        Dlg_Rift_1047 = 1051,
        Dlg_Rift_1048 = 1052,
        Dlg_Rift_1049 = 1053,
        Dlg_Rift_1050 = 1054,
        Dlg_Rift_1051 = 1055,
        Dlg_Rift_1052 = 1056,
        Dlg_Rift_1053 = 1057,
        Dlg_Rift_1054 = 1058,
        Dlg_Rift_1055 = 1059,
        Dlg_Rift_1056 = 1060,
        Dlg_Rift_1057 = 1061,
        Dlg_Rift_1058 = 1062,
        Dlg_Rift_1059 = 1063,
        Dlg_Rift_1060 = 1064,
        SfxAcquireWeaponOn = 1065,
        SfxAcquireWeaponOff = 1066,
        SfxAcquireDroneOn = 1067,
        SfxAcquireDroneOff = 1068,
        SfxAcquireCameraOn = 1069,
        SfxAcquireCameraOff = 1070,
        SfxTakeAllUniversal = 1071,
        SfxTakeAllAmmo = 1072,
        SfxTakeAllFuel = 1073,
        SfxTakeAllMedkit = 1074,
        SfxTakeAllOxygen = 1075,
        SfxTakeAllRepair = 1076,
        MusLightFight_KA03 = 1077,
        MusLightFight_KA04 = 1078,
        MusLightFight_KA05 = 1079,
        MusLightFight_KA07 = 1080,
        MusLightFight_KA08 = 1081,
        MusLightFight_KA10 = 1082,
        MusLightFight_KA11 = 1083,
        MusSpecial_MM02 = 1084,
        MusSpecial_MM03 = 1085,
        MusSpecial_KA01 = 1086,
        MusHeavyFight_KA03 = 1087,
        SfxProgressTake = 1088,
        SfxCancelTake = 1089,
        SfxProgressPut = 1090,
        SfxCancelPut = 1091,
        HudAmmoNoWarning = 1092,
        HudArmorNoWarning = 1093,
        VehToolCrusherDrillCollidingOtherRelease2d = 1094,
        VehToolCrusherDrillCollidingOtherRelease3d = 1095,
        VehToolLaserCollidingOther2d = 1096,
        VehToolLaserCollidingOther3d = 1097,
        VehToolLaserCollidingOtherRelease2d = 1098,
        VehToolLaserCollidingOtherRelease3d = 1099,
        VehToolPressureDrillBlastOther2d = 1100,
        VehToolPressureDrillBlastOther3d = 1101,
        VehToolSawCutOther2d = 1102,
        VehToolSawCutOther3d = 1103,
        VehToolSawCutOtherRelease2d = 1104,
        VehToolSawCutOtherRelease3d = 1105,
        VehToolSawCutRelease2d = 1106,
        VehToolSawCutRelease3d = 1107,
        VehToolThermalDrillCollidingOther2d = 1108,
        VehToolThermalDrillCollidingOther3d = 1109,
        VehToolThermalDrillCollidingOtherRelease2d = 1110,
        VehToolThermalDrillCollidingOtherRelease3d = 1111,
        VehToolCrusherDrillCollidingOther2d = 1112,
        VehToolCrusherDrillCollidingOther3d = 1113,
        SfxFlareLoop01 = 1114,
        SfxFlareDeploy = 1115,
        Amb3D_EngineThrust = 1116,
        VehLoopDrone = 1117,
        VehLoopCamera = 1118,
        WepEpmExplosion = 1119,
        HudOxygenLeakingWarning = 1120,
        VehLoopLargeShip = 1121,
        MusLightFight_KA12 = 1122,
        MusHeavyFight_KA05 = 1123,
        MusSpecial_KA02 = 1124,
        MusHeavyFight_KA07 = 1125,
        MusSpecial_MM04 = 1126,
        MusHeavyFight_KA04 = 1127,
        MusSpecial_KA03 = 1128,
        SfxMeteorFly = 1129,
        SfxMeteorExplosion = 1130,
        WepLargeShipAutocannonRotateRelease = 1131,
        SfxNuclearExplosion = 1132,
        MusLightFight_KA27 = 1133,
        MusSpecial_KA04 = 1134,
        MusSpecial_KA05 = 1135,
        MusSpecial_MM05 = 1136,
        Mus_lightfight_KA_26 = 1137,
        Mus_lightfight_KA_28 = 1138,
        Mus_special_KA_4_loop = 1139,
        VehToolNuclearDrillLoop3d = 1140,
        VehToolNuclearDrillLoop2d = 1141,
        VehToolNuclearDrillRelease3d = 1142,
        VehToolNuclearDrillRelease2d = 1143,
        VehToolNuclearDrillColliding3d = 1144,
        VehToolNuclearDrillColliding2d = 1145,
        VehToolNuclearDrillCollidingOther3d = 1146,
        VehToolNuclearDrillCollidingOther2d = 1147,
        VehToolNuclearDrillCollidingRelease3d = 1148,
        VehToolNuclearDrillCollidingRelease2d = 1149,
        VehToolNuclearDrillCollidingOtherRelease3d = 1150,
        VehToolNuclearDrillCollidingOtherRelease2d = 1151,
        VehToolNuclearDrillIdle3d = 1152,
        VehToolNuclearDrillIdle2d = 1153,
        VehToolPressureDrillBlast3d = 1154,
        VehToolPressureDrillBlastRock3d = 1155,
        VehToolPressureDrillIdle3d = 1156,
        VehToolPressureDrillRecharge3d = 1157,
        SfxPlayerDeathBeep = 1158,
        SfxPlayerDeathBreath = 1159,
        Dlg_EACSurveySite_1000 = 1160,
        Dlg_EACSurveySite_1001 = 1161,
        Dlg_EACSurveySite_1002 = 1162,
        Dlg_EACSurveySite_1003 = 1163,
        Dlg_EACSurveySite_1004 = 1164,
        Sht_Reich_JE_ForTheFuture = 1165,
        Sht_Reich_JE_Vanquish = 1166,
        Sht_Reich_JE_WeControl = 1167,
        Sht_Reich_JE_YouAreTheWeakLink = 1168,
        Sht_Reich_JE_YourFinalDay = 1169,
        Sht_Russian_JE_DasVidanya = 1170,
        Sht_Russian_JE_ForKremlin = 1171,
        Sht_Russian_JE_ForMotherland = 1172,
        Sht_Russian_JE_Harasho = 1173,
        Sht_Russian_JE_ShootThem = 1174,
        Sht_Russian_JE_TovaryshRemember = 1175,
        Sht_Russian_JE_UnitedAndMighty = 1176,
        Amb3D_RadioChatterAllied01_04 = 1177,
        Amb3D_RadioChatterAllied05_08 = 1178,
        Amb2D_RacingFans = 1179,
        Amb3D_RadioChatterAllied13_16 = 1180,
        Amb3D_RadioChatterChinese01_04 = 1181,
        Amb3D_RadioChatterChinese05_08 = 1182,
        Amb3D_RadioChatterChinese09_12 = 1183,
        Amb2D_RedHeat2 = 1184,
        Amb3D_RadioChatterRussian01_04 = 1185,
        Amb3D_RadioChatterRussian05_08 = 1186,
        Amb3D_RadioChatterRussian09_12 = 1187,
        Amb3D_RadioChatterRussian13_16 = 1188,
        MusMainMenu_KA_MM = 1189,
        HudFuelNoWarning = 1190,
        VehLoopLargeShipMadelyn = 1191,
        VocRace01Welcome = 1192,
        VocRace02countdown = 1193,
        VocRace03cleanStart = 1194,
        VocRace04running = 1195,
        VocRace05minefields = 1196,
        VocRace06speedDuel = 1197,
        VocRace07allThose = 1198,
        VocRace08moving = 1199,
        VocRace09dirtyTricks = 1200,
        VocRace10inside = 1201,
        VocRace11wire = 1202,
        Dlg_RussianTransmitter_1000 = 1203,
        Dlg_RussianTransmitter_1001 = 1204,
        Dlg_RussianTransmitter_1002 = 1205,
        Dlg_RussianTransmitter_1003 = 1206,
        Dlg_RussianTransmitter_1004 = 1207,
        Dlg_RussianTransmitter_1005 = 1208,
        Dlg_RussianTransmitter_1006 = 1209,
        Dlg_RussianTransmitter_1007 = 1210,
        Dlg_RussianTransmitter_1008 = 1211,
        Dlg_RussianTransmitter_1009 = 1212,
        Dlg_RussianTransmitter_1010 = 1213,
        Dlg_RussianTransmitter_1011 = 1214,
        Dlg_RussianTransmitter_1012 = 1215,
        Dlg_RussianTransmitter_1013 = 1216,
        Dlg_RussianTransmitter_1014 = 1217,
        Dlg_RussianTransmitter_1015 = 1218,
        Dlg_RussianTransmitter_1016 = 1219,
        Dlg_RussianTransmitter_1017 = 1220,
        Dlg_RussianTransmitter_1018 = 1221,
        Dlg_RussianTransmitter_1019 = 1222,
        Dlg_RussianTransmitter_1020 = 1223,
        Dlg_RussianTransmitter_1021 = 1224,
        Dlg_RussianTransmitter_1022 = 1225,
        Dlg_RussianTransmitter_1023 = 1226,
        Dlg_RussianTransmitter_1024 = 1227,
        Dlg_RussianTransmitter_1025 = 1228,
        Dlg_RussianTransmitter_1026 = 1229,
        Dlg_RussianTransmitter_1027 = 1230,
        Dlg_RussianTransmitter_1028 = 1231,
        Dlg_RussianTransmitter_1029 = 1232,
        Dlg_RussianTransmitter_1030 = 1233,
        Dlg_RussianTransmitter_1031 = 1234,
        Dlg_RussianTransmitter_1032 = 1235,
        Dlg_RussianTransmitter_1033 = 1236,
        Dlg_RussianTransmitter_1034 = 1237,
        Dlg_RussianTransmitter_1035 = 1238,
        Dlg_RussianTransmitter_1036 = 1239,
        Dlg_RussianTransmitter_1037 = 1240,
        Dlg_RussianTransmitter_1038 = 1241,
        Dlg_RussianTransmitter_1039 = 1242,
        Dlg_RussianTransmitter_1040 = 1243,
        Dlg_RussianTransmitter_1041 = 1244,
        Dlg_RussianTransmitter_1042 = 1245,
        Dlg_RussianTransmitter_1043 = 1246,
        Dlg_RussianTransmitter_1044 = 1247,
        Dlg_RussianTransmitter_1045 = 1248,
        Dlg_RussianTransmitter_1046 = 1249,
        Dlg_RussianTransmitter_1047 = 1250,
        Dlg_RussianTransmitter_1048 = 1251,
        Dlg_RussianTransmitter_1049 = 1252,
        Dlg_RussianTransmitter_1050 = 1253,
        Dlg_RussianTransmitter_1051 = 1254,
        Dlg_RussianTransmitter_1052 = 1255,
        Dlg_RussianTransmitter_1053 = 1256,
        Dlg_RussianTransmitter_1054 = 1257,
        Dlg_RussianTransmitter_1055 = 1258,
        Dlg_RussianTransmitter_1056 = 1259,
        Dlg_RussianTransmitter_1057 = 1260,
        Dlg_RussianTransmitter_1058 = 1261,
        Dlg_RussianTransmitter_1059 = 1262,
        Dlg_RussianTransmitter_1060 = 1263,
        Dlg_RussianTransmitter_1061 = 1264,
        Dlg_RussianTransmitter_1062 = 1265,
        Dlg_RussianTransmitter_1063 = 1266,
        Dlg_RussianTransmitter_1064 = 1267,
        Dlg_RussianTransmitter_1065 = 1268,
        Dlg_RussianTransmitter_1066 = 1269,
        Dlg_RussianTransmitter_1067 = 1270,
        Dlg_RussianTransmitter_1068 = 1271,
        Dlg_RussianTransmitter_1069 = 1272,
        Dlg_RussianTransmitter_1070 = 1273,
        Dlg_RussianTransmitter_1071 = 1274,
        Dlg_RussianTransmitter_1072 = 1275,
        Dlg_RussianTransmitter_1073 = 1276,
        Dlg_RussianTransmitter_1074 = 1277,
        Dlg_RussianTransmitter_1075 = 1278,
        Dlg_RussianTransmitter_1076 = 1279,
        Dlg_RussianTransmitter_1077 = 1280,
        Dlg_RussianTransmitter_1078 = 1281,
        Dlg_RussianTransmitter_1079 = 1282,
        Dlg_RussianTransmitter_1080 = 1283,
        Dlg_RussianTransmitter_1081 = 1284,
        Dlg_RussianTransmitter_1082 = 1285,
        Dlg_RussianTransmitter_1083 = 1286,
        Dlg_RussianTransmitter_1084 = 1287,
        Dlg_RussianTransmitter_1085 = 1288,
        Dlg_RussianTransmitter_1086 = 1289,
        Dlg_RussianTransmitter_1087 = 1290,
        Dlg_RussianTransmitter_1088 = 1291,
        Dlg_RussianTransmitter_1089 = 1292,
        Dlg_RussianTransmitter_1090 = 1293,
        Dlg_RussianTransmitter_1091 = 1294,
        Dlg_RussianTransmitter_1092 = 1295,
        Dlg_RussianTransmitter_1093 = 1296,
        Dlg_RussianTransmitter_1094 = 1297,
        Dlg_RussianTransmitter_1095 = 1298,
        Dlg_RussianTransmitter_1096 = 1299,
        Dlg_RussianTransmitter_1097 = 1300,
        Dlg_RussianTransmitter_1098 = 1301,
        Dlg_RussianTransmitter_1099 = 1302,
        Dlg_RussianTransmitter_1100 = 1303,
        Dlg_RussianTransmitter_1101 = 1304,
        Dlg_RussianTransmitter_1102 = 1305,
        Dlg_RussianTransmitter_1103 = 1306,
        Dlg_RussianTransmitter_1104 = 1307,
        Dlg_RussianTransmitter_1105 = 1308,
        Dlg_RussianTransmitter_1106 = 1309,
        Dlg_RussianTransmitter_1107 = 1310,
        Dlg_RussianTransmitter_1108 = 1311,
        Dlg_RussianTransmitter_1109 = 1312,
        Dlg_RussianTransmitter_1110 = 1313,
        Dlg_RussianTransmitter_1111 = 1314,
        Dlg_BarthsMoonTransmitter_1001 = 1315,
        Dlg_BarthsMoonTransmitter_1002 = 1316,
        Dlg_BarthsMoonTransmitter_1003 = 1317,
        Dlg_BarthsMoonTransmitter_1004 = 1318,
        Dlg_BarthsMoonTransmitter_1005 = 1319,
        Dlg_BarthsMoonTransmitter_1006 = 1320,
        Dlg_BarthsMoonTransmitter_1007 = 1321,
        Dlg_BarthsMoonTransmitter_1008 = 1322,
        Dlg_BarthsMoonTransmitter_1009 = 1323,
        Dlg_BarthsMoonTransmitter_1010 = 1324,
        Dlg_BarthsMoonTransmitter_1011 = 1325,
        Dlg_BarthsMoonTransmitter_1012 = 1326,
        Dlg_BarthsMoonTransmitter_1013 = 1327,
        Dlg_BarthsMoonTransmitter_1014 = 1328,
        Dlg_BarthsMoonTransmitter_1015 = 1329,
        Dlg_BarthsMoonTransmitter_1016 = 1330,
        Dlg_BarthsMoonTransmitter_1018 = 1331,
        Dlg_BarthsMoonTransmitter_1019 = 1332,
        Dlg_BarthsMoonTransmitter_1020 = 1333,
        Dlg_BarthsMoonTransmitter_1021 = 1334,
        Dlg_BarthsMoonTransmitter_1022 = 1335,
        Dlg_BarthsMoonTransmitter_1023 = 1336,
        Dlg_BarthsMoonTransmitter_1024 = 1337,
        Dlg_BarthsMoonTransmitter_1025 = 1338,
        Dlg_BarthsMoonTransmitter_1026 = 1339,
        Dlg_BarthsMoonTransmitter_1027 = 1340,
        Dlg_BarthsMoonTransmitter_1028 = 1341,
        Dlg_BarthsMoonTransmitter_1029 = 1342,
        Dlg_BarthsMoonTransmitter_1030 = 1343,
        Dlg_BarthsMoonTransmitter_1031 = 1344,
        Dlg_BarthsMoonTransmitter_1032 = 1345,
        Dlg_BarthsMoonTransmitter_1033 = 1346,
        Dlg_BarthsMoonTransmitter_1034 = 1347,
        Dlg_BarthsMoonTransmitter_1035 = 1348,
        Dlg_BarthsMoonTransmitter_1036 = 1349,
        Dlg_BarthsMoonTransmitter_1037 = 1350,
        Dlg_BarthsMoonTransmitter_1038 = 1351,
        Dlg_BarthsMoonTransmitter_1039 = 1352,
        Dlg_BarthsMoonTransmitter_1040 = 1353,
        Dlg_BarthsMoonTransmitter_1041 = 1354,
        Dlg_BarthsMoonTransmitter_1042 = 1355,
        Dlg_BarthsMoonTransmitter_1043 = 1356,
        Dlg_BarthsMoonTransmitter_1044 = 1357,
        Dlg_BarthsMoonTransmitter_1045 = 1358,
        Dlg_BarthsMoonTransmitter_1046 = 1359,
        Dlg_BarthsMoonTransmitter_1047 = 1360,
        Dlg_BarthsMoonTransmitter_1048 = 1361,
        Dlg_BarthsMoonTransmitter_1049 = 1362,
        Dlg_BarthsMoonTransmitter_1050 = 1363,
        Dlg_BarthsMoonTransmitter_1051 = 1364,
        Dlg_BarthsMoonTransmitter_1052 = 1365,
        Dlg_BarthsMoonTransmitter_1053 = 1366,
        Dlg_BarthsMoonTransmitter_1054 = 1367,
        Dlg_BarthsMoonTransmitter_1055 = 1368,
        Dlg_BarthsMoonTransmitter_1056 = 1369,
        Dlg_BarthsMoonTransmitter_1057 = 1370,
        Dlg_BarthsMoonTransmitter_1058 = 1371,
        Dlg_BarthsMoonTransmitter_1059 = 1372,
        Dlg_BarthsMoonTransmitter_1060 = 1373,
        Dlg_BarthsMoonTransmitter_1061 = 1374,
        Dlg_BarthsMoonTransmitter_1062 = 1375,
        Dlg_BarthsMoonTransmitter_1063 = 1376,
        Dlg_BarthsMoonTransmitter_1064 = 1377,
        Dlg_BarthsMoonTransmitter_1065 = 1378,
        Dlg_BarthsMoonTransmitter_1066 = 1379,
        Dlg_BarthsMoonTransmitter_1067 = 1380,
        Dlg_BarthsMoonTransmitter_1068 = 1381,
        Dlg_BarthsMoonTransmitter_1069 = 1382,
        Dlg_BarthsMoonTransmitter_1070 = 1383,
        Dlg_BarthsMoonTransmitter_1071 = 1384,
        Dlg_BarthsMoonTransmitter_1072 = 1385,
        Dlg_BarthsMoonTransmitter_1073 = 1386,
        Dlg_BarthsMoonTransmitter_1074 = 1387,
        Dlg_BarthsMoonTransmitter_1075 = 1388,
        Dlg_BarthsMoonTransmitter_1076 = 1389,
        Dlg_BarthsMoonTransmitter_1077 = 1390,
        Dlg_BarthsMoonTransmitter_1078 = 1391,
        Dlg_BarthsMoonTransmitter_1079 = 1392,
        Dlg_BarthsMoonTransmitter_1079b = 1393,
        Dlg_BarthsMoonTransmitter_1080 = 1394,
        Dlg_BarthsMoonTransmitter_1081 = 1395,
        Dlg_BarthsMoonTransmitter_1082 = 1396,
        Dlg_BarthsMoonTransmitter_1083 = 1397,
        Dlg_BarthsMoonTransmitter_1084 = 1398,
        Dlg_BarthsMoonTransmitter_1085 = 1399,
        Dlg_BarthsMoonTransmitter_1086 = 1400,
        Dlg_BarthsMoonTransmitter_1087 = 1401,
        Dlg_BarthsMoonTransmitter_1088 = 1402,
        Dlg_BarthsMoonTransmitter_1089 = 1403,
        Dlg_BarthsMoonTransmitter_1090 = 1404,
        Dlg_BarthsMoonTransmitter_1091 = 1405,
        Dlg_BarthsMoonTransmitter_1092 = 1406,
        Dlg_BarthsMoonTransmitter_1093 = 1407,
        Dlg_BarthsMoonTransmitter_1094 = 1408,
        Dlg_BarthsMoonTransmitter_1095 = 1409,
        Dlg_BarthsMoonTransmitter_1096 = 1410,
        Dlg_BarthsMoonTransmitter_1097 = 1411,
        Dlg_BarthsMoonTransmitter_1098 = 1412,
        Dlg_BarthsMoonTransmitter_1099 = 1413,
        Dlg_BarthsMoonTransmitter_1100 = 1414,
        Dlg_BarthsMoonTransmitter_1101 = 1415,
        Dlg_BarthsMoonTransmitter_1102 = 1416,
        Dlg_BarthsMoonTransmitter_1103 = 1417,
        Dlg_BarthsMoonTransmitter_1104 = 1418,
        Dlg_BarthsMoonTransmitter_1105 = 1419,
        Dlg_BarthsMoonTransmitter_1106 = 1420,
        Dlg_BarthsMoonTransmitter_1107 = 1421,
        Dlg_BarthsMoonTransmitter_1108 = 1422,
        Dlg_BarthsMoonTransmitter_1109 = 1423,
        Dlg_BarthsMoonTransmitter_1110 = 1424,
        Dlg_BarthsMoonTransmitter_1111 = 1425,
        Dlg_BarthsMoonTransmitter_1112 = 1426,
        Dlg_BarthsMoonTransmitter_1113 = 1427,
        Dlg_BarthsMoonTransmitter_1114 = 1428,
        Dlg_BarthsMoonTransmitter_1115 = 1429,
        Dlg_BarthsMoonTransmitter_1116 = 1430,
        Dlg_BarthsMoonTransmitter_1117 = 1431,
        Dlg_BarthsMoonTransmitter_1118 = 1432,
        Dlg_BarthsMoonTransmitter_1119 = 1433,
        Dlg_BarthsMoonTransmitter_1120 = 1434,
        Dlg_BarthsMoonTransmitter_1121 = 1435,
        Dlg_BarthsMoonTransmitter_1122 = 1436,
        Dlg_BarthsMoonTransmitter_1123 = 1437,
        Dlg_BarthsMoonTransmitter_1124 = 1438,
        Dlg_BarthsMoonTransmitter_1125 = 1439,
        Dlg_BarthsMoonTransmitter_1126 = 1440,
        Dlg_BarthsMoonTransmitter_1127 = 1441,
        Dlg_BarthsMoonTransmitter_1128 = 1442,
        Dlg_BarthsMoonTransmitter_1129 = 1443,
        Dlg_BarthsMoonTransmitter_1130 = 1444,
        Dlg_BarthsMoonTransmitter_1131 = 1445,
        Dlg_BarthsMoonTransmitter_1132 = 1446,
        Dlg_BarthsMoonTransmitter_1133 = 1447,
        Dlg_BarthsMoonTransmitter_1134 = 1448,
        Dlg_BarthsMoonTransmitter_1135 = 1449,
        Dlg_BarthsMoonTransmitter_1136 = 1450,
        Dlg_BarthsMoonTransmitter_1137 = 1451,
        Dlg_BarthsMoonTransmitter_1138 = 1452,
        Dlg_BarthsMoonTransmitter_1139 = 1453,
        Dlg_BarthsMoonTransmitter_1140 = 1454,
        Dlg_BarthsMoonTransmitter_1141 = 1455,
        Dlg_BarthsMoonTransmitter_1142 = 1456,
        Dlg_BarthsMoonTransmitter_1143 = 1457,
        Dlg_BarthsMoonTransmitter_1144 = 1458,
        Dlg_BarthsMoonTransmitter_1145 = 1459,
        Dlg_BarthsMoonTransmitter_1146 = 1460,
        Dlg_BarthsMoonTransmitter_1147 = 1461,
        Dlg_BarthsMoonTransmitter_1148 = 1462,
        Dlg_BarthsMoonTransmitter_1149 = 1463,
        Dlg_BarthsMoonTransmitter_1150 = 1464,
        Dlg_BarthsMoonTransmitter_1151 = 1465,
        Dlg_BarthsMoonTransmitter_1152 = 1466,
        Dlg_BarthsMoonTransmitter_1153 = 1467,
        Dlg_BarthsMoonTransmitter_1154 = 1468,
        Dlg_BarthsMoonTransmitter_1155 = 1469,
        Dlg_BarthsMoonTransmitter_1156 = 1470,
        Dlg_BarthsMoonTransmitter_1157 = 1471,
        Dlg_BarthsMoonTransmitter_1158 = 1472,
        Dlg_BarthsMoonTransmitter_1159 = 1473,
        Dlg_BarthsMoonTransmitter_1160 = 1474,
        Dlg_BarthsMoonTransmitter_1161 = 1475,
        Dlg_BarthsMoonTransmitter_1162 = 1476,
        Dlg_BarthsMoonTransmitter_1163 = 1477,
        Dlg_BarthsMoonTransmitter_1164 = 1478,
        Dlg_BarthsMoonTransmitter_1165 = 1479,
        Dlg_BarthsMoonTransmitter_1166 = 1480,
        Dlg_BarthsMoonTransmitter_1167 = 1481,
        Dlg_BarthsMoonTransmitter_1168 = 1482,
        Dlg_BarthsMoonTransmitter_1169 = 1483,
        Dlg_BarthsMoonTransmitter_1170 = 1484,
        Dlg_BarthsMoonTransmitter_1171 = 1485,
        Dlg_BarthsMoonTransmitter_1172 = 1486,
        Dlg_BarthsMoonTransmitter_1173 = 1487,
        Dlg_BarthsMoonTransmitter_1174 = 1488,
        Dlg_BarthsMoonTransmitter_1175 = 1489,
        Dlg_BarthsMoonTransmitter_1176 = 1490,
        Dlg_BarthsMoonTransmitter_1177 = 1491,
        Dlg_BarthsMoonTransmitter_1178 = 1492,
        Dlg_BarthsMoonTransmitter_1179 = 1493,
        Dlg_BarthsMoonTransmitter_1180 = 1494,
        Dlg_BarthsMoonTransmitter_1181 = 1495,
        Dlg_BarthsMoonTransmitter_1182 = 1496,
        Dlg_BarthsMoonTransmitter_1183 = 1497,
        Dlg_BarthsMoonTransmitter_1184 = 1498,
        Dlg_BarthsMoonTransmitter_1185 = 1499,
        Dlg_BarthsMoonTransmitter_1186 = 1500,
        Dlg_BarthsMoonTransmitter_1187 = 1501,
        Dlg_BarthsMoonTransmitter_1188 = 1502,
        Dlg_BarthsMoonTransmitter_1189 = 1503,
        Dlg_BarthsMoonTransmitter_1190 = 1504,
        Dlg_BarthsMoonTransmitter_1191 = 1505,
        Dlg_BarthsMoonTransmitter_1192 = 1506,
        Dlg_BarthsMoonTransmitter_1193 = 1507,
        Dlg_BarthsMoonTransmitter_1194 = 1508,
        Dlg_BarthsMoonTransmitter_1195 = 1509,
        Dlg_BarthsMoonTransmitter_1196 = 1510,
        Dlg_BarthsMoonTransmitter_1197 = 1511,
        Dlg_BarthsMoonTransmitter_1198 = 1512,
        Dlg_BarthsMoonTransmitter_1199 = 1513,
        Dlg_BarthsMoonTransmitter_1200 = 1514,
        Dlg_BarthsMoonTransmitter_1201 = 1515,
        Dlg_BarthsMoonTransmitter_1202 = 1516,
        Dlg_BarthsMoonTransmitter_1203 = 1517,
        Dlg_BarthsMoonTransmitter_1204 = 1518,
        Dlg_BarthsMoonTransmitter_1205 = 1519,
        Dlg_BarthsMoonTransmitter_1206 = 1520,
        Dlg_BarthsMoonTransmitter_1207 = 1521,
        Dlg_BarthsMoonTransmitter_1208 = 1522,
        Dlg_BarthsMoonTransmitter_1209 = 1523,
        Dlg_BarthsMoonTransmitter_1210 = 1524,
        Dlg_BarthsMoonTransmitter_1211 = 1525,
        Dlg_BarthsMoonTransmitter_1212 = 1526,
        Dlg_BarthsMoonTransmitter_1213 = 1527,
        Dlg_BarthsMoonTransmitter_1214 = 1528,
        Dlg_BarthsMoonTransmitter_1215 = 1529,
        Dlg_FortValiant_1000 = 1530,
        Dlg_FortValiant_1001 = 1531,
        Dlg_FortValiant_1002 = 1532,
        Dlg_FortValiant_1003 = 1533,
        Dlg_FortValiant_1004 = 1534,
        Dlg_FortValiant_1005 = 1535,
        Dlg_FortValiant_1006 = 1536,
        Dlg_FortValiant_1007 = 1537,
        Dlg_FortValiant_1008 = 1538,
        Dlg_FortValiant_1009 = 1539,
        Dlg_FortValiant_1010 = 1540,
        Dlg_FortValiant_1011 = 1541,
        Dlg_FortValiant_1012 = 1542,
        Dlg_FortValiant_1013 = 1543,
        Dlg_FortValiant_1014 = 1544,
        Dlg_FortValiant_1015 = 1545,
        Dlg_FortValiant_1016 = 1546,
        Dlg_FortValiant_1017 = 1547,
        Dlg_FortValiant_1018 = 1548,
        Dlg_FortValiant_1019 = 1549,
        Dlg_FortValiant_1020 = 1550,
        Dlg_FortValiant_1021 = 1551,
        Dlg_FortValiant_1022 = 1552,
        Dlg_FortValiant_1023 = 1553,
        Dlg_FortValiant_1024 = 1554,
        Dlg_FortValiant_1025 = 1555,
        Dlg_FortValiant_1026 = 1556,
        Dlg_FortValiant_1027 = 1557,
        Dlg_FortValiant_1028 = 1558,
        Dlg_FortValiant_1029 = 1559,
        Dlg_FortValiant_1030 = 1560,
        Dlg_FortValiant_1031 = 1561,
        Dlg_FortValiant_1032 = 1562,
        Dlg_FortValiant_1033 = 1563,
        Dlg_FortValiant_1034 = 1564,
        Dlg_FortValiant_1035 = 1565,
        Dlg_FortValiant_1036 = 1566,
        Dlg_FortValiant_1037 = 1567,
        Dlg_FortValiant_1038 = 1568,
        Dlg_FortValiant_1039 = 1569,
        Dlg_FortValiant_1040 = 1570,
        Dlg_FortValiant_1041 = 1571,
        Dlg_FortValiant_1042 = 1572,
        Dlg_FortValiant_1043 = 1573,
        Dlg_FortValiant_1044 = 1574,
        Dlg_FortValiant_1045 = 1575,
        Dlg_FortValiant_1046 = 1576,
        Dlg_FortValiantB_1000 = 1577,
        Dlg_FortValiantB_1001 = 1578,
        Dlg_FortValiantB_1002 = 1579,
        Dlg_FortValiantB_1003 = 1580,
        Dlg_FortValiantB_1004 = 1581,
        Dlg_FortValiantB_1005 = 1582,
        Dlg_FortValiantB_1006 = 1583,
        Dlg_FortValiantB_1007 = 1584,
        Dlg_FortValiantB_1008 = 1585,
        Dlg_FortValiantB_1009 = 1586,
        Dlg_FortValiantB_1010 = 1587,
        Dlg_FortValiantB_1011 = 1588,
        Dlg_FortValiantB_1012 = 1589,
        Dlg_FortValiantB_1013 = 1590,
        Dlg_FortValiantB_1014 = 1591,
        Dlg_FortValiantB_1015 = 1592,
        Dlg_FortValiantB_1016 = 1593,
        Dlg_FortValiantB_1017 = 1594,
        Dlg_FortValiantB_1018 = 1595,
        Dlg_FortValiantB_1019 = 1596,
        Dlg_FortValiantB_1020 = 1597,
        Dlg_FortValiantB_1021 = 1598,
        Dlg_FortValiantB_1022 = 1599,
        Dlg_FortValiantB_1023 = 1600,
        Dlg_FortValiantB_1024 = 1601,
        Dlg_FortValiantB_1025 = 1602,
        Dlg_FortValiantB_1026 = 1603,
        Dlg_FortValiantB_1027 = 1604,
        Dlg_FortValiantB_1028 = 1605,
        Dlg_FortValiantB_1029 = 1606,
        Dlg_FortValiantB_1030 = 1607,
        Dlg_FortValiantB_1031 = 1608,
        Dlg_FortValiantB_1032 = 1609,
        Dlg_ReichstagA_1000 = 1610,
        Dlg_ReichstagA_1001 = 1611,
        Dlg_ReichstagA_1002 = 1612,
        Dlg_ReichstagA_1003 = 1613,
        Dlg_ReichstagA_1004 = 1614,
        Dlg_ReichstagA_1005 = 1615,
        Dlg_ReichstagA_1006 = 1616,
        Dlg_ReichstagA_1007 = 1617,
        Dlg_ReichstagA_1008 = 1618,
        Dlg_ReichstagA_1009 = 1619,
        Dlg_ReichstagA_1010 = 1620,
        Dlg_ReichstagA_1011 = 1621,
        Dlg_ReichstagA_1012 = 1622,
        Dlg_ReichstagA_1013 = 1623,
        Dlg_ReichstagA_1014 = 1624,
        Dlg_ReichstagA_1015 = 1625,
        Dlg_ReichstagA_1016 = 1626,
        Dlg_ReichstagA_1017 = 1627,
        Dlg_ReichstagA_1018 = 1628,
        Dlg_ReichstagA_1019 = 1629,
        Dlg_ReichstagA_1020 = 1630,
        Dlg_ReichstagA_1021 = 1631,
        Dlg_ReichstagA_1022 = 1632,
        Dlg_ReichstagA_1023 = 1633,
        Dlg_ReichstagA_1024 = 1634,
        Dlg_ReichstagA_1025 = 1635,
        Dlg_ReichstagA_1026 = 1636,
        Dlg_ReichstagA_1027 = 1637,
        Dlg_ReichstagA_1028 = 1638,
        Dlg_ReichstagA_1029 = 1639,
        Dlg_ReichstagA_1030 = 1640,
        Dlg_ReichstagA_1031 = 1641,
        Dlg_ReichstagA_1032 = 1642,
        Dlg_ReichstagA_1033 = 1643,
        Dlg_ReichstagA_1034 = 1644,
        Dlg_ReichstagA_1035 = 1645,
        Dlg_ReichstagA_1036 = 1646,
        Dlg_ReichstagA_1037 = 1647,
        Dlg_ReichstagA_1038 = 1648,
        Dlg_ReichstagC_1000 = 1649,
        Dlg_ReichstagC_1001 = 1650,
        Dlg_ReichstagC_1002 = 1651,
        Dlg_ReichstagC_1003 = 1652,
        Dlg_ReichstagC_1004 = 1653,
        Dlg_ReichstagC_1005 = 1654,
        Dlg_ReichstagC_1006 = 1655,
        Dlg_ReichstagC_1007 = 1656,
        Dlg_ReichstagC_1008 = 1657,
        Dlg_ReichstagC_1009 = 1658,
        Dlg_ReichstagC_1010 = 1659,
        Dlg_ReichstagC_1011 = 1660,
        Dlg_ReichstagC_1012 = 1661,
        Dlg_ReichstagC_1013 = 1662,
        Dlg_ReichstagC_1014 = 1663,
        Dlg_ReichstagC_1015 = 1664,
        Dlg_ReichstagC_1016 = 1665,
        Dlg_ReichstagC_1017 = 1666,
        Dlg_ReichstagC_1018 = 1667,
        Dlg_ReichstagC_1019 = 1668,
        Dlg_ReichstagC_1020 = 1669,
        Dlg_ReichstagC_1021 = 1670,
        Dlg_ReichstagC_1022 = 1671,
        Dlg_ReichstagC_1023 = 1672,
        Dlg_ReichstagC_1024 = 1673,
        Dlg_ReichstagC_1025 = 1674,
        Dlg_ReichstagC_1026 = 1675,
        Dlg_ReichstagC_1027 = 1676,
        Dlg_ReichstagC_1028 = 1677,
        Dlg_ReichstagC_1029 = 1678,
        Dlg_ReichstagC_1030 = 1679,
        Dlg_ReichstagC_1031 = 1680,
        Dlg_ReichstagC_1032 = 1681,
        Dlg_ReichstagC_1033 = 1682,
        Dlg_ReichstagC_1034 = 1683,
        Dlg_ReichstagC_1035 = 1684,
        Dlg_ReichstagC_1036 = 1685,
        Dlg_ReichstagC_1037 = 1686,
        Dlg_ReichstagC_1038 = 1687,
        Dlg_ReichstagC_1039 = 1688,
        Dlg_ReichstagC_1040 = 1689,
        Dlg_ReichstagC_1041 = 1690,
        Dlg_ReichstagC_1042 = 1691,
        Dlg_ReichstagC_1043 = 1692,
        Dlg_ReichstagC_1044 = 1693,
        Dlg_ReichstagC_1045 = 1694,
        Dlg_ReichstagC_1046 = 1695,
        Dlg_ReichstagC_1047 = 1696,
        Dlg_ReichstagC_1048 = 1697,
        Dlg_ReichstagC_1049 = 1698,
        Dlg_ReichstagC_1050 = 1699,
        Dlg_ReichstagC_1051 = 1700,
        Dlg_ReichstagC_1052 = 1701,
        Dlg_ReichstagC_1053 = 1702,
        Dlg_ReichstagC_1054 = 1703,
        Dlg_ReichstagC_1055 = 1704,
        Dlg_ReichstagC_1056 = 1705,
        Dlg_ReichstagC_1057 = 1706,
        Dlg_ReichstagC_1058 = 1707,
        Dlg_ReichstagC_1059 = 1708,
        Dlg_ReichstagC_1060 = 1709,
        Dlg_ReichstagC_1061 = 1710,
        Dlg_ReichstagC_1062 = 1711,
        Dlg_ReichstagC_1063 = 1712,
        Dlg_ReichstagC_1064 = 1713,
        Dlg_ReichstagC_1065 = 1714,
        Dlg_EacPrison_1000 = 1715,
        Dlg_EacPrison_1001 = 1716,
        Dlg_EacPrison_1002 = 1717,
        Dlg_EacPrison_1003 = 1718,
        Dlg_EacPrison_1004 = 1719,
        Dlg_EacPrison_1005 = 1720,
        Dlg_EacPrison_1006 = 1721,
        Dlg_EacPrison_1007 = 1722,
        Dlg_EacPrison_1008 = 1723,
        Dlg_EacPrison_1009 = 1724,
        Dlg_EacPrison_1010 = 1725,
        Dlg_EacPrison_1011 = 1726,
        Dlg_EacPrison_1012 = 1727,
        Dlg_EacPrison_1013 = 1728,
        Dlg_EacPrison_1014 = 1729,
        Dlg_EacPrison_1015 = 1730,
        Dlg_EacPrison_1016 = 1731,
        Dlg_EacPrison_1017 = 1732,
        Dlg_EacPrison_1018 = 1733,
        Dlg_EacPrison_1019 = 1734,
        Dlg_EacPrison_1020 = 1735,
        Dlg_EacPrison_1021 = 1736,
        Dlg_EacPrison_1022 = 1737,
        Dlg_EacPrison_1023 = 1738,
        Dlg_EacPrison_1024 = 1739,
        Dlg_EacPrison_1025 = 1740,
        Dlg_EacPrison_1026 = 1741,
        Dlg_EacPrison_1027 = 1742,
        Dlg_EacPrison_1028 = 1743,
        Dlg_EacPrison_1029 = 1744,
        Dlg_EacPrison_1030 = 1745,
        Dlg_EacPrison_1031 = 1746,
        Dlg_EacPrison_1032 = 1747,
        Dlg_EacPrison_1033 = 1748,
        Dlg_EacPrison_1034 = 1749,
        Dlg_EacPrison_1035 = 1750,
        Dlg_EacPrison_1036 = 1751,
        Dlg_EacPrison_1037 = 1752,
        Dlg_EacPrison_1038 = 1753,
        Dlg_EacPrison_1039 = 1754,
        Dlg_EacPrison_1040 = 1755,
        Dlg_EacPrison_1041 = 1756,
        Dlg_EacPrison_1042 = 1757,
        Dlg_EacPrison_1043 = 1758,
        Dlg_EacPrison_1044 = 1759,
        Dlg_EacPrison_1045 = 1760,
        Dlg_EacPrison_1046 = 1761,
        Dlg_EacPrison_1047 = 1762,
        Dlg_EacPrison_1048 = 1763,
        Dlg_EacPrison_1049 = 1764,
        Dlg_EacPrison_1050 = 1765,
        Dlg_EacPrison_1051 = 1766,
        Dlg_EacPrison_1052 = 1767,
        Dlg_EacPrison_1053 = 1768,
        Dlg_EacPrison_1054 = 1769,
        Dlg_EacPrison_1055 = 1770,
        Dlg_EacPrison_1056 = 1771,
        Dlg_EacPrison_1057 = 1772,
        Dlg_EacPrison_1058 = 1773,
        Dlg_EacPrison_1059 = 1774,
        Dlg_EacPrison_1060 = 1775,
        Dlg_EacPrison_1061 = 1776,
        Dlg_EacPrison_1062 = 1777,
        Dlg_EacPrison_1063 = 1778,
        Dlg_EacPrison_1064 = 1779,
        Dlg_EacPrison_1065 = 1780,
        Dlg_EacPrison_1066 = 1781,
        Dlg_EacPrison_1067 = 1782,
        Dlg_EacPrison_1068 = 1783,
        Dlg_EacPrison_1069 = 1784,
        Dlg_EacPrison_1070 = 1785,
        Dlg_EacPrison_1071 = 1786,
        Dlg_EacPrison_1072 = 1787,
        Dlg_EacPrison_1073 = 1788,
        Dlg_EacPrison_1074 = 1789,
        Dlg_EacPrison_1075 = 1790,
        Dlg_EacPrison_1076 = 1791,
        Dlg_EacPrison_1077 = 1792,
        Dlg_EacPrison_1078 = 1793,
        Dlg_EacPrison_1079 = 1794,
        Dlg_EacPrison_1080 = 1795,
        Dlg_EacPrison_1081 = 1796,
        Dlg_EacPrison_1082 = 1797,
        Dlg_EacPrison_1083 = 1798,
        Dlg_EacPrison_1084 = 1799,
        Dlg_EacPrison_1085 = 1800,
        Dlg_EacPrison_1086 = 1801,
        Dlg_EacPrison_1087 = 1802,
        Dlg_EacPrison_1088 = 1803,
        Dlg_ChineseEscape_1000 = 1804,
        Dlg_ChineseEscape_1001 = 1805,
        Dlg_ChineseEscape_1002 = 1806,
        Dlg_ChineseEscape_1003 = 1807,
        Dlg_ChineseEscape_1004 = 1808,
        Dlg_ChineseEscape_1005 = 1809,
        Dlg_ChineseEscape_1006 = 1810,
        Dlg_ChineseEscape_1007 = 1811,
        Dlg_ChineseEscape_1008 = 1812,
        Dlg_ChineseEscape_1009 = 1813,
        Dlg_ChineseEscape_1010 = 1814,
        Dlg_ChineseEscape_1011 = 1815,
        Dlg_ChineseEscape_1012 = 1816,
        Dlg_ChineseEscape_1013 = 1817,
        Dlg_ChineseEscape_1014 = 1818,
        Dlg_ChineseEscape_1015 = 1819,
        Dlg_ChineseEscape_1016 = 1820,
        Dlg_ChineseEscape_1017 = 1821,
        Dlg_ChineseEscape_1018 = 1822,
        Dlg_ChineseEscape_1019 = 1823,
        Dlg_ChineseEscape_1020 = 1824,
        Dlg_ChineseEscape_1021 = 1825,
        Dlg_ChineseEscape_1022 = 1826,
        Dlg_ChineseEscape_1023 = 1827,
        Dlg_ChineseEscape_1024 = 1828,
        Dlg_ChineseEscape_1025 = 1829,
        Dlg_ChineseEscape_1026 = 1830,
        Dlg_ChineseEscape_1027 = 1831,
        Dlg_ResearchVessel_1000 = 1832,
        Dlg_ResearchVessel_1001 = 1833,
        Dlg_ResearchVessel_1002 = 1834,
        Dlg_ResearchVessel_1003 = 1835,
        Dlg_ResearchVessel_1004 = 1836,
        Dlg_ResearchVessel_1005 = 1837,
        Dlg_ResearchVessel_1006 = 1838,
        Dlg_ResearchVessel_1007 = 1839,
        Dlg_ResearchVessel_1008 = 1840,
        Dlg_ResearchVessel_1009 = 1841,
        Dlg_ResearchVessel_1010 = 1842,
        Dlg_ResearchVessel_1011 = 1843,
        Dlg_ResearchVessel_1012 = 1844,
        Dlg_ResearchVessel_1013 = 1845,
        Dlg_ResearchVessel_1014 = 1846,
        Dlg_ResearchVessel_1015 = 1847,
        Dlg_ResearchVessel_1016 = 1848,
        Dlg_ResearchVessel_1017 = 1849,
        Dlg_ResearchVessel_1018 = 1850,
        Dlg_ResearchVessel_1019 = 1851,
        Dlg_ResearchVessel_1020 = 1852,
        Dlg_ResearchVessel_1021 = 1853,
        Dlg_ResearchVessel_1022 = 1854,
        Dlg_ResearchVessel_1023 = 1855,
        Dlg_ResearchVessel_1024 = 1856,
        Dlg_ResearchVessel_1025 = 1857,
        Dlg_ResearchVessel_1026 = 1858,
        Dlg_ResearchVessel_1027 = 1859,
        Dlg_ResearchVessel_1028 = 1860,
        Dlg_ResearchVessel_1029 = 1861,
        Dlg_ResearchVessel_1030 = 1862,
        Dlg_ResearchVessel_1031 = 1863,
        Dlg_ResearchVessel_1032 = 1864,
        Dlg_ResearchVessel_1033 = 1865,
        Dlg_ResearchVessel_1034 = 1866,
        Dlg_ResearchVessel_1035 = 1867,
        Dlg_ResearchVessel_1036 = 1868,
        Dlg_ResearchVessel_1037 = 1869,
        Dlg_ResearchVessel_1038 = 1870,
        Dlg_ResearchVessel_1039 = 1871,
        Dlg_ResearchVessel_1040 = 1872,
        Dlg_ResearchVessel_1041 = 1873,
        Dlg_ResearchVessel_1042 = 1874,
        Dlg_ResearchVessel_1043 = 1875,
        Dlg_ResearchVessel_1044 = 1876,
        Dlg_ResearchVessel_1045 = 1877,
        Dlg_ResearchVessel_1046 = 1878,
        Dlg_ResearchVessel_1047 = 1879,
        Dlg_ResearchVessel_1048 = 1880,
        Dlg_ResearchVessel_1049 = 1881,
        Dlg_ResearchVessel_1050 = 1882,
        Dlg_ResearchVessel_1051 = 1883,
        Dlg_ResearchVessel_1052 = 1884,
        Dlg_ResearchVessel_1053 = 1885,
        Dlg_ResearchVessel_1054 = 1886,
        Dlg_ResearchVessel_1055 = 1887,
        Dlg_ResearchVessel_1056 = 1888,
        Dlg_Laika_1000 = 1889,
        Dlg_Laika_1001 = 1890,
        Dlg_Laika_1002 = 1891,
        Dlg_Laika_1003 = 1892,
        Dlg_Laika_1004 = 1893,
        Dlg_Laika_1005 = 1894,
        Dlg_Laika_1006 = 1895,
        Dlg_Laika_1007 = 1896,
        Dlg_Laika_1008 = 1897,
        Dlg_Laika_1009 = 1898,
        Dlg_Laika_1010 = 1899,
        Dlg_Laika_1011 = 1900,
        Dlg_Laika_1012 = 1901,
        Dlg_Laika_1013 = 1902,
        Dlg_Laika_1014 = 1903,
        Dlg_Laika_1015 = 1904,
        Dlg_Laika_1016 = 1905,
        Dlg_Laika_1017 = 1906,
        Dlg_Laika_1018 = 1907,
        Dlg_Laika_1019 = 1908,
        Dlg_Laika_1020 = 1909,
        Dlg_Laika_1021 = 1910,
        Dlg_Laika_1022 = 1911,
        Dlg_Laika_1023 = 1912,
        Dlg_Laika_1024 = 1913,
        Dlg_Laika_1025 = 1914,
        Dlg_Laika_1026 = 1915,
        Dlg_Laika_1027 = 1916,
        Dlg_Laika_1028 = 1917,
        Dlg_Laika_1029 = 1918,
        Dlg_Laika_1030 = 1919,
        Dlg_Laika_1031 = 1920,
        Dlg_Laika_1032 = 1921,
        Dlg_Laika_1033 = 1922,
        Dlg_Laika_1035 = 1923,
        Dlg_Laika_1036 = 1924,
        Dlg_Laika_1037 = 1925,
        Dlg_Laika_1038 = 1926,
        Dlg_Laika_1039 = 1927,
        Dlg_Laika_1040 = 1928,
        Dlg_Laika_1041 = 1929,
        Dlg_Laika_1042 = 1930,
        Dlg_Laika_1043 = 1931,
        Dlg_Laika_1044 = 1932,
        Dlg_Laika_1045 = 1933,
        Dlg_Laika_1046 = 1934,
        Dlg_Laika_1047 = 1935,
        Dlg_Laika_1048 = 1936,
        Dlg_Laika_1049 = 1937,
        Dlg_Laika_1050 = 1938,
        Dlg_Laika_1051 = 1939,
        Dlg_Laika_1052 = 1940,
        Dlg_Laika_1053 = 1941,
        Dlg_Laika_1054 = 1942,
        Dlg_Laika_1055 = 1943,
        Dlg_Laika_1056 = 1944,
        Dlg_Laika_1057 = 1945,
        Dlg_Laika_1058 = 1946,
        Dlg_Laika_1059 = 1947,
        Dlg_Laika_1060 = 1948,
        Dlg_Laika_1061 = 1949,
        Dlg_Laika_1062 = 1950,
        Dlg_Laika_1063 = 1951,
        Dlg_Laika_1064 = 1952,
        Dlg_Laika_1065 = 1953,
        Dlg_Laika_1066 = 1954,
        Dlg_Laika_1067 = 1955,
        Dlg_Laika_1068 = 1956,
        Dlg_Laika_1069 = 1957,
        Dlg_Laika_1070 = 1958,
        Dlg_Laika_1071 = 1959,
        Dlg_Laika_1072 = 1960,
        Dlg_Laika_1073 = 1961,
        Dlg_Laika_1074 = 1962,
        Dlg_Laika_1075 = 1963,
        Dlg_Laika_1076 = 1964,
        Dlg_Laika_1077 = 1965,
        Dlg_Laika_1078 = 1966,
        Dlg_Laika_1079 = 1967,
        Dlg_Laika_1080 = 1968,
        Dlg_Laika_1081 = 1969,
        Dlg_Laika_1082 = 1970,
        Dlg_Laika_1084 = 1971,
        Dlg_Laika_1085 = 1972,
        Dlg_Laika_1086 = 1973,
        Dlg_Laika_1087 = 1974,
        Dlg_Laika_1088 = 1975,
        Dlg_Laika_1089 = 1976,
        Dlg_Laika_1090 = 1977,
        Dlg_Laika_1091 = 1978,
        Dlg_Laika_1092 = 1979,
        Dlg_Laika_1093 = 1980,
        Dlg_Laika_1094 = 1981,
        Dlg_Laika_1095 = 1982,
        Dlg_Laika_1096 = 1983,
        Dlg_Laika_1097 = 1984,
        Dlg_Laika_1098 = 1985,
        Dlg_Laika_1099 = 1986,
        Dlg_Laika_1100 = 1987,
        Dlg_Laika_1101 = 1988,
        Dlg_Laika_1102 = 1989,
        Dlg_Laika_1103 = 1990,
        Dlg_Laika_1104 = 1991,
        Dlg_Laika_1105 = 1992,
        Dlg_Laika_1106 = 1993,
        Dlg_Laika_1107 = 1994,
        Dlg_Laika_1108 = 1995,
        Dlg_Laika_1109 = 1996,
        Dlg_Laika_1110 = 1997,
        Dlg_Laika_1111 = 1998,
        Dlg_Laika_1112 = 1999,
        Dlg_Laika_1113 = 2000,
        Dlg_Laika_1114 = 2001,
        Dlg_Laika_1115 = 2002,
        Dlg_Laika_1116 = 2003,
        Dlg_Laika_1117 = 2004,
        Dlg_Laika_1118 = 2005,
        Dlg_Laika_1119 = 2006,
        Dlg_Laika_1120 = 2007,
        Dlg_Laika_1121 = 2008,
        Dlg_Laika_1122 = 2009,
        Dlg_Laika_1123 = 2010,
        Dlg_Laika_1124 = 2011,
        Dlg_Laika_1125 = 2012,
        Dlg_BarthsMoonPlant_1000 = 2013,
        Dlg_BarthsMoonPlant_1001 = 2014,
        Dlg_BarthsMoonPlant_1002 = 2015,
        Dlg_BarthsMoonPlant_1003 = 2016,
        Dlg_BarthsMoonPlant_1004 = 2017,
        Dlg_BarthsMoonPlant_1005 = 2018,
        Dlg_BarthsMoonPlant_1006 = 2019,
        Dlg_BarthsMoonPlant_1007 = 2020,
        Dlg_BarthsMoonPlant_1008 = 2021,
        Dlg_BarthsMoonPlant_1009 = 2022,
        Dlg_BarthsMoonPlant_1010 = 2023,
        Dlg_BarthsMoonPlant_1011 = 2024,
        Dlg_BarthsMoonPlant_1012 = 2025,
        Dlg_BarthsMoonPlant_1013 = 2026,
        Dlg_BarthsMoonPlant_1014 = 2027,
        Dlg_BarthsMoonPlant_1015 = 2028,
        Dlg_BarthsMoonPlant_1016 = 2029,
        Dlg_BarthsMoonPlant_1017 = 2030,
        Dlg_BarthsMoonPlant_1018 = 2031,
        Dlg_BarthsMoonPlant_1019 = 2032,
        Dlg_BarthsMoonPlant_1020 = 2033,
        Dlg_BarthsMoonPlant_1021 = 2034,
        Dlg_BarthsMoonPlant_1022 = 2035,
        Dlg_BarthsMoonPlant_1023 = 2036,
        Dlg_BarthsMoonPlant_1024 = 2037,
        Dlg_BarthsMoonPlant_1025 = 2038,
        Dlg_BarthsMoonPlant_1026 = 2039,
        Dlg_BarthsMoonPlant_1027 = 2040,
        Dlg_BarthsMoonPlant_1028 = 2041,
        Dlg_BarthsMoonPlant_1029 = 2042,
        Dlg_BarthsMoonPlant_1030 = 2043,
        Dlg_BarthsMoonPlant_1031 = 2044,
        Dlg_BarthsMoonPlant_1032 = 2045,
        Dlg_BarthsMoonPlant_1033 = 2046,
        Dlg_BarthsMoonPlant_1034 = 2047,
        Dlg_BarthsMoonPlant_1035 = 2048,
        Dlg_BarthsMoonPlant_1036 = 2049,
        Dlg_BarthsMoonPlant_1037 = 2050,
        Dlg_BarthsMoonPlant_1038 = 2051,
        Dlg_BarthsMoonPlant_1039 = 2052,
        Dlg_BarthsMoonPlant_1040 = 2053,
        Dlg_BarthsMoonPlant_1041 = 2054,
        Dlg_BarthsMoonPlant_1042 = 2055,
        Dlg_BarthsMoonPlant_1043 = 2056,
        Dlg_BarthsMoonPlant_1044 = 2057,
        Dlg_BarthsMoonPlant_1045 = 2058,
        Dlg_BarthsMoonPlant_1046 = 2059,
        Dlg_BarthsMoonPlant_1047 = 2060,
        Dlg_BarthsMoonPlant_1048 = 2061,
        Dlg_BarthsMoonPlant_1049 = 2062,
        Dlg_BarthsMoonPlant_1050 = 2063,
        Dlg_BarthsMoonPlant_1051 = 2064,
        Dlg_BarthsMoonPlant_1052 = 2065,
        Dlg_BarthsMoonPlant_1053 = 2066,
        Dlg_BarthsMoonPlant_1054 = 2067,
        Dlg_BarthsMoonPlant_1055 = 2068,
        Dlg_BarthsMoonPlant_1056 = 2069,
        Dlg_BarthsMoonPlant_1057 = 2070,
        Dlg_BarthsMoonPlant_1058 = 2071,
        Dlg_BarthsMoonPlant_1059 = 2072,
        Dlg_BarthsMoonPlant_1060 = 2073,
        Dlg_BarthsMoonPlant_1061 = 2074,
        Dlg_BarthsMoonPlant_1062 = 2075,
        Dlg_BarthsMoonPlant_1063 = 2076,
        Dlg_BarthsMoonPlant_1064 = 2077,
        Dlg_BarthsMoonPlant_1065 = 2078,
        Dlg_BarthsMoonPlant_1066 = 2079,
        Dlg_BarthsMoonPlant_1067 = 2080,
        Dlg_BarthsMoonPlant_1068 = 2081,
        Dlg_BarthsMoonPlant_1069 = 2082,
        Dlg_BarthsMoonPlant_1070 = 2083,
        Dlg_BarthsMoonPlant_1071 = 2084,
        Dlg_BarthsMoonPlant_1072 = 2085,
        Dlg_BarthsMoonPlant_1073 = 2086,
        Dlg_BarthsMoonPlant_1074 = 2087,
        Dlg_BarthsMoonPlant_1075 = 2088,
        Dlg_BarthsMoonPlant_1076 = 2089,
        Dlg_BarthsMoonPlant_1077 = 2090,
        Dlg_BarthsMoonPlant_1078 = 2091,
        Dlg_BarthsMoonPlant_1079 = 2092,
        Dlg_BarthsMoonPlant_1080 = 2093,
        Dlg_BarthsMoonPlant_1081 = 2094,
        Dlg_BarthsMoonPlant_1082 = 2095,
        Dlg_BarthsMoonPlant_1083 = 2096,
        Dlg_BarthsMoonPlant_1084 = 2097,
        Dlg_BarthsMoonPlant_1085 = 2098,
        Dlg_BarthsMoonPlant_1086 = 2099,
        Dlg_BarthsMoonPlant_1087 = 2100,
        Dlg_BarthsMoonPlant_1088 = 2101,
        Dlg_BarthsMoonPlant_1089 = 2102,
        Dlg_BarthsMoonPlant_1090 = 2103,
        Dlg_BarthsMoonPlant_1091 = 2104,
        Dlg_BarthsMoonPlant_1092 = 2105,
        Dlg_BarthsMoonPlant_1093 = 2106,
        Dlg_BarthsMoonPlant_1094 = 2107,
        Dlg_BarthsMoonPlant_1095 = 2108,
        Dlg_BarthsMoonPlant_1096 = 2109,
        Dlg_BarthsMoonPlant_1097 = 2110,
        Dlg_BarthsMoonPlant_1098 = 2111,
        Dlg_BarthsMoonPlant_1099 = 2112,
        Dlg_BarthsMoonPlant_1100 = 2113,
        Dlg_BarthsMoonPlant_1101 = 2114,
        Dlg_BarthsMoonPlant_1102 = 2115,
        Dlg_BarthsMoonPlant_1103 = 2116,
        Dlg_BarthsMoonPlant_1104 = 2117,
        Dlg_BarthsMoonPlant_1105 = 2118,
        Dlg_BarthsMoonPlant_1106 = 2119,
        Dlg_BarthsMoonPlant_1107 = 2120,
        Dlg_BarthsMoonPlant_1108 = 2121,
        Dlg_BarthsMoonPlant_1109 = 2122,
        Dlg_BarthsMoonPlant_1110 = 2123,
        Dlg_BarthsMoonPlant_1111 = 2124,
        Dlg_BarthsMoonPlant_1112 = 2125,
        Dlg_BarthsMoonPlant_1113 = 2126,
        Dlg_BarthsMoonPlant_1114 = 2127,
        Dlg_BarthsMoonPlant_1115 = 2128,
        Dlg_BarthsMoonPlant_1116 = 2129,
        Dlg_BarthsMoonPlant_1117 = 2130,
        Dlg_BarthsMoonPlant_1118 = 2131,
        Dlg_BarthsMoonPlant_1119 = 2132,
        Dlg_BarthsMoonPlant_1120 = 2133,
        Dlg_BarthsMoonPlant_1121 = 2134,
        Dlg_BarthsMoonPlant_1122 = 2135,
        Dlg_BarthsMoonPlant_1123 = 2136,
        Dlg_BarthsMoonPlant_1124 = 2137,
        Dlg_BarthsMoonPlant_1125 = 2138,
        Dlg_BarthsMoonPlant_1126 = 2139,
        Dlg_FortValiantC_1000 = 2140,
        Dlg_FortValiantC_1001 = 2141,
        Dlg_FortValiantC_1002 = 2142,
        Dlg_FortValiantC_1003 = 2143,
        Dlg_FortValiantC_1004 = 2144,
        Dlg_FortValiantC_1005 = 2145,
        Dlg_FortValiantC_1006 = 2146,
        Dlg_FortValiantC_1007 = 2147,
        Dlg_FortValiantC_1008 = 2148,
        Dlg_FortValiantC_1009 = 2149,
        Dlg_FortValiantC_1010 = 2150,
        Dlg_FortValiantC_1011 = 2151,
        Dlg_FortValiantC_1012 = 2152,
        Dlg_FortValiantC_1013 = 2153,
        Dlg_FortValiantC_1014 = 2154,
        Dlg_FortValiantC_1015 = 2155,
        Dlg_FortValiantC_1016 = 2156,
        Dlg_FortValiantC_1017 = 2157,
        Dlg_FortValiantC_1018 = 2158,
        Dlg_FortValiantC_1019 = 2159,
        Dlg_FortValiantC_1020 = 2160,
        Dlg_FortValiantC_1021 = 2161,
        Dlg_FortValiantC_1022 = 2162,
        Dlg_FortValiantC_1023 = 2163,
        Dlg_FortValiantC_1024 = 2164,
        Dlg_FortValiantC_1025 = 2165,
        Dlg_FortValiantC_1026 = 2166,
        Dlg_FortValiantC_1027 = 2167,
        Dlg_FortValiantC_1028 = 2168,
        Dlg_FortValiantC_1029 = 2169,
        Dlg_FortValiantC_1030 = 2170,
        Dlg_FortValiantC_1031 = 2171,
        Dlg_FortValiantC_1032 = 2172,
        Dlg_FortValiantC_1033 = 2173,
        Dlg_FortValiantC_1034 = 2174,
        Dlg_FortValiantC_1035 = 2175,
        Dlg_FortValiantC_1036 = 2176,
        Dlg_FortValiantC_1037 = 2177,
        Dlg_FortValiantC_1038 = 2178,
        Dlg_FortValiantC_1039 = 2179,
        Dlg_FortValiantC_1040 = 2180,
        Dlg_FortValiantC_1041 = 2181,
        Dlg_FortValiantC_1042 = 2182,
        Dlg_FortValiantC_1043 = 2183,
        Dlg_FortValiantC_1044 = 2184,
        Dlg_FortValiantC_1045 = 2185,
        Dlg_FortValiantC_1046 = 2186,
        Dlg_FortValiantC_1047 = 2187,
        Dlg_FortValiantC_1048 = 2188,
        Dlg_FortValiantC_1049 = 2189,
        Dlg_FortValiantC_1050 = 2190,
        Dlg_FortValiantC_1051 = 2191,
        Dlg_FortValiantC_1052 = 2192,
        Dlg_FortValiantC_1053 = 2193,
        Dlg_FortValiantC_1054 = 2194,
        Dlg_FortValiantC_1055 = 2195,
        Dlg_FortValiantC_1056 = 2196,
        Dlg_FortValiantC_1057 = 2197,
        Dlg_FortValiantC_1058 = 2198,
        Dlg_FortValiantC_1059 = 2199,
        Dlg_FortValiantC_1060 = 2200,
        Dlg_FortValiantC_1061 = 2201,
        Dlg_FortValiantC_1062 = 2202,
        Dlg_FortValiantC_1063 = 2203,
        Dlg_FortValiantC_1064 = 2204,
        Dlg_FortValiantC_1065 = 2205,
        Dlg_FortValiantC_1066 = 2206,
        Dlg_FortValiantC_1067 = 2207,
        Dlg_FortValiantC_1068 = 2208,
        Dlg_FortValiantC_1069 = 2209,
        Dlg_FortValiantC_1070 = 2210,
        Dlg_FortValiantC_1071 = 2211,
        Dlg_FortValiantC_1072 = 2212,
        Dlg_FortValiantC_1073 = 2213,
        Dlg_FortValiantC_1074 = 2214,
        Dlg_FortValiantC_1075 = 2215,
        Dlg_FortValiantC_1076 = 2216,
        Dlg_FortValiantC_1077 = 2217,
        Dlg_FortValiantC_1078 = 2218,
        Dlg_FortValiantC_1079 = 2219,
        Dlg_FortValiantC_1080 = 2220,
        Dlg_FortValiantC_1081 = 2221,
        Dlg_FortValiantC_1082 = 2222,
        Dlg_FortValiantC_1083 = 2223,
        Dlg_FortValiantC_1084 = 2224,
        Dlg_FortValiantC_1085 = 2225,
        Dlg_FortValiantC_1086 = 2226,
        Dlg_FortValiantC_1087 = 2227,
        Dlg_FortValiantC_1088 = 2228,
        Dlg_FortValiantC_1089 = 2229,
        Dlg_FortValiantC_1090 = 2230,
        Dlg_FortValiantC_1091 = 2231,
        Dlg_FortValiantC_1092 = 2232,
        Dlg_FortValiantC_1093 = 2233,
        Dlg_FortValiantC_1094 = 2234,
        Dlg_FortValiantC_1095 = 2235,
        Dlg_FortValiantC_1096 = 2236,
        Dlg_FortValiantC_1097 = 2237,
        Dlg_FortValiantC_1098 = 2238,
        Dlg_FortValiantC_1099 = 2239,
        Dlg_FortValiantC_1100 = 2240,
        Dlg_FortValiantC_1101 = 2241,
        Dlg_FortValiantC_1102 = 2242,
        Dlg_FortValiantC_1103 = 2243,
        Dlg_FortValiantC_1104 = 2244,
        Dlg_FortValiantC_1105 = 2245,
        Dlg_FortValiantC_1106 = 2246,
        Dlg_FortValiantC_1107 = 2247,
        Dlg_FortValiantC_1108 = 2248,
        Dlg_FortValiantC_1109 = 2249,
        Dlg_FortValiantC_1110 = 2250,
        Dlg_FortValiantC_1111 = 2251,
        Dlg_FortValiantC_1112 = 2252,
        Dlg_FortValiantC_1113 = 2253,
        Dlg_FortValiantC_1114 = 2254,
        Dlg_FortValiantC_1115 = 2255,
        Dlg_FortValiantC_1116 = 2256,
        Dlg_FortValiantC_1117 = 2257,
        Dlg_FortValiantC_1118 = 2258,
        Dlg_FortValiantC_1119 = 2259,
        Dlg_FortValiantC_1120 = 2260,
        Dlg_FortValiantC_1121 = 2261,
        Dlg_FortValiantC_1122 = 2262,
        Dlg_FortValiantC_1123 = 2263,
        Dlg_FortValiantC_1124 = 2264,
        Dlg_FortValiantC_1125 = 2265,
        Dlg_FortValiantC_1126 = 2266,
        Dlg_FortValiantC_1127 = 2267,
        Dlg_FortValiantC_1128 = 2268,
        Dlg_FortValiantC_1129 = 2269,
        Dlg_FortValiantC_1130 = 2270,
        Dlg_FortValiantC_1131 = 2271,
        Dlg_FortValiantC_1132 = 2272,
        Dlg_FortValiantC_1133 = 2273,
        Dlg_FortValiantC_1134 = 2274,
        Dlg_FortValiantC_1135 = 2275,
        Dlg_FortValiantC_1136 = 2276,
        Dlg_FortValiantC_1137 = 2277,
        Dlg_FortValiantC_1138 = 2278,
        Dlg_AlienGate_1000 = 2279,
        Dlg_AlienGate_1001 = 2280,
        Dlg_AlienGate_1002 = 2281,
        Dlg_AlienGate_1003 = 2282,
        Dlg_AlienGate_1004 = 2283,
        Dlg_AlienGate_1005 = 2284,
        Dlg_AlienGate_1006 = 2285,
        Dlg_AlienGate_1007 = 2286,
        Dlg_AlienGate_1008 = 2287,
        Dlg_AlienGate_1009 = 2288,
        Dlg_AlienGate_1010 = 2289,
        Dlg_AlienGate_1011 = 2290,
        Dlg_AlienGate_1012 = 2291,
        Dlg_AlienGate_1013 = 2292,
        Dlg_AlienGate_1014 = 2293,
        Dlg_AlienGate_1015 = 2294,
        Dlg_AlienGate_1016 = 2295,
        Dlg_AlienGate_1017 = 2296,
        Dlg_AlienGate_1018 = 2297,
        Dlg_AlienGate_1019 = 2298,
        Dlg_AlienGate_1020 = 2299,
        Dlg_AlienGate_1021 = 2300,
        Dlg_AlienGate_1022 = 2301,
        Dlg_AlienGate_1023 = 2302,
        Dlg_AlienGate_1024 = 2303,
        Dlg_AlienGate_1025 = 2304,
        Dlg_AlienGate_1026 = 2305,
        Dlg_AlienGate_1027 = 2306,
        Dlg_AlienGate_1028 = 2307,
        Dlg_AlienGate_1029 = 2308,
        Dlg_AlienGate_1030 = 2309,
        Dlg_AlienGate_1031 = 2310,
        Dlg_AlienGate_1032 = 2311,
        Dlg_AlienGate_1033 = 2312,
        Dlg_AlienGate_1034 = 2313,
        Dlg_AlienGate_1035 = 2314,
        Dlg_AlienGate_1036 = 2315,
        Dlg_AlienGate_1037 = 2316,
        Dlg_AlienGate_1038 = 2317,
        Dlg_AlienGate_1039 = 2318,
        Dlg_AlienGate_1040 = 2319,
        Dlg_AlienGate_1041 = 2320,
        Dlg_AlienGate_1042 = 2321,
        Dlg_AlienGate_1043 = 2322,
        Dlg_AlienGate_1044 = 2323,
        Dlg_AlienGate_1045 = 2324,
        Dlg_AlienGate_1046 = 2325,
        Dlg_AlienGate_1047 = 2326,
        Dlg_AlienGate_1048 = 2327,
        Dlg_AlienGate_1049 = 2328,
        Dlg_AlienGate_1050 = 2329,
        Dlg_AlienGate_1051 = 2330,
        Dlg_AlienGate_1052 = 2331,
        Dlg_AlienGate_1053 = 2332,
        Dlg_AlienGate_1054 = 2333,
        Dlg_AlienGate_1055 = 2334,
        Dlg_AlienGate_1056 = 2335,
        Dlg_AlienGate_1057 = 2336,
        Dlg_AlienGate_1058 = 2337,
        Dlg_AlienGate_1059 = 2338,
        Dlg_AlienGate_1060 = 2339,
        Dlg_AlienGate_1061 = 2340,
        Dlg_AlienGate_1062 = 2341,
        Dlg_AlienGate_1063 = 2342,
        Dlg_AlienGate_1064 = 2343,
        Dlg_AlienGate_1065 = 2344,
        Dlg_AlienGate_1066 = 2345,
        Dlg_AlienGate_1067 = 2346,
        Dlg_AlienGate_1068 = 2347,
        Dlg_AlienGate_1069 = 2348,
        Dlg_AlienGate_1070 = 2349,
        Dlg_AlienGate_1071 = 2350,
        Dlg_AlienGate_1072 = 2351,
        Dlg_AlienGate_1073 = 2352,
        Dlg_AlienGate_1074 = 2353,
        Dlg_AlienGate_1075 = 2354,
        Dlg_AlienGate_1076 = 2355,
        Dlg_AlienGate_1077 = 2356,
        Dlg_AlienGate_1078 = 2357,
        Dlg_AlienGate_1079 = 2358,
        Dlg_AlienGate_1080 = 2359,
        Dlg_AlienGate_1081 = 2360,
        Dlg_AlienGate_1082 = 2361,
        Dlg_AlienGate_1083 = 2362,
        Dlg_AlienGate_1084 = 2363,
        Dlg_AlienGate_1085 = 2364,
        Dlg_AlienGate_1086 = 2365,
        Dlg_AlienGate_1087 = 2366,
        Dlg_AlienGate_1088 = 2367,
        Dlg_AlienGate_1089 = 2368,
        Dlg_AlienGate_1090 = 2369,
        Dlg_AlienGate_1091 = 2370,
        Dlg_AlienGate_1092 = 2371,
        Dlg_AlienGate_1093 = 2372,
        Dlg_AlienGate_1094 = 2373,
        Dlg_AlienGate_1095 = 2374,
        Dlg_AlienGate_1096 = 2375,
        Dlg_AlienGate_1097 = 2376,
        Dlg_AlienGate_1098 = 2377,
        Dlg_AlienGate_1099 = 2378,
        Dlg_AlienGate_1100 = 2379,
        Dlg_AlienGate_1101 = 2380,
        Dlg_AlienGate_1102 = 2381,
        Dlg_AlienGate_1103 = 2382,
        Dlg_AlienGate_1104 = 2383,
        Dlg_AlienGate_1105 = 2384,
        Dlg_AlienGate_1106 = 2385,
        Dlg_AlienGate_1107 = 2386,
        Dlg_AlienGate_1108 = 2387,
        Dlg_AlienGate_1109 = 2388,
        Dlg_AlienGate_1110 = 2389,
        Dlg_AlienGate_1111 = 2390,
        Dlg_AlienGate_1112 = 2391,
        Dlg_AlienGate_1113 = 2392,
        Dlg_AlienGate_1114 = 2393,
        Dlg_AlienGate_1115 = 2394,
        Dlg_AlienGate_1116 = 2395,
        Dlg_AlienGate_1117 = 2396,
        Dlg_AlienGate_1118 = 2397,
        Dlg_AlienGate_1119 = 2398,
        Dlg_AlienGate_1120 = 2399,
        Dlg_AlienGate_1121 = 2400,
        Dlg_AlienGate_1122 = 2401,
        Dlg_AlienGate_1123 = 2402,
        Dlg_AlienGate_1124 = 2403,
        Dlg_AlienGate_1125 = 2404,
        Dlg_AlienGate_1126 = 2405,
        Dlg_AlienGate_1127 = 2406,
        Dlg_AlienGate_1128 = 2407,
        Dlg_AlienGate_1129 = 2408,
        Dlg_AlienGate_1130 = 2409,
        Dlg_AlienGate_1131 = 2410,
        Dlg_AlienGate_1132 = 2411,
        Dlg_AlienGate_1133 = 2412,
        Dlg_AlienGate_1134 = 2413,
        Dlg_AlienGate_1135 = 2414,
        Dlg_AlienGate_1136 = 2415,
        Dlg_AlienGate_1137 = 2416,
        Dlg_AlienGate_1138 = 2417,
        Dlg_AlienGate_1139 = 2418,
        Dlg_AlienGate_1140 = 2419,
        Dlg_AlienGate_1141 = 2420,
        Dlg_AlienGate_1142 = 2421,
        Dlg_AlienGate_1143 = 2422,
        Dlg_AlienGate_1144 = 2423,
        Dlg_AlienGate_1145 = 2424,
        Dlg_AlienGate_1146 = 2425,
        Dlg_AlienGate_1147 = 2426,
        Dlg_AlienGate_1148 = 2427,
        Dlg_AlienGate_1149 = 2428,
        Dlg_AlienGate_1150 = 2429,
        Dlg_AlienGate_1151 = 2430,
        Dlg_AlienGate_1152 = 2431,
        Dlg_AlienGate_1153 = 2432,
        Dlg_AlienGate_1154 = 2433,
        Dlg_AlienGate_1155 = 2434,
        Dlg_AlienGate_1156 = 2435,
        Dlg_AlienGate_1157 = 2436,
        Dlg_AlienGate_1158 = 2437,
        Dlg_AlienGate_1159 = 2438,
        Dlg_AlienGate_1160 = 2439,
        Dlg_AlienGate_1161 = 2440,
        Dlg_AlienGate_1162 = 2441,
        Dlg_AlienGate_1163 = 2442,
        Dlg_AlienGate_1164 = 2443,
        Dlg_AlienGate_1165 = 2444,
        Dlg_AlienGate_1166 = 2445,
        Dlg_PirateBase_1000 = 2446,
        Dlg_PirateBase_1001 = 2447,
        Dlg_PirateBase_1002 = 2448,
        Dlg_PirateBase_1003 = 2449,
        Dlg_PirateBase_1004 = 2450,
        Dlg_PirateBase_1005 = 2451,
        Dlg_PirateBase_1006 = 2452,
        Dlg_PirateBase_1007 = 2453,
        Dlg_PirateBase_1008 = 2454,
        Dlg_PirateBase_1009 = 2455,
        Dlg_PirateBase_1010 = 2456,
        Dlg_PirateBase_1011 = 2457,
        Dlg_PirateBase_1012 = 2458,
        Dlg_PirateBase_1013 = 2459,
        Dlg_PirateBase_1014 = 2460,
        Dlg_PirateBase_1015 = 2461,
        Dlg_PirateBase_1016 = 2462,
        Dlg_PirateBase_1017 = 2463,
        Dlg_PirateBase_1018 = 2464,
        Dlg_PirateBase_1019 = 2465,
        Dlg_PirateBase_1020 = 2466,
        Dlg_PirateBase_1021 = 2467,
        Dlg_PirateBase_1022 = 2468,
        Dlg_PirateBase_1023 = 2469,
        Dlg_PirateBase_1024 = 2470,
        Dlg_PirateBase_1025 = 2471,
        Dlg_PirateBase_1026 = 2472,
        Dlg_PirateBase_1027 = 2473,
        Dlg_PirateBase_1028 = 2474,
        Dlg_PirateBase_1029 = 2475,
        Dlg_PirateBase_1030 = 2476,
        Dlg_PirateBase_1031 = 2477,
        Dlg_PirateBase_1032 = 2478,
        Dlg_PirateBase_1033 = 2479,
        Dlg_PirateBase_1034 = 2480,
        Dlg_PirateBase_1035 = 2481,
        Dlg_PirateBase_1036 = 2482,
        Dlg_PirateBase_1037 = 2483,
        Dlg_PirateBase_1038 = 2484,
        Dlg_PirateBase_1039 = 2485,
        Dlg_PirateBase_1040 = 2486,
        Dlg_PirateBase_1041 = 2487,
        Dlg_PirateBase_1042 = 2488,
        Dlg_PirateBase_1043 = 2489,
        Dlg_PirateBase_1044 = 2490,
        Dlg_PirateBase_1045 = 2491,
        Dlg_PirateBase_1046 = 2492,
        Dlg_PirateBase_1047 = 2493,
        Dlg_PirateBase_1048 = 2494,
        Dlg_PirateBase_1049 = 2495,
        Dlg_PirateBase_1050 = 2496,
        Dlg_PirateBase_1051 = 2497,
        Dlg_PirateBase_1052 = 2498,
        Dlg_PirateBase_1053 = 2499,
        Dlg_PirateBase_1054 = 2500,
        Dlg_PirateBase_1055 = 2501,
        Dlg_PirateBase_1056 = 2502,
        Dlg_PirateBase_1057 = 2503,
        Dlg_PirateBase_1058 = 2504,
        Dlg_PirateBase_1059 = 2505,
        Dlg_PirateBase_1060 = 2506,
        Dlg_PirateBase_1061 = 2507,
        Dlg_PirateBase_1062 = 2508,
        Dlg_PirateBase_1063 = 2509,
        Dlg_PirateBase_1064 = 2510,
        Dlg_PirateBase_1065 = 2511,
        Dlg_PirateBase_1066 = 2512,
        Dlg_PirateBase_1067 = 2513,
        Dlg_PirateBase_1068 = 2514,
        Dlg_PirateBase_1069 = 2515,
        Dlg_PirateBase_1070 = 2516,
        Dlg_PirateBase_1071 = 2517,
        Dlg_PirateBase_1072 = 2518,
        Dlg_PirateBase_1073 = 2519,
        Dlg_PirateBase_1074 = 2520,
        Dlg_PirateBase_1075 = 2521,
        Dlg_PirateBase_1076 = 2522,
        Dlg_PirateBase_1077 = 2523,
        Dlg_PirateBase_1078 = 2524,
        Dlg_PirateBase_1079 = 2525,
        Dlg_PirateBase_1080 = 2526,
        Dlg_PirateBase_1081 = 2527,
        Dlg_PirateBase_1082 = 2528,
        Dlg_PirateBase_1083 = 2529,
        Dlg_PirateBase_1084 = 2530,
        Dlg_PirateBase_1085 = 2531,
        Dlg_PirateBase_1086 = 2532,
        Dlg_PirateBase_1087 = 2533,
        Dlg_PirateBase_1088 = 2534,
        Dlg_PirateBase_1089 = 2535,
        Dlg_PirateBase_1090 = 2536,
        Dlg_PirateBase_1091 = 2537,
        Dlg_PirateBase_1092 = 2538,
        Dlg_PirateBase_1093 = 2539,
        Dlg_PirateBase_1094 = 2540,
        Dlg_PirateBase_1095 = 2541,
        Dlg_PirateBase_1096 = 2542,
        Dlg_PirateBase_1097 = 2543,
        Dlg_PirateBase_1098 = 2544,
        Dlg_PirateBase_1099 = 2545,
        Dlg_PirateBase_1100 = 2546,
        Dlg_PirateBase_1101 = 2547,
        Dlg_PirateBase_1102 = 2548,
        Dlg_PirateBase_1103 = 2549,
        Dlg_PirateBase_1104 = 2550,
        Dlg_PirateBase_1105 = 2551,
        Dlg_PirateBase_1106 = 2552,
        Dlg_PirateBase_1107 = 2553,
        Dlg_PirateBase_1108 = 2554,
        Dlg_PirateBase_1109 = 2555,
        Dlg_PirateBase_1110 = 2556,
        Dlg_PirateBase_1111 = 2557,
        Dlg_PirateBase_1112 = 2558,
        Dlg_PirateBase_1113 = 2559,
        Dlg_PirateBase_1114 = 2560,
        Dlg_PirateBase_1115 = 2561,
        Dlg_PirateBase_1116 = 2562,
        Dlg_PirateBase_1117 = 2563,
        Dlg_PirateBase_1118 = 2564,
        Dlg_PirateBase_1119 = 2565,
        Dlg_PirateBase_1120 = 2566,
        Dlg_PirateBase_1121 = 2567,
        Dlg_PirateBase_1122 = 2568,
        Dlg_PirateBase_1123 = 2569,
        Dlg_PirateBase_1124 = 2570,
        Dlg_PirateBase_1125 = 2571,
        Dlg_PirateBase_1126 = 2572,
        Dlg_PirateBase_1127 = 2573,
        Dlg_PirateBase_1128 = 2574,
        Dlg_PirateBase_1129 = 2575,
        Dlg_PirateBase_1130 = 2576,
        Dlg_PirateBase_1131 = 2577,
        Dlg_PirateBase_1132 = 2578,
        Dlg_PirateBase_1133 = 2579,
        Dlg_PirateBase_1134 = 2580,
        Dlg_PirateBase_1135 = 2581,
        Dlg_PirateBase_1136 = 2582,
        Dlg_PirateBase_1137 = 2583,
        Dlg_PirateBase_1138 = 2584,
        Dlg_PirateBase_1139 = 2585,
        Dlg_PirateBase_1140 = 2586,
        Dlg_PirateBase_1141 = 2587,
        Dlg_PirateBase_1142 = 2588,
        Dlg_PirateBase_1143 = 2589,
        Dlg_PirateBase_1144 = 2590,
        Dlg_PirateBase_1145 = 2591,
        Dlg_PirateBase_1146 = 2592,
        Dlg_PirateBase_1147 = 2593,
        Dlg_PirateBase_1148 = 2594,
        Dlg_PirateBase_1149 = 2595,
        Dlg_PirateBase_1150 = 2596,
        Dlg_PirateBase_1151 = 2597,
        Dlg_PirateBase_1152 = 2598,
        Dlg_PirateBase_1153 = 2599,
        Dlg_PirateBase_1154 = 2600,
        Dlg_PirateBase_1155 = 2601,
        Dlg_PirateBase_1156 = 2602,
        Dlg_PirateBase_1157 = 2603,
        Dlg_PirateBase_1158 = 2604,
        Dlg_PirateBase_1159 = 2605,
        Dlg_PirateBase_1160 = 2606,
        Dlg_BarthsMoonConvince_1000 = 2607,
        Dlg_BarthsMoonConvince_1001 = 2608,
        Dlg_BarthsMoonConvince_1002 = 2609,
        Dlg_BarthsMoonConvince_1003 = 2610,
        Dlg_BarthsMoonConvince_1004 = 2611,
        Dlg_BarthsMoonConvince_1005 = 2612,
        Dlg_BarthsMoonConvince_1006 = 2613,
        Dlg_BarthsMoonConvince_1007 = 2614,
        Dlg_BarthsMoonConvince_1008 = 2615,
        Dlg_BarthsMoonConvince_1009 = 2616,
        Dlg_BarthsMoonConvince_1010 = 2617,
        Dlg_BarthsMoonConvince_1011 = 2618,
        Dlg_BarthsMoonConvince_1012 = 2619,
        Dlg_BarthsMoonConvince_1013 = 2620,
        Dlg_BarthsMoonConvince_1014 = 2621,
        Dlg_BarthsMoonConvince_1015 = 2622,
        Dlg_BarthsMoonConvince_1016 = 2623,
        Dlg_BarthsMoonConvince_1017 = 2624,
        Dlg_BarthsMoonConvince_1018 = 2625,
        Dlg_BarthsMoonConvince_1019 = 2626,
        Dlg_BarthsMoonConvince_1020 = 2627,
        Dlg_BarthsMoonConvince_1021 = 2628,
        Dlg_BarthsMoonConvince_1022 = 2629,
        Dlg_BarthsMoonConvince_1023 = 2630,
        Dlg_BarthsMoonConvince_1024 = 2631,
        Dlg_BarthsMoonConvince_1025 = 2632,
        Dlg_BarthsMoonConvince_1026 = 2633,
        Dlg_BarthsMoonConvince_1027 = 2634,
        Dlg_BarthsMoonConvince_1028 = 2635,
        Dlg_BarthsMoonConvince_1029 = 2636,
        Dlg_BarthsMoonConvince_1030 = 2637,
        Dlg_BarthsMoonConvince_1031 = 2638,
        Dlg_BarthsMoonConvince_1032 = 2639,
        Dlg_BarthsMoonConvince_1033 = 2640,
        Dlg_BarthsMoonConvince_1034 = 2641,
        Dlg_BarthsMoonConvince_1035 = 2642,
        Dlg_BarthsMoonConvince_1036 = 2643,
        Dlg_BarthsMoonConvince_1037 = 2644,
        Dlg_BarthsMoonConvince_1038 = 2645,
        Dlg_BarthsMoonConvince_1039 = 2646,
        Dlg_BarthsMoonConvince_1040 = 2647,
        Dlg_BarthsMoonConvince_1041 = 2648,
        Dlg_BarthsMoonConvince_1042 = 2649,
        Dlg_BarthsMoonConvince_1043 = 2650,
        Dlg_BarthsMoonConvince_1044 = 2651,
        Dlg_BarthsMoonConvince_1045 = 2652,
        Dlg_BarthsMoonConvince_1046 = 2653,
        Dlg_BarthsMoonConvince_1047 = 2654,
        Dlg_BarthsMoonConvince_1048 = 2655,
        Dlg_BarthsMoonConvince_1049 = 2656,
        Dlg_BarthsMoonConvince_1050 = 2657,
        Dlg_BarthsMoonConvince_1051 = 2658,
        Dlg_BarthsMoonConvince_1052 = 2659,
        Dlg_BarthsMoonConvince_1053 = 2660,
        Dlg_BarthsMoonConvince_1054 = 2661,
        Dlg_BarthsMoonConvince_1055 = 2662,
        Dlg_BarthsMoonConvince_1056 = 2663,
        Dlg_BarthsMoonConvince_1057 = 2664,
        Dlg_BarthsMoonConvince_1058 = 2665,
        Dlg_BarthsMoonConvince_1059 = 2666,
        Dlg_BarthsMoonConvince_1060 = 2667,
        Dlg_BarthsMoonConvince_1061 = 2668,
        Dlg_BarthsMoonConvince_1062 = 2669,
        Dlg_BarthsMoonConvince_1063 = 2670,
        Dlg_BarthsMoonConvince_1064 = 2671,
        Dlg_BarthsMoonConvince_1065 = 2672,
        Dlg_BarthsMoonConvince_1066 = 2673,
        Dlg_BarthsMoonConvince_1067 = 2674,
        Dlg_BarthsMoonConvince_1068 = 2675,
        Dlg_BarthsMoonConvince_1069 = 2676,
        Dlg_BarthsMoonConvince_1070 = 2677,
        Dlg_BarthsMoonConvince_1071 = 2678,
        Dlg_BarthsMoonConvince_1072 = 2679,
        Dlg_BarthsMoonConvince_1073 = 2680,
        Dlg_BarthsMoonConvince_1074 = 2681,
        Dlg_BarthsMoonConvince_1075 = 2682,
        Dlg_BarthsMoonConvince_1076 = 2683,
        Dlg_BarthsMoonConvince_1077 = 2684,
        Dlg_BarthsMoonConvince_1078 = 2685,
        Dlg_BarthsMoonConvince_1079 = 2686,
        Dlg_BarthsMoonConvince_1080 = 2687,
        Dlg_BarthsMoonConvince_1081 = 2688,
        Dlg_BarthsMoonConvince_1082 = 2689,
        Dlg_BarthsMoonConvince_1083 = 2690,
        Dlg_BarthsMoonConvince_1084 = 2691,
        Dlg_BarthsMoonConvince_1085 = 2692,
        Dlg_BarthsMoonConvince_1086 = 2693,
        Dlg_BarthsMoonConvince_1087 = 2694,
        Dlg_BarthsMoonConvince_1088 = 2695,
        Dlg_BarthsMoonConvince_1089 = 2696,
        Dlg_BarthsMoonConvince_1090 = 2697,
        Dlg_BarthsMoonConvince_1091 = 2698,
        Dlg_BarthsMoonConvince_1092 = 2699,
        Dlg_BarthsMoonConvince_1093 = 2700,
        Dlg_BarthsMoonConvince_1094 = 2701,
        Dlg_BarthsMoonConvince_1095 = 2702,
        Dlg_BarthsMoonConvince_1096 = 2703,
        Dlg_BarthsMoonConvince_1097 = 2704,
        Dlg_BarthsMoonConvince_1098 = 2705,
        Dlg_BarthsMoonConvince_1099 = 2706,
        Dlg_BarthsMoonConvince_1100 = 2707,
        Dlg_BarthsMoonConvince_1101 = 2708,
        Dlg_BarthsMoonConvince_1102 = 2709,
        Dlg_BarthsMoonConvince_1103 = 2710,
        Dlg_BarthsMoonConvince_1104 = 2711,
        Dlg_BarthsMoonConvince_1105 = 2712,
        Dlg_BarthsMoonConvince_1106 = 2713,
        Dlg_BarthsMoonConvince_1107 = 2714,
        Dlg_BarthsMoonConvince_1108 = 2715,
        Dlg_BarthsMoonConvince_1109 = 2716,
        Dlg_BarthsMoonConvince_1110 = 2717,
        Dlg_BarthsMoonConvince_1111 = 2718,
        Dlg_JunkyardConvince_1000 = 2719,
        Dlg_JunkyardConvince_1001 = 2720,
        Dlg_JunkyardConvince_1002 = 2721,
        Dlg_JunkyardConvince_1003 = 2722,
        Dlg_JunkyardConvince_1004 = 2723,
        Dlg_JunkyardConvince_1005 = 2724,
        Dlg_JunkyardConvince_1006 = 2725,
        Dlg_JunkyardConvince_1007 = 2726,
        Dlg_JunkyardConvince_1008 = 2727,
        Dlg_JunkyardConvince_1009 = 2728,
        Dlg_JunkyardConvince_1010 = 2729,
        Dlg_JunkyardConvince_1011 = 2730,
        Dlg_JunkyardConvince_1012 = 2731,
        Dlg_JunkyardConvince_1013 = 2732,
        Dlg_JunkyardConvince_1014 = 2733,
        Dlg_JunkyardConvince_1015 = 2734,
        Dlg_JunkyardConvince_1016 = 2735,
        Dlg_JunkyardConvince_1017 = 2736,
        Dlg_JunkyardConvince_1018 = 2737,
        Dlg_JunkyardConvince_1019 = 2738,
        Dlg_JunkyardConvince_1020 = 2739,
        Dlg_JunkyardConvince_1021 = 2740,
        Dlg_JunkyardConvince_1022 = 2741,
        Dlg_JunkyardConvince_1023 = 2742,
        Dlg_JunkyardConvince_1024 = 2743,
        Dlg_JunkyardConvince_1025 = 2744,
        Dlg_JunkyardConvince_1026 = 2745,
        Dlg_JunkyardConvince_1027 = 2746,
        Dlg_JunkyardConvince_1028 = 2747,
        Dlg_JunkyardConvince_1029 = 2748,
        Dlg_JunkyardConvince_1030 = 2749,
        Dlg_JunkyardConvince_1031 = 2750,
        Dlg_JunkyardConvince_1032 = 2751,
        Dlg_JunkyardConvince_1034 = 2752,
        Dlg_JunkyardConvince_1035 = 2753,
        Dlg_JunkyardConvince_1036 = 2754,
        Dlg_JunkyardConvince_1037 = 2755,
        Dlg_JunkyardConvince_1038 = 2756,
        Dlg_JunkyardConvince_1039 = 2757,
        Dlg_JunkyardConvince_1040 = 2758,
        Dlg_JunkyardConvince_1041 = 2759,
        Dlg_JunkyardConvince_1042 = 2760,
        Dlg_JunkyardConvince_1043 = 2761,
        Dlg_JunkyardConvince_1044 = 2762,
        Dlg_JunkyardConvince_1045 = 2763,
        Dlg_JunkyardConvince_1046 = 2764,
        Dlg_JunkyardConvince_1047 = 2765,
        Dlg_JunkyardConvince_1048 = 2766,
        Dlg_JunkyardConvince_1049 = 2767,
        Dlg_JunkyardConvince_1050 = 2768,
        Dlg_JunkyardConvince_1051 = 2769,
        Dlg_JunkyardConvince_1052 = 2770,
        Dlg_JunkyardConvince_1053 = 2771,
        Dlg_JunkyardConvince_1054 = 2772,
        Dlg_JunkyardConvince_1055 = 2773,
        Dlg_JunkyardConvince_1056 = 2774,
        Dlg_JunkyardConvince_1057 = 2775,
        Dlg_JunkyardConvince_1058 = 2776,
        Dlg_JunkyardConvince_1059 = 2777,
        Dlg_JunkyardConvince_1060 = 2778,
        Dlg_JunkyardConvince_1061 = 2779,
        Dlg_JunkyardConvince_1062 = 2780,
        Dlg_JunkyardConvince_1063 = 2781,
        Dlg_JunkyardConvince_1064 = 2782,
        Dlg_JunkyardConvince_1065 = 2783,
        Dlg_JunkyardConvince_1066 = 2784,
        Dlg_JunkyardConvince_1067 = 2785,
        Dlg_JunkyardConvince_1068 = 2786,
        Dlg_JunkyardConvince_1069 = 2787,
        Dlg_JunkyardConvince_1070 = 2788,
        Dlg_JunkyardConvince_1071 = 2789,
        Dlg_JunkyardConvince_1072 = 2790,
        Dlg_JunkyardConvince_1073 = 2791,
        Dlg_JunkyardConvince_1074 = 2792,
        Dlg_JunkyardConvince_1075 = 2793,
        Dlg_JunkyardConvince_1076 = 2794,
        Dlg_JunkyardConvince_1077 = 2795,
        Dlg_JunkyardConvince_1078 = 2796,
        Dlg_JunkyardConvince_1079 = 2797,
        Dlg_JunkyardConvince_1080 = 2798,
        Dlg_JunkyardConvince_1081 = 2799,
        Dlg_JunkyardConvince_1082 = 2800,
        Dlg_JunkyardConvince_1083 = 2801,
        Dlg_JunkyardConvince_1084 = 2802,
        Dlg_JunkyardConvince_1085 = 2803,
        Dlg_JunkyardConvince_1086 = 2804,
        Dlg_JunkyardConvince_1087 = 2805,
        Dlg_JunkyardConvince_1088 = 2806,
        Dlg_JunkyardConvince_1089 = 2807,
        Dlg_JunkyardConvince_1090 = 2808,
        Dlg_JunkyardConvince_1091 = 2809,
        Dlg_JunkyardConvince_1092 = 2810,
        Dlg_JunkyardConvince_1093 = 2811,
        Dlg_JunkyardConvince_1094 = 2812,
        Dlg_JunkyardConvince_1095 = 2813,
        Dlg_JunkyardConvince_1096 = 2814,
        Dlg_JunkyardConvince_1097 = 2815,
        Dlg_JunkyardConvince_1098 = 2816,
        Dlg_JunkyardConvince_1099 = 2817,
        Dlg_JunkyardConvince_1100 = 2818,
        Dlg_JunkyardConvince_1101 = 2819,
        Dlg_JunkyardConvince_1102 = 2820,
        Dlg_JunkyardConvince_1103 = 2821,
        Dlg_JunkyardConvince_1104 = 2822,
        Dlg_JunkyardConvince_1105 = 2823,
        Dlg_JunkyardConvince_1106 = 2824,
        Dlg_JunkyardConvince_1107 = 2825,
        Dlg_JunkyardConvince_1108 = 2826,
        Dlg_JunkyardConvince_1109 = 2827,
        Dlg_JunkyardConvince_1110 = 2828,
        Dlg_JunkyardConvince_1111 = 2829,
        Dlg_JunkyardConvince_1112 = 2830,
        Dlg_JunkyardConvince_1113 = 2831,
        Dlg_JunkyardConvince_1114 = 2832,
        Dlg_JunkyardConvince_1115 = 2833,
        Dlg_JunkyardConvince_1116 = 2834,
        Dlg_JunkyardConvince_1117 = 2835,
        Dlg_JunkyardConvince_1118 = 2836,
        Dlg_JunkyardConvince_1119 = 2837,
        Dlg_JunkyardConvince_1120 = 2838,
        Dlg_JunkyardConvince_1121 = 2839,
        Dlg_JunkyardConvince_1122 = 2840,
        Dlg_JunkyardConvince_1123 = 2841,
        Dlg_JunkyardConvince_1124 = 2842,
        Dlg_JunkyardConvince_1125 = 2843,
        Dlg_JunkyardConvince_1126 = 2844,
        Dlg_JunkyardConvince_1127 = 2845,
        Dlg_JunkyardConvince_1128 = 2846,
        Dlg_JunkyardConvince_1129 = 2847,
        Dlg_JunkyardConvince_1130 = 2848,
        Dlg_JunkyardConvince_1131 = 2849,
        Dlg_JunkyardConvince_1132 = 2850,
        Dlg_JunkyardConvince_1133 = 2851,
        Dlg_JunkyardConvince_1134 = 2852,
        Dlg_JunkyardConvince_1135 = 2853,
        Dlg_JunkyardConvince_1136 = 2854,
        Dlg_JunkyardConvince_1137 = 2855,
        Dlg_JunkyardConvince_1138 = 2856,
        Dlg_JunkyardConvince_1139 = 2857,
        Dlg_JunkyardConvince_1140 = 2858,
        Dlg_JunkyardConvince_1141 = 2859,
        Dlg_JunkyardConvince_1142 = 2860,
        Dlg_JunkyardConvince_1143 = 2861,
        Dlg_JunkyardConvince_1144 = 2862,
        Dlg_JunkyardConvince_1145 = 2863,
        Dlg_JunkyardConvince_1146 = 2864,
        Dlg_JunkyardConvince_1147 = 2865,
        Dlg_JunkyardConvince_1148 = 2866,
        Dlg_JunkyardConvince_1149 = 2867,
        Dlg_JunkyardConvince_1150 = 2868,
        Dlg_JunkyardReturn_1000 = 2869,
        Dlg_JunkyardReturn_1001 = 2870,
        Dlg_JunkyardReturn_1002 = 2871,
        Dlg_JunkyardReturn_1003 = 2872,
        Dlg_JunkyardReturn_1004 = 2873,
        Dlg_JunkyardReturn_1005 = 2874,
        Dlg_JunkyardReturn_1006 = 2875,
        Dlg_JunkyardReturn_1007 = 2876,
        Dlg_JunkyardReturn_1008 = 2877,
        Dlg_JunkyardReturn_1009 = 2878,
        Dlg_JunkyardReturn_1010 = 2879,
        Dlg_JunkyardReturn_1011 = 2880,
        Dlg_JunkyardReturn_1012 = 2881,
        Dlg_JunkyardReturn_1013 = 2882,
        Dlg_JunkyardReturn_1014 = 2883,
        Dlg_JunkyardReturn_1015 = 2884,
        Dlg_JunkyardReturn_1016 = 2885,
        Dlg_JunkyardReturn_1017 = 2886,
        Dlg_JunkyardReturn_1018 = 2887,
        Dlg_JunkyardReturn_1019 = 2888,
        Dlg_JunkyardReturn_1020 = 2889,
        Dlg_JunkyardReturn_1021 = 2890,
        Dlg_JunkyardReturn_1022 = 2891,
        Dlg_JunkyardReturn_1023 = 2892,
        Dlg_JunkyardReturn_1024 = 2893,
        Dlg_JunkyardReturn_1025 = 2894,
        Dlg_JunkyardReturn_1026 = 2895,
        Dlg_JunkyardReturn_1027 = 2896,
        Amb3D_ThunderClapSmall = 2897,
        Amb3D_Welding01 = 2898,
        Dlg_ChineseRafinery_1040 = 2899,
        Dlg_ChineseRafinery_1041 = 2900,
        Dlg_ChineseTransmitter_1000 = 2901,
        Dlg_ChineseTransmitter_1001 = 2902,
        Dlg_ChineseTransmitter_1002 = 2903,
        Dlg_ChineseTransmitter_1003 = 2904,
        Dlg_ChineseTransmitter_1045 = 2905,
        Dlg_ChineseTransmitter_1046 = 2906,
        Dlg_JunkyardReturn_1038 = 2907,
        Dlg_JunkyardReturn_1039 = 2908,
        Dlg_JunkyardReturn_1040 = 2909,
        Dlg_JunkyardReturn_1041 = 2910,
        Dlg_JunkyardReturn_1042 = 2911,
        Dlg_JunkyardReturn_1043 = 2912,
        Dlg_JunkyardReturn_1044 = 2913,
        Dlg_JunkyardReturn_1045 = 2914,
        Dlg_JunkyardReturn_1046 = 2915,
        Dlg_JunkyardReturn_1047 = 2916,
        Dlg_JunkyardReturn_1048 = 2917,
        Dlg_JunkyardReturn_1049 = 2918,
        Dlg_JunkyardReturn_1050 = 2919,
        Dlg_JunkyardReturn_1051 = 2920,
        Dlg_JunkyardReturn_1052 = 2921,
        Dlg_JunkyardReturn_1053 = 2922,
        Dlg_JunkyardReturn_1054 = 2923,
        Dlg_JunkyardReturn_1055 = 2924,
        Dlg_JunkyardReturn_1056 = 2925,
        Dlg_JunkyardReturn_1057 = 2926,
        Dlg_JunkyardReturn_1058 = 2927,
        Dlg_JunkyardReturn_1059 = 2928,
        Dlg_JunkyardReturn_1060 = 2929,
        Dlg_JunkyardReturn_1061 = 2930,
        Dlg_JunkyardReturn_1062 = 2931,
        Dlg_JunkyardReturn_1063 = 2932,
        Dlg_JunkyardReturn_1064 = 2933,
        Dlg_JunkyardReturn_1065 = 2934,
        Dlg_JunkyardReturn_1066 = 2935,
        Dlg_JunkyardReturn_1067 = 2936,
        Dlg_JunkyardReturn_1068 = 2937,
        Dlg_JunkyardReturn_1069 = 2938,
        Dlg_JunkyardReturn_1070 = 2939,
        Dlg_JunkyardReturn_1071 = 2940,
        Dlg_JunkyardReturn_1072 = 2941,
        Dlg_JunkyardReturn_1073 = 2942,
        Dlg_JunkyardReturn_1074 = 2943,
        Dlg_JunkyardReturn_1075 = 2944,
        Dlg_JunkyardReturn_1076 = 2945,
        Dlg_JunkyardReturn_1077 = 2946,
        Dlg_JunkyardReturn_1078 = 2947,
        Dlg_JunkyardReturn_1079 = 2948,
        Dlg_JunkyardReturn_1080 = 2949,
        Dlg_JunkyardReturn_1081 = 2950,
        Dlg_JunkyardReturn_1082 = 2951,
        Dlg_JunkyardReturn_1083 = 2952,
        Dlg_JunkyardReturn_1084 = 2953,
        Dlg_JunkyardReturn_1085 = 2954,
        Dlg_JunkyardReturn_1086 = 2955,
        Dlg_JunkyardReturn_1087 = 2956,
        Dlg_JunkyardReturn_1088 = 2957,
        Dlg_JunkyardReturn_1089 = 2958,
        Dlg_JunkyardReturn_1090 = 2959,
        Dlg_LastHope_1000 = 2960,
        Dlg_LastHope_1001 = 2961,
        Dlg_LastHope_1002 = 2962,
        Dlg_LastHope_1003 = 2963,
        Dlg_LastHope_1004 = 2964,
        Dlg_LastHope_1005 = 2965,
        Dlg_LastHope_1006 = 2966,
        Dlg_LastHope_1007 = 2967,
        Dlg_LastHope_1008 = 2968,
        Dlg_LastHope_1009 = 2969,
        Dlg_LastHope_1010 = 2970,
        Dlg_LastHope_1011 = 2971,
        Dlg_LastHope_1012 = 2972,
        Dlg_LastHope_1013 = 2973,
        Dlg_LastHope_1014 = 2974,
        Dlg_LastHope_1015 = 2975,
        Dlg_LastHope_1016 = 2976,
        Dlg_LastHope_1017 = 2977,
        Dlg_LastHope_1018 = 2978,
        Dlg_LastHope_1019 = 2979,
        Dlg_LastHope_1020 = 2980,
        Dlg_LastHope_1021 = 2981,
        Dlg_LastHope_1022 = 2982,
        Dlg_LastHope_1023 = 2983,
        Dlg_LastHope_1024 = 2984,
        Dlg_LastHope_1025 = 2985,
        Dlg_LastHope_1026 = 2986,
        Dlg_LastHope_1027 = 2987,
        Dlg_LastHope_1028 = 2988,
        Dlg_LastHope_1029 = 2989,
        Dlg_LastHope_1030 = 2990,
        Dlg_LastHope_1031 = 2991,
        Dlg_LastHope_1032 = 2992,
        Dlg_LastHope_1033 = 2993,
        Dlg_LastHope_1034 = 2994,
        Dlg_LastHope_1035 = 2995,
        Dlg_LastHope_1036 = 2996,
        Dlg_LastHope_1037 = 2997,
        Dlg_LastHope_1038 = 2998,
        Dlg_LastHope_1039 = 2999,
        Dlg_LastHope_1040 = 3000,
        Dlg_LastHope_1041 = 3001,
        Dlg_LastHope_1042 = 3002,
        Dlg_LastHope_1043 = 3003,
        Dlg_LastHope_1044 = 3004,
        Dlg_LastHope_1045 = 3005,
        Dlg_LastHope_1046 = 3006,
        Dlg_LastHope_1047 = 3007,
        Dlg_LastHope_1048 = 3008,
        Dlg_LastHope_1049 = 3009,
        Dlg_LastHope_1050 = 3010,
        Dlg_LastHope_1051 = 3011,
        Dlg_LastHope_1052 = 3012,
        Dlg_LastHope_1053 = 3013,
        Dlg_LastHope_1054 = 3014,
        Dlg_LastHope_1055 = 3015,
        Dlg_LastHope_1056 = 3016,
        Dlg_LastHope_1057 = 3017,
        Dlg_LastHope_1058 = 3018,
        Dlg_LastHope_1059 = 3019,
        Dlg_LastHope_1060 = 3020,
        Dlg_LastHope_1061 = 3021,
        Dlg_LastHope_1062 = 3022,
        Dlg_LastHope_1063 = 3023,
        Dlg_LastHope_1064 = 3024,
        Dlg_LastHope_1065 = 3025,
        Dlg_LastHope_1066 = 3026,
        Dlg_LastHope_1067 = 3027,
        Dlg_LastHope_1068 = 3028,
        Dlg_LastHope_1069 = 3029,
        Dlg_LastHope_1070 = 3030,
        Dlg_LastHope_1071 = 3031,
        Dlg_LastHope_1072 = 3032,
        Dlg_LastHope_1073 = 3033,
        Dlg_LastHope_1074 = 3034,
        Dlg_LastHope_1075 = 3035,
        Dlg_LastHope_1076 = 3036,
        Dlg_LastHope_1077 = 3037,
        Dlg_LastHope_1082 = 3038,
        Dlg_LastHope_1083 = 3039,
        Dlg_LastHope_1088 = 3040,
        Dlg_LastHope_1089 = 3041,
        Dlg_LastHope_1090 = 3042,
        Dlg_LastHope_1091 = 3043,
        Dlg_LastHope_1092 = 3044,
        Dlg_LastHope_1093 = 3045,
        Dlg_LastHope_1094 = 3046,
        Dlg_LastHope_1095 = 3047,
        Dlg_LastHope_1096 = 3048,
        Dlg_LastHope_1097 = 3049,
        Dlg_LastHope_1098 = 3050,
        Dlg_LastHope_1099 = 3051,
        Dlg_LastHope_1100 = 3052,
        Dlg_LastHope_1101 = 3053,
        Dlg_LastHope_1102 = 3054,
        Dlg_LastHope_1103 = 3055,
        Dlg_LastHope_1104 = 3056,
        Dlg_LastHope_1105 = 3057,
        Dlg_LastHope_1106 = 3058,
        Dlg_LastHope_1107 = 3059,
        Dlg_LastHope_1108 = 3060,
        Dlg_LastHope_1109 = 3061,
        Dlg_LastHope_1110 = 3062,
        Dlg_LastHope_1111 = 3063,
        Dlg_LastHope_1112 = 3064,
        Dlg_LastHope_1113 = 3065,
        Dlg_LastHope_1114 = 3066,
        Dlg_LastHope_1115 = 3067,
        Dlg_LastHope_1116 = 3068,
        Dlg_LastHope_1117 = 3069,
        Dlg_LastHope_1118 = 3070,
        Dlg_LastHope_1119 = 3071,
        Dlg_LastHope_1120 = 3072,
        Dlg_LastHope_1121 = 3073,
        Dlg_LastHope_1122 = 3074,
        Dlg_LastHope_1123 = 3075,
        Dlg_LastHope_1124 = 3076,
        Dlg_LastHope_1125 = 3077,
        Dlg_LastHope_1126 = 3078,
        Dlg_LastHope_1127 = 3079,
        Dlg_LastHope_1128 = 3080,
        Dlg_LastHope_1129 = 3081,
        Dlg_LastHope_1130 = 3082,
        Dlg_LastHope_1131 = 3083,
        Dlg_LastHope_1132 = 3084,
        Dlg_LastHope_1133 = 3085,
        Dlg_LastHope_1134 = 3086,
        Dlg_RussianWarehouse_1000 = 3087,
        Dlg_RussianWarehouse_1001 = 3088,
        Dlg_RussianWarehouse_1002 = 3089,
        Dlg_RussianWarehouse_1003 = 3090,
        Dlg_RussianWarehouse_1004 = 3091,
        Dlg_RussianWarehouse_1005 = 3092,
        Dlg_RussianWarehouse_1006 = 3093,
        Dlg_RussianWarehouse_1007 = 3094,
        Dlg_RussianWarehouse_1008 = 3095,
        Dlg_RussianWarehouse_1009 = 3096,
        Dlg_RussianWarehouse_1010 = 3097,
        Dlg_RussianWarehouse_1011 = 3098,
        Dlg_RussianWarehouse_1012 = 3099,
        Dlg_RussianWarehouse_1013 = 3100,
        Dlg_RussianWarehouse_1014 = 3101,
        Dlg_RussianWarehouse_1015 = 3102,
        Dlg_RussianWarehouse_1016 = 3103,
        Dlg_RussianWarehouse_1017 = 3104,
        Dlg_RussianWarehouse_1018 = 3105,
        Dlg_RussianWarehouse_1019 = 3106,
        Dlg_RussianWarehouse_1020 = 3107,
        Dlg_RussianWarehouse_1021 = 3108,
        Dlg_RussianWarehouse_1022 = 3109,
        Dlg_RussianWarehouse_1023 = 3110,
        Dlg_RussianWarehouse_1024 = 3111,
        Dlg_RussianWarehouse_1025 = 3112,
        Dlg_RussianWarehouse_1026 = 3113,
        Dlg_RussianWarehouse_1027 = 3114,
        Dlg_RussianWarehouse_1028 = 3115,
        Dlg_RussianWarehouse_1029 = 3116,
        Dlg_RussianWarehouse_1030 = 3117,
        Dlg_RussianWarehouse_1031 = 3118,
        Dlg_RussianWarehouse_1032 = 3119,
        Dlg_RussianWarehouse_1033 = 3120,
        Dlg_RussianWarehouse_1034 = 3121,
        Dlg_RussianWarehouse_1035 = 3122,
        Dlg_RussianWarehouse_1036 = 3123,
        Dlg_RussianWarehouse_1037 = 3124,
        Dlg_RussianWarehouse_1038 = 3125,
        Dlg_RussianWarehouse_1039 = 3126,
        Dlg_RussianWarehouse_1040 = 3127,
        Dlg_RussianWarehouse_1041 = 3128,
        Dlg_RussianWarehouse_1042 = 3129,
        Dlg_RussianWarehouse_1043 = 3130,
        Dlg_RussianWarehouse_1044 = 3131,
        Dlg_RussianWarehouse_1045 = 3132,
        Dlg_RussianWarehouse_1046 = 3133,
        Dlg_RussianWarehouse_1047 = 3134,
        Dlg_RussianWarehouse_1048 = 3135,
        Dlg_RussianWarehouse_1049 = 3136,
        Dlg_RussianWarehouse_1050 = 3137,
        Dlg_RussianWarehouse_1051 = 3138,
        Dlg_RussianWarehouse_1052 = 3139,
        Dlg_RussianWarehouse_1053 = 3140,
        Dlg_RussianWarehouse_1054 = 3141,
        Dlg_RussianWarehouse_1055 = 3142,
        Dlg_RussianWarehouse_1056 = 3143,
        Dlg_RussianWarehouse_1057 = 3144,
        Dlg_RussianWarehouse_1058 = 3145,
        Dlg_RussianWarehouse_1059 = 3146,
        Dlg_RussianWarehouse_1060 = 3147,
        Dlg_RussianWarehouse_1061 = 3148,
        Dlg_RussianWarehouse_1062 = 3149,
        SfxVoxelCrack = 3150,
        Dlg_JunkyardEacAmbush_1000 = 3151,
        Dlg_JunkyardEacAmbush_1001 = 3152,
        Dlg_JunkyardEacAmbush_1002 = 3153,
        Dlg_JunkyardEacAmbush_1003 = 3154,
        Dlg_JunkyardEacAmbush_1004 = 3155,
        Dlg_JunkyardEacAmbush_1005 = 3156,
        Dlg_JunkyardEacAmbush_1006 = 3157,
        Dlg_JunkyardEacAmbush_1007 = 3158,
        Dlg_JunkyardEacAmbush_1008 = 3159,
        Dlg_JunkyardEacAmbush_1009 = 3160,
        Dlg_JunkyardEacAmbush_1010 = 3161,
        Dlg_JunkyardEacAmbush_1011 = 3162,
        Dlg_JunkyardEacAmbush_1012 = 3163,
        Dlg_JunkyardEacAmbush_1013 = 3164,
        Dlg_JunkyardEacAmbush_1014 = 3165,
        Dlg_JunkyardEacAmbush_1015 = 3166,
        Dlg_JunkyardEacAmbush_1016 = 3167,
        Dlg_JunkyardEacAmbush_1017 = 3168,
        Dlg_JunkyardEacAmbush_1018 = 3169,
        Dlg_JunkyardEacAmbush_1019 = 3170,
        Dlg_JunkyardEacAmbush_1020 = 3171,
        Dlg_JunkyardEacAmbush_1021 = 3172,
        Dlg_JunkyardEacAmbush_1022 = 3173,
        Dlg_JunkyardEacAmbush_1023 = 3174,
        Dlg_JunkyardEacAmbush_1024 = 3175,
        Dlg_JunkyardEacAmbush_1025 = 3176,
        Dlg_JunkyardEacAmbush_1026 = 3177,
        Dlg_JunkyardEacAmbush_1027 = 3178,
        Dlg_JunkyardEacAmbush_1028 = 3179,
        Dlg_JunkyardEacAmbush_1029 = 3180,
        Dlg_JunkyardEacAmbush_1030 = 3181,
        Dlg_JunkyardEacAmbush_1031 = 3182,
        Dlg_JunkyardEacAmbush_1032 = 3183,
        Dlg_JunkyardEacAmbush_1033 = 3184,
        Dlg_JunkyardEacAmbush_1034 = 3185,
        Dlg_JunkyardEacAmbush_1035 = 3186,
        Dlg_JunkyardEacAmbush_1036 = 3187,
        Dlg_JunkyardEacAmbush_1037 = 3188,
        Dlg_JunkyardEacAmbush_1038 = 3189,
        Dlg_JunkyardEacAmbush_1039 = 3190,
        Dlg_JunkyardEacAmbush_1040 = 3191,
        Dlg_JunkyardEacAmbush_1041 = 3192,
        Dlg_JunkyardEacAmbush_1042 = 3193,
        Dlg_JunkyardEacAmbush_1043 = 3194,
        Dlg_JunkyardEacAmbush_1044 = 3195,
        Dlg_JunkyardEacAmbush_1045 = 3196,
        Dlg_JunkyardEacAmbush_1046 = 3197,
        Dlg_JunkyardEacAmbush_1047 = 3198,
        Dlg_JunkyardEacAmbush_1048 = 3199,
        Dlg_JunkyardEacAmbush_1049 = 3200,
        Dlg_JunkyardEacAmbush_1050 = 3201,
        Dlg_JunkyardEacAmbush_1051 = 3202,
        Dlg_JunkyardEacAmbush_1052 = 3203,
        Dlg_JunkyardEacAmbush_1053 = 3204,
        Dlg_JunkyardEacAmbush_1054 = 3205,
        Dlg_JunkyardEacAmbush_1055 = 3206,
        Dlg_JunkyardEacAmbush_1056 = 3207,
        Dlg_JunkyardEacAmbush_1057 = 3208,
        Dlg_JunkyardEacAmbush_1058 = 3209,
        Dlg_JunkyardEacAmbush_1059 = 3210,
        Dlg_JunkyardEacAmbush_1060 = 3211,
        Dlg_JunkyardEacAmbush_1061 = 3212,
        Dlg_JunkyardEacAmbush_1062 = 3213,
        Dlg_JunkyardEacAmbush_1063 = 3214,
        Dlg_JunkyardEacAmbush_1064 = 3215,
        Dlg_JunkyardEacAmbush_1065 = 3216,
        Dlg_JunkyardEacAmbush_1066 = 3217,
        Dlg_JunkyardEacAmbush_1067 = 3218,
        Dlg_JunkyardEacAmbush_1068 = 3219,
        Dlg_JunkyardEacAmbush_1069 = 3220,
        Dlg_JunkyardEacAmbush_1070 = 3221,
        Dlg_JunkyardEacAmbush_1071 = 3222,
        Dlg_JunkyardEacAmbush_1072 = 3223,
        Dlg_JunkyardEacAmbush_1073 = 3224,
        Dlg_JunkyardEacAmbush_1074 = 3225,
        Dlg_JunkyardEacAmbush_1075 = 3226,
        Dlg_JunkyardEacAmbush_1076 = 3227,
        Dlg_JunkyardEacAmbush_1077 = 3228,
        Dlg_JunkyardEacAmbush_1078 = 3229,
        Dlg_JunkyardEacAmbush_1079 = 3230,
        Dlg_JunkyardEacAmbush_1080 = 3231,
        Dlg_JunkyardEacAmbush_1081 = 3232,
        Dlg_JunkyardEacAmbush_1082 = 3233,
        Dlg_JunkyardEacAmbush_1083 = 3234,
        Dlg_JunkyardEacAmbush_1084 = 3235,
        Dlg_JunkyardEacAmbush_1085 = 3236,
        Dlg_JunkyardEacAmbush_1086 = 3237,
        Dlg_JunkyardEacAmbush_1087 = 3238,
        Dlg_JunkyardEacAmbush_1088 = 3239,
        Dlg_JunkyardEacAmbush_1089 = 3240,
        Dlg_JunkyardEacAmbush_1090 = 3241,
        Dlg_JunkyardEacAmbush_1091 = 3242,
        Dlg_JunkyardEacAmbush_1092 = 3243,
        Dlg_JunkyardEacAmbush_1093 = 3244,
        Dlg_JunkyardEacAmbush_1094 = 3245,
        Dlg_JunkyardEacAmbush_1095 = 3246,
        Dlg_JunkyardEacAmbush_1096 = 3247,
        Dlg_JunkyardEacAmbush_1097 = 3248,
        Dlg_JunkyardEacAmbush_1098 = 3249,
        Dlg_JunkyardEacAmbush_1099 = 3250,
        Dlg_JunkyardEacAmbush_1100 = 3251,
        Dlg_JunkyardEacAmbush_1101 = 3252,
        Dlg_JunkyardEacAmbush_1102 = 3253,
        Dlg_JunkyardEacAmbush_1103 = 3254,
        Dlg_JunkyardEacAmbush_1104 = 3255,
        Dlg_JunkyardEacAmbush_1105 = 3256,
        Dlg_JunkyardEacAmbush_1106 = 3257,
        Dlg_JunkyardEacAmbush_1107 = 3258,
        Dlg_JunkyardEacAmbush_1108 = 3259,
        Dlg_JunkyardEacAmbush_1109 = 3260,
        Dlg_JunkyardEacAmbush_1110 = 3261,
        Dlg_JunkyardEacAmbush_1111 = 3262,
        Dlg_JunkyardEacAmbush_1112 = 3263,
        Dlg_JunkyardEacAmbush_1113 = 3264,
        Dlg_JunkyardEacAmbush_1114 = 3265,
        Dlg_JunkyardEacAmbush_1115 = 3266,
        Dlg_JunkyardEacAmbush_1116 = 3267,
        Dlg_JunkyardEacAmbush_1117 = 3268,
        Dlg_JunkyardEacAmbush_1118 = 3269,
        Dlg_JunkyardEacAmbush_1119 = 3270,
        Dlg_JunkyardEacAmbush_1120 = 3271,
        Dlg_JunkyardEacAmbush_1121 = 3272,
        Dlg_JunkyardEacAmbush_1122 = 3273,
        Dlg_JunkyardEacAmbush_1123 = 3274,
        Dlg_JunkyardEacAmbush_1124 = 3275,
        Dlg_JunkyardEacAmbush_1125 = 3276,
        Dlg_JunkyardEacAmbush_1126 = 3277,
        Dlg_JunkyardEacAmbush_1127 = 3278,
        Dlg_JunkyardEacAmbush_1128 = 3279,
        Dlg_JunkyardEacAmbush_1129 = 3280,
        Dlg_JunkyardEacAmbush_1130 = 3281,
        Dlg_JunkyardEacAmbush_1131 = 3282,
        Dlg_JunkyardEacAmbush_1132 = 3283,
        Dlg_JunkyardEacAmbush_1133 = 3284,
        Dlg_JunkyardEacAmbush_1134 = 3285,
        Dlg_JunkyardEacAmbush_1135 = 3286,
        Dlg_JunkyardEacAmbush_1136 = 3287,
        Dlg_JunkyardEacAmbush_1137 = 3288,
        Dlg_EacTransmitter_1000 = 3289,
        Dlg_EacTransmitter_1001 = 3290,
        Dlg_EacTransmitter_1002 = 3291,
        Dlg_EacTransmitter_1003 = 3292,
        Dlg_EacTransmitter_1004 = 3293,
        Dlg_EacTransmitter_1005 = 3294,
        Dlg_EacTransmitter_1006 = 3295,
        Dlg_EacTransmitter_1007 = 3296,
        Dlg_EacTransmitter_1008 = 3297,
        Dlg_EacTransmitter_1009 = 3298,
        Dlg_EacTransmitter_1010 = 3299,
        Dlg_EacTransmitter_1011 = 3300,
        Dlg_EacTransmitter_1012 = 3301,
        Dlg_EacTransmitter_1013 = 3302,
        Dlg_EacTransmitter_1014 = 3303,
        Dlg_EacTransmitter_1015 = 3304,
        Dlg_EacTransmitter_1016 = 3305,
        Dlg_EacTransmitter_1017 = 3306,
        Dlg_EacTransmitter_1018 = 3307,
        Dlg_EacTransmitter_1019 = 3308,
        Dlg_EacTransmitter_1020 = 3309,
        Dlg_EacTransmitter_1021 = 3310,
        Dlg_EacTransmitter_1022 = 3311,
        Dlg_EacTransmitter_1023 = 3312,
        Dlg_EacTransmitter_1024 = 3313,
        Dlg_EacTransmitter_1025 = 3314,
        Dlg_EacTransmitter_1026 = 3315,
        Dlg_EacTransmitter_1027 = 3316,
        Dlg_EacTransmitter_1028 = 3317,
        Dlg_EacTransmitter_1029 = 3318,
        Dlg_EacTransmitter_1030 = 3319,
        Dlg_EacTransmitter_1031 = 3320,
        Dlg_EacTransmitter_1032 = 3321,
        Dlg_EacTransmitter_1033 = 3322,
        Dlg_EacTransmitter_1034 = 3323,
        Dlg_EacTransmitter_1035 = 3324,
        Dlg_EacTransmitter_1036 = 3325,
        Dlg_EacTransmitter_1037 = 3326,
        Dlg_EacTransmitter_1038 = 3327,
        Dlg_EacTransmitter_1039 = 3328,
        Dlg_EacTransmitter_1040 = 3329,
        Dlg_EacTransmitter_1041 = 3330,
        Dlg_EacTransmitter_1042 = 3331,
        Dlg_EacTransmitter_1043 = 3332,
        Dlg_EacTransmitter_1044 = 3333,
        Dlg_EacTransmitter_1045 = 3334,
        Dlg_EacTransmitter_1046 = 3335,
        Dlg_EacTransmitter_1047 = 3336,
        Dlg_EacTransmitter_1048 = 3337,
        Dlg_EacTransmitter_1049 = 3338,
        Dlg_EacTransmitter_1050 = 3339,
        Dlg_EacTransmitter_1051 = 3340,
        Dlg_EacTransmitter_1052 = 3341,
        Dlg_EacTransmitter_1053 = 3342,
        Dlg_EacTransmitter_1054 = 3343,
        Dlg_EacTransmitter_1055 = 3344,
        Dlg_EacTransmitter_1056 = 3345,
        Dlg_EacTransmitter_1057 = 3346,
        Dlg_EacTransmitter_1058 = 3347,
        Dlg_EacTransmitter_1059 = 3348,
        Dlg_EacTransmitter_1060 = 3349,
        Dlg_EacTransmitter_1061 = 3350,
        Dlg_EacTransmitter_1062 = 3351,
        Dlg_EacTransmitter_1063 = 3352,
        Dlg_EacTransmitter_1064 = 3353,
        Dlg_EacTransmitter_1065 = 3354,
        Dlg_EacTransmitter_1066 = 3355,
        Dlg_EacTransmitter_1067 = 3356,
        Dlg_EacTransmitter_1068 = 3357,
        Dlg_EacTransmitter_1069 = 3358,
        Dlg_EacTransmitter_1070 = 3359,
        Dlg_EacTransmitter_1071 = 3360,
        Dlg_EacTransmitter_1072 = 3361,
        Amb3D_GenSmallDamaged = 3362,
        Amb3D_GenSmallDestroyed = 3363,
        Amb3D_GenSmallNormal = 3364,
        Amb3D_PipeFlow01 = 3365,
        Amb3D_Spark01 = 3366,
        Amb3D_SteamDischarge01 = 3367,
        Amb3D_SteamDischarge02 = 3368,
        Amb3D_SteamDischarge03 = 3369,
        Amb3D_SteamDischarge04 = 3370,
        Amb3D_SteamLoop01 = 3371,
        Amb3D_SteamLoop02 = 3372,
        Amb3D_SteamLoop03 = 3373,
        Amb3D_SteamLoop04 = 3374,
        Amb3D_SteamLoop05 = 3375,
        Amb3D_ThunderClapLarge = 3376,
        Amb3D_ThunderClapMed = 3377,
        Amb3D_RadioChatterAllied09_12 = 3378,
        Amb3D_RadioChatterChinese13_16 = 3379,
        Amb2D_Electrical01 = 3380,
        Amb2D_Electrical02 = 3381,
        Amb2D_EngineThrust = 3382,
        Amb2D_FanLargeDamaged = 3383,
        Amb2D_FanLargeDestroyed = 3384,
        Amb2D_FanLargeNormal = 3385,
        Amb2D_FanMediumDamaged = 3386,
        Amb2D_FanMediumDestroyed = 3387,
        Amb2D_FanMediumNormal = 3388,
        Amb2D_FanSmallDamaged = 3389,
        Amb2D_FanSmallDestroyed = 3390,
        Amb2D_FanSmallNormal = 3391,
        Amb2D_GenLargeDamaged = 3392,
        Amb2D_GenLargeDestroyed = 3393,
        Amb2D_GenLargeNormal = 3394,
        Amb2D_GenMediumDamaged = 3395,
        Amb2D_GenMediumDestroyed = 3396,
        Amb2D_GenMediumNormal = 3397,
        Amb2D_GenSmallDamaged = 3398,
        Amb2D_GenSmallDestroyed = 3399,
        Amb2D_GenSmallNormal = 3400,
        Amb2D_GenXend = 3401,
        Amb2D_GenXloop = 3402,
        Amb2D_GenXstart = 3403,
        Amb2D_PipeFlow01 = 3404,
        Amb2D_PrefabFire01 = 3405,
        Amb2D_RadioChatterAL01 = 3406,
        Amb2D_RadioChatterAL02 = 3407,
        Amb2D_RadioChatterAL03 = 3408,
        Amb2D_RadioChatterAL04 = 3409,
        Amb2D_RadioChatterAL05 = 3410,
        Amb2D_RadioChatterAL06 = 3411,
        Amb2D_RadioChatterAL07 = 3412,
        Amb2D_RadioChatterAL08 = 3413,
        Amb2D_RadioChatterAL09 = 3414,
        Amb2D_RadioChatterAL10 = 3415,
        Amb2D_RadioChatterAL11 = 3416,
        Amb2D_RadioChatterAL12 = 3417,
        Amb2D_RadioChatterAL13 = 3418,
        Amb2D_RadioChatterAL14 = 3419,
        Amb2D_RadioChatterAL15 = 3420,
        Amb2D_RadioChatterAL16 = 3421,
        Amb2D_RadioChatterCH01 = 3422,
        Amb2D_RadioChatterCH02 = 3423,
        Amb2D_RadioChatterCH03 = 3424,
        Amb2D_RadioChatterCH04 = 3425,
        Amb2D_RadioChatterCH05 = 3426,
        Amb2D_RadioChatterCH06 = 3427,
        Amb2D_RadioChatterCH07 = 3428,
        Amb2D_RadioChatterCH08 = 3429,
        Amb2D_RadioChatterCH09 = 3430,
        Amb2D_RadioChatterCH10 = 3431,
        Amb2D_RadioChatterCH11 = 3432,
        Amb2D_RadioChatterCH12 = 3433,
        Amb2D_RadioChatterCH13 = 3434,
        Amb2D_RadioChatterCH14 = 3435,
        Amb2D_RadioChatterCH15 = 3436,
        Amb2D_RadioChatterCH16 = 3437,
        Amb2D_RadioChatterRS01 = 3438,
        Amb2D_RadioChatterRS02 = 3439,
        Amb2D_RadioChatterRS03 = 3440,
        Amb2D_RadioChatterRS04 = 3441,
        Amb2D_RadioChatterRS05 = 3442,
        Amb2D_RadioChatterRS06 = 3443,
        Amb2D_RadioChatterRS07 = 3444,
        Amb2D_RadioChatterRS08 = 3445,
        Amb2D_RadioChatterRS09 = 3446,
        Amb2D_RadioChatterRS10 = 3447,
        Amb2D_RadioChatterRS11 = 3448,
        Amb2D_RadioChatterRS12 = 3449,
        Amb2D_RadioChatterRS13 = 3450,
        Amb2D_RadioChatterRS14 = 3451,
        Amb2D_RadioChatterRS15 = 3452,
        Amb2D_RadioChatterRS16 = 3453,
        Amb2D_Spark01 = 3454,
        Amb2D_SteamDischarge01 = 3455,
        Amb2D_SteamDischarge02 = 3456,
        Amb2D_SteamDischarge03 = 3457,
        Amb2D_SteamDischarge04 = 3458,
        Amb2D_SteamLoop01 = 3459,
        Amb2D_SteamLoop02 = 3460,
        Amb2D_SteamLoop03 = 3461,
        Amb2D_SteamLoop04 = 3462,
        Amb2D_SteamLoop05 = 3463,
        Amb2D_Temple1 = 3464,
        Amb2D_Temple2 = 3465,
        Amb2D_Temple3 = 3466,
        Amb2D_ThunderClapLarge01 = 3467,
        Amb2D_ThunderClapMed01 = 3468,
        Amb2D_ThunderClapMed02 = 3469,
        Amb2D_ThunderClapMed03 = 3470,
        Amb2D_ThunderClapMed04 = 3471,
        Amb2D_ThunderClapSmall01 = 3472,
        Amb2D_ThunderClapSmall02 = 3473,
        Amb2D_ThunderClapSmall03 = 3474,
        Amb2D_ThunderClapSmall04 = 3475,
        Amb2D_Welding01 = 3476,
        SfxScanner = 3477,
        Dlg_EACSurveySite_0069_02 = 3478,
        Dlg_EACSurveySite_0069_03 = 3479,
        Dlg_JunkyardReturn_1028 = 3480,
        Dlg_JunkyardReturn_1029 = 3481,
        Dlg_JunkyardReturn_1030 = 3482,
        Dlg_JunkyardReturn_1031 = 3483,
        Dlg_JunkyardReturn_1032 = 3484,
        Dlg_JunkyardReturn_1033 = 3485,
        Dlg_JunkyardReturn_1034 = 3486,
        Dlg_JunkyardReturn_1035 = 3487,
        Dlg_JunkyardReturn_1036 = 3488,
        Dlg_JunkyardReturn_1037 = 3489,
        Dlg_Laika_1126 = 3490,
        Dlg_ReichstagC_1022_2 = 3491,
        Dlg_AlienGate_1038_2 = 3492,
        Dlg_AlienGate_1067_2 = 3493,
        MusSpecial_KA07 = 3494,
        WepAutocanon3Fire3d_nonLoop = 3495,

        MusSpecial_KA12 = 3496,
        MusSpecial_KA14 = 3497,
        MusStressOrTimeRush_KA03 = 3498,
        MusSpecial_KA09 = 3499,
        MusSpecial_KA10 = 3500,
        MusSpecial_KA11 = 3501,
        MusSpecial_KA08 = 3502,
        MusSpecial_KA13 = 3503,
        MusHeavyFight_KA15 = 3504,
        MusSpecial_KA15 = 3505,
        MusSpecial_KA16 = 3506,
        MusCalmAtmosphere_KA05 = 3507,
    }

    struct MyCueParameters
    {
        public bool UseOcclusion;
        public MySoundCuesEnum? Wrapped2dEnum;
        public bool Loopable;
        public float MaxDistance;
        public int LastUpdate;
        public bool IsHudCue;

        public MyCueParameters(bool useOcclusion, MySoundCuesEnum? wrapped2dEnum, bool loopable)
            : this(useOcclusion, wrapped2dEnum, loopable, false)
        {
        }

        public MyCueParameters(bool useOcclusion, MySoundCuesEnum? wrapped2dEnum, bool loopable, bool isHudCue)
        {
            UseOcclusion = useOcclusion;
            LastUpdate = 0;
            Wrapped2dEnum = wrapped2dEnum;
            Loopable = loopable;
            MaxDistance = MyFakes.USE_LONG_SOUND_DISTANCE ? MyFakes.LONG_SOUND_DISTANCE : MyAudio.DISTANCE_1000_METERS;  //  Default distance
            IsHudCue = isHudCue;
        }

        public MyCueParameters(bool useOcclusion, MySoundCuesEnum? wrapped2dEnum, bool loopable, float maxDistance)
            : this(useOcclusion, wrapped2dEnum, loopable)
        {
            MaxDistance = maxDistance;
        }

        public MyCueParameters(bool useOcclusion, MySoundCuesEnum? wrapper2dEnum, bool loopable, float maxDistance, bool isHudCue)
            : this(useOcclusion, wrapper2dEnum, loopable, maxDistance)
        {
            IsHudCue = isHudCue;
        }
    }

    static partial class MyAudio
    {
        //sound enum groups
        const string GROUP_GUI = "Gui";
        const string GROUP_IMP = "Imp";
        const string GROUP_VEH = "Veh";
        const string GROUP_WEP = "Wep";
        const string GROUP_SFX = "Sfx";
        const string GROUP_MUS = "Mus";
        const string GROUP_VOC = "Voc";
        const string GROUP_HUD = "Hud";
        const string GROUP_DLG = "Dlg";  // TODO
        const string GROUP_SHT = "Sht";  // shouts

        public static float DISTANCE_250_METERS = MyFakes.USE_LONG_SOUND_DISTANCE ? MyFakes.LONG_SOUND_DISTANCE : 250.0f;
        public static float DISTANCE_500_METERS = MyFakes.USE_LONG_SOUND_DISTANCE ? MyFakes.LONG_SOUND_DISTANCE : 500.0f;
        public static float DISTANCE_1000_METERS = MyFakes.USE_LONG_SOUND_DISTANCE ? MyFakes.LONG_SOUND_DISTANCE : 1000.0f;
        public static float DISTANCE_5000_METERS = MyFakes.USE_LONG_SOUND_DISTANCE ? MyFakes.LONG_SOUND_DISTANCE : 5000.0f;
        public static float DISTANCE_15000_METERS = MyFakes.USE_LONG_SOUND_DISTANCE ? MyFakes.LONG_SOUND_DISTANCE : 15000.0f;


        static MyCueParameters?[] m_cueParameters;
        static Dictionary<string, MySoundCuesEnum>[] m_musicTransitionCues;

        static void InitCueParameters()
        {
            MyMwcLog.WriteLine("MyAudio.InitCueParameters() - START", LoggingOptions.AUDIO);
            MyMwcLog.IncreaseIndent(LoggingOptions.AUDIO);
            m_musicTransitionCues = new Dictionary<string, MySoundCuesEnum>[MyMwcUtils.GetMaxValueFromEnum<MyMusicTransitionEnum>() + 1];
            m_cueParameters = new MyCueParameters?[GetNumberOfSounds()];

            m_cueParameters[(int)MySoundCuesEnum.GuiMouseClick] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiMouseOver] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorFlyOutsideBorder] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorObjectAttach] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorObjectDelete] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorObjectDetach] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorObjectMoveInvalid] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorObjectMoveStep] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorObjectRotateStep] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorObjectSelect] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorPrefabCommit] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorPrefabEnter] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorPrefabExit] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorVoxelHandAdd] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorVoxelHandRemove] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorVoxelHandSoften] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorVoxelHandMaterial] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiEditorVoxelHandSwitch] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiWheelControlOpen] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.GuiWheelControlClose] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpBulletHitRock] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpBulletHitMetal] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpBulletHitGlass] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpRockCollideMetal] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpRockCollideRock] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpShipCollideMetal] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpShipCollideRock] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpPlayerShipCollideMetal] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpPlayerShipCollideRock] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpPlayerShipCollideShip] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpPlayerShipScrapeShipLoop] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpPlayerShipScrapeShipRelease] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpBulletHitShip] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpExpHitGlass] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpExpHitMetal] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpExpHitShip] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.ImpExpHitRock] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaEngineIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaEngineIdle3d] = new MyCueParameters(true, MySoundCuesEnum.VehShipaEngineIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaEngineHigh2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaEngineHigh3d] = new MyCueParameters(true, MySoundCuesEnum.VehShipaEngineHigh2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaThrust2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaThrust3d] = new MyCueParameters(true, MySoundCuesEnum.VehShipaThrust2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaLightsOn] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaLightsOff] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaEngineOn] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehShipaEngineOff] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeRelease2d] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehHarvesterTubeRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeMovingLoop2d] = new MyCueParameters(true, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeMovingLoop3d] = new MyCueParameters(true, MySoundCuesEnum.VehHarvesterTubeMovingLoop2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeColliding2d] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeColliding3d] = new MyCueParameters(true, MySoundCuesEnum.VehHarvesterTubeColliding2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeCollision2d] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeCollision3d] = new MyCueParameters(true, MySoundCuesEnum.VehHarvesterTubeCollision2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeImplode2d] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehHarvesterTubeImplode3d] = new MyCueParameters(true, MySoundCuesEnum.VehHarvesterTubeImplode2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillLoop2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillLoop3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolCrusherDrillLoop2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillRelease2d] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolCrusherDrillRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillColliding2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillColliding3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolCrusherDrillColliding2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillColliding2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillColliding3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolCrusherDrillColliding2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillCollidingRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillCollidingRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolCrusherDrillCollidingRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillLoop2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillLoop3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolThermalDrillLoop2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillRelease2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillRelease3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolThermalDrillRelease2d, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillLoop2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillLoop3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolLaserDrillLoop2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillRelease2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillRelease3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolLaserDrillRelease2d, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCut2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCut3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolSawCut2d, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawLoop2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawLoop3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolSawLoop2d, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawRelease2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawRelease3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolSawRelease2d, false);
            m_cueParameters[(int)MySoundCuesEnum.WepAutocanonFire2d] = new MyCueParameters(false, null, true, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepAutocanonFire3d] = new MyCueParameters(true, MySoundCuesEnum.WepAutocanonFire2d, true, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepAutocanon3Fire3d_nonLoop] = new MyCueParameters(true, MySoundCuesEnum.WepAutocanonFire2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMissileLaunch2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.WepMissileLaunch3d] = new MyCueParameters(true, MySoundCuesEnum.WepMissileLaunch2d, false);
            m_cueParameters[(int)MySoundCuesEnum.WepCannon2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.WepCannon3d] = new MyCueParameters(true, MySoundCuesEnum.WepCannon3d, false);
            m_cueParameters[(int)MySoundCuesEnum.WepAutocanonRel2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepAutocanonRel3d] = new MyCueParameters(true, MySoundCuesEnum.WepAutocanonRel2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMissileLock] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.WepMissileExplosion] = new MyCueParameters(true, null, false, DISTANCE_15000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMissileFly] = new MyCueParameters(true, null, true, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepSniperScopeZoomRel] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.WepSniperScopeZoomALoop] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.WepMineMoveALoop] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.WepUnivLaunch2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.WepUnivLaunch3d] = new MyCueParameters(true, MySoundCuesEnum.WepUnivLaunch2d, false);
            m_cueParameters[(int)MySoundCuesEnum.WepArsHighShot2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepArsHighShot3d] = new MyCueParameters(true, MySoundCuesEnum.WepArsHighShot2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepArsNormShot2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepArsNormShot3d] = new MyCueParameters(true, MySoundCuesEnum.WepArsNormShot2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepSniperNormFire2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepSniperNormFire3d] = new MyCueParameters(true, MySoundCuesEnum.WepSniperNormFire2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepSniperHighFire2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepSniperHighFire3d] = new MyCueParameters(true, MySoundCuesEnum.WepSniperHighFire2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepBombExplosion] = new MyCueParameters(true, null, false, DISTANCE_15000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepLargeShipAutocannonRotate] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepLargeShipAutocannonRotateRelease] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunHighFire2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunHighFire3d] = new MyCueParameters(true, MySoundCuesEnum.WepMachineGunHighFire2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunHighRel2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunHighRel3d] = new MyCueParameters(true, MySoundCuesEnum.WepMachineGunHighRel2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunNormFire2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunNormFire3d] = new MyCueParameters(true, MySoundCuesEnum.WepMachineGunNormFire2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunNormRel2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepMachineGunNormRel3d] = new MyCueParameters(true, MySoundCuesEnum.WepMachineGunNormRel2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepShotgunNormShot2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepShotgunNormShot3d] = new MyCueParameters(true, MySoundCuesEnum.WepShotgunNormShot2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepShotgunHighShot2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepShotgunHighShot3d] = new MyCueParameters(true, MySoundCuesEnum.WepShotgunHighShot2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepRailNormShot2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepRailNormShot3d] = new MyCueParameters(true, MySoundCuesEnum.WepRailNormShot2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepRailHighShot2d] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepRailHighShot3d] = new MyCueParameters(true, MySoundCuesEnum.WepRailHighShot2d, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepBombSmartTimer] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepBombSmartSmoke] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepBombSmartPlant] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepBombSmartDrone] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepBombGravSuck] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepEpmExplosion] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepBombFlash] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxFlareLoop01] = new MyCueParameters(false, null, false, DISTANCE_500_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxFlareDeploy] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.WepNoAmmo] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxSolarWind] = new MyCueParameters(true, null, true, DISTANCE_15000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxShipSmallExplosion] = new MyCueParameters(true, null, false, DISTANCE_15000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudReflectorRange] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudSlowMovementOff] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudSlowMovementOn] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAutolevelingOn] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAutolevelingOff] = new MyCueParameters(true, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudWeaponScroll] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudWeaponSelect] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudRadarMode] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudCockpitOn] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudCockpitOff] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxSpark] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudBackcameraOn] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudBackcameraOff] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxPlayerBreath] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxPlayerDeathBeep] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxPlayerDeathBreath] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor1AClose] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor1AOpen] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor1BClose] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor1BOpen] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor2AClose] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor2AOpen] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor2BClose] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor2BOpen] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor3AClose] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDoor3AOpen] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.HudFuelLowWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudFuelCriticalWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudOxygenLowWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudOxygenCriticalWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudOxygenNoWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudDamageAlertWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudDamageCriticalWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudAmmoCriticalWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudAmmoLowWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudArmorCriticalWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudArmorLowWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudDestinationReached] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudRadarJammedWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudTargetDestroyed] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudRadiationWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.SfxGeigerCounterHeavyLoop] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.HudEnemyAlertWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudFriendlyFireWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudHarvestingComplete] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudSolarFlareWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudInventoryComplete] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudInventoryFullWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudInventoryTransfer] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudObjectiveComplete] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            //m_cueParameters[(int)MySoundCuesEnum.HudMissionFailed] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudHealthLowWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudHealthCriticalWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            //m_cueParameters[(int)MySoundCuesEnum.HudMissionComplete] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.SfxClaxonAlert] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillIdle3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolCrusherDrillIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillIdle3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolLaserDrillIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawIdle3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolSawIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillIdle3d] = new MyCueParameters(false, MySoundCuesEnum.VehToolThermalDrillIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.SfxGps] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxGpsFail] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAlarmIncoming] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAlarmDamageA] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAlarmDamageB] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAlarmDamageC] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAlarmDamageD] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxHudAlarmDamageE] = new MyCueParameters(false, null, false, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxNuclearExplosion] = new MyCueParameters(false, null, false, DISTANCE_15000_METERS);

            m_cueParameters[(int)MySoundCuesEnum.MenuWelcome] = new MyCueParameters(false, null, false);

            // new music
            m_cueParameters[(int)MySoundCuesEnum.MusCalmAtmosphere_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusCalmAtmosphere_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusDesperateWithStress_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusDesperateWithStress_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusCalmAtmosphere_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusCalmAtmosphere_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSadnessOrDesperation_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSadnessOrDesperation_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSadnessOrDesperation_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSadnessOrDesperation_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStealthAction_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStealthAction_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStealthAction_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStressOrTimeRush_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStressOrTimeRush_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStressOrTimeRush_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStressOrTimeRush_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusTensionBeforeAnAction_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusTensionBeforeAnAction_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusTensionBeforeAnAction_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusTensionBeforeAnAction_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusVictory_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusVictory_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusVictory_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusVictory_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Mus_victory_KA_6] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA15] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusCalmAtmosphere_KA03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusCalmAtmosphere_KA05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusDesperateWithStress_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusDesperateWithStress_KA03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusDesperateWithStress_KA04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Mus_desperatestress_KA_2_longer_2b] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusMainMenu_KA_MM] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHorror_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHorror_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusMystery_MM01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusMystery_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusMystery_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusMystery_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHorror_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHorror_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA07] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA10] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA11] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_MM02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_MM03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA07] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_MM04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusLightFight_KA27] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_MM05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Mus_lightfight_KA_26] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Mus_lightfight_KA_28] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Mus_special_KA_4_loop] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA19] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA17] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusHeavyFight_KA06] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStealthAction_KA03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA06] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusCalmAtmosphere_MM_b] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_LostInTheDistance] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA07] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA09] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA10] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA11] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA13] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA14] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA15] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusSpecial_KA16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.MusStressOrTimeRush_KA03] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.MovDock1Start] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock1Loop] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock1End] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock2Start] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock2Loop] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock2End] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock3Start] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock3Loop] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.MovDock3End] = new MyCueParameters(false, null, false, DISTANCE_1000_METERS);

            m_cueParameters[(int)MySoundCuesEnum.VehCH1EngineIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehCH1EngineIdle3d] = new MyCueParameters(true, MySoundCuesEnum.VehCH1EngineIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehCH1EngineHigh2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehCH1EngineHigh3d] = new MyCueParameters(true, MySoundCuesEnum.VehCH1EngineHigh2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehCH1EngineOn] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehCH1EngineOff] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehEL1EngineIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehEL1EngineIdle3d] = new MyCueParameters(true, MySoundCuesEnum.VehEL1EngineIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehEL1EngineHigh2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehEL1EngineHigh3d] = new MyCueParameters(true, MySoundCuesEnum.VehEL1EngineHigh2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehEL1EngineOn] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehEL1EngineOff] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehNU1EngineIdle2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehNU1EngineIdle3d] = new MyCueParameters(true, MySoundCuesEnum.VehNU1EngineIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehNU1EngineHigh2d] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.VehNU1EngineHigh3d] = new MyCueParameters(true, MySoundCuesEnum.VehNU1EngineHigh2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehNU1EngineOn] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehNU1EngineOff] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.ImpShipQuake] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomLarge01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomLarge02] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomLarge03] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomLarge04] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomLarge05] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomMed01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomMed02] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomMed03] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomMed04] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomMed05] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomSmall01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomSmall02] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomSmall03] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RoomSmall04] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_StressLoop] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_TunnelLarge01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_TunnelMedium01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_TunnelSmall01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_Electrical01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_Electrical02] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanLargeDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanLargeDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanLargeNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanMediumDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanMediumDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanMediumNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanSmallDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanSmallDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_FanSmallNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenLargeDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenLargeDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenLargeNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenMediumDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenMediumDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenMediumNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenSmallDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenSmallDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenSmallNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_PipeFlow01] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterAllied01_04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterAllied05_08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterAllied09_12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterAllied13_16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterChinese01_04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterChinese05_08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterChinese09_12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterChinese13_16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterRussian01_04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterRussian05_08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterRussian09_12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_RadioChatterRussian13_16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_Spark01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamDischarge01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamDischarge02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamDischarge03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamDischarge04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamLoop01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamLoop02] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamLoop03] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamLoop04] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_SteamLoop05] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_ThunderClapLarge] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_ThunderClapMed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_ThunderClapSmall] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_Welding01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_PrefabFire] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_EngineThrust] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Amb2D_City] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ComputerRoom] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Factory] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_LostPlace] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RedHeat] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_War] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_City2] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Factory2] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillColliding2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillColliding3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolLaserDrillColliding2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillCollidingRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserDrillCollidingRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolLaserDrillCollidingRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillColliding2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillColliding3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolThermalDrillColliding2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillCollidingRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillCollidingRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolThermalDrillCollidingRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOther2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOther3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolLaserCollidingOther2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOtherRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOtherRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolLaserCollidingOtherRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillCollidingOther2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillCollidingOther3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolCrusherDrillCollidingOther2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillCollidingOtherRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolCrusherDrillCollidingOtherRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolCrusherDrillCollidingOtherRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOther2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOther3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolLaserCollidingOther2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOtherRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolLaserCollidingOtherRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolLaserCollidingOtherRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillBlastOther2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillBlastOther3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolPressureDrillBlastOther2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCutOther2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCutOther3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolSawCutOther2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCutOtherRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCutOtherRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolSawCutOtherRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCutRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolSawCutRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolSawCutRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillCollidingOther2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillCollidingOther3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolThermalDrillCollidingOther2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillCollidingOtherRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolThermalDrillCollidingOtherRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolThermalDrillCollidingOtherRelease2d, true);

            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillBlast3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolPressureDrillBlast2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillBlastRock3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolPressureDrillBlastRock2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillIdle3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolPressureDrillIdle2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillRecharge3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolPressureDrillRecharge2d, true);

            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillLoop2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillLoop3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolNuclearDrillLoop2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolNuclearDrillRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillColliding2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillColliding3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolNuclearDrillColliding2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillCollidingOther2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillCollidingOther3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolNuclearDrillCollidingOther2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillCollidingRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillCollidingRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolNuclearDrillCollidingRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillCollidingOtherRelease2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillCollidingOtherRelease3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolNuclearDrillCollidingOtherRelease2d, true);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillIdle2d] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehToolNuclearDrillIdle3d] = new MyCueParameters(true, MySoundCuesEnum.VehToolNuclearDrillIdle2d, true);


            m_cueParameters[(int)MySoundCuesEnum.VehLoopDrone] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehLoopCamera] = new MyCueParameters(false, null, true, DISTANCE_250_METERS);





            m_cueParameters[(int)MySoundCuesEnum.SfxProgressHack] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxCancelHack] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxProgressRepair] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxProgressBuild] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxCancelRepair] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxCancelBuild] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxProgressActivation] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxCancelActivation] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxProgressTake] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxCancelTake] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxProgressPut] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxCancelPut] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenXstart] = new MyCueParameters(false, null, false, DISTANCE_500_METERS);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenXloop] = new MyCueParameters(false, null, true, DISTANCE_500_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxAlertVoc] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_GenXend] = new MyCueParameters(false, null, false, DISTANCE_500_METERS);

            m_cueParameters[(int)MySoundCuesEnum.Amb3D_Temple1] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_Temple2] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb3D_Temple3] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxShipLargeExplosion] = new MyCueParameters(true, null, false, DISTANCE_15000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.HudFuelNoWarning] = new MyCueParameters(false, null, false, true);

            m_cueParameters[(int)MySoundCuesEnum.VocRace01Welcome] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace02countdown] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace03cleanStart] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace04running] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace05minefields] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace06speedDuel] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace07allThose] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace08moving] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace09dirtyTricks] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace10inside] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VocRace11wire] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillBlast2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillIdle2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillRecharge2d] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.VehToolPressureDrillBlastRock2d] = new MyCueParameters(false, null, false);

            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0001] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0002] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0003] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0004] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0005] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0006] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0007] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0008] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0009] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0010] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0011] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0012_01] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0012_02] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0013] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0014_01] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0014_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0017_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0017_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0018_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0018_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0018_03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0024_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0024_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0025_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0025_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0027_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0027_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0032_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0032_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0040_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0040_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0045] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0046] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0047] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0048] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0049_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0049_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0050] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0051] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0052] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0053_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0053_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0053_03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0054] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0055_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0055_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0056] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0057] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0058] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0059] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0060_01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0060_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0061] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0062] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0063] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0064] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0065] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0066] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0067] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0068] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0069] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0069_02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0069_03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0070] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0071] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0072] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0073] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0074] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0075] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0076] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0077] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_0078] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EACSurveySite_1004] = new MyCueParameters(false, null, false);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransport_1045] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseRafinery_1041] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1045] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1046] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1047] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1048] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1049] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1050] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1051] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1052] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1053] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1054] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1055] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1056] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1057] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1058] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1059] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1060] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1061] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1062] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1063] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1064] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1065] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1066] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1067] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1068] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseTransmitter_1069] = new MyCueParameters(false, null, false);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1045] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1046] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1047] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1048] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1049] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1050] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1051] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBaseWipeout_1052] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_SlaverBase2_1044] = new MyCueParameters(false, null, false);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1044] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1045] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1046] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1047] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1048] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1049] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1050] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1051] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1052] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1053] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1054] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1055] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1056] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1057] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1058] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1059] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1060] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1061] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1062] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1063] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1064] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1065] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1066] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1067] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1068] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1069] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1070] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1071] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1072] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1073] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1074] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1075] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1076] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1077] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1078] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1079] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1080] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1081] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1082] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1083] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1084] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1085] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1086] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1087] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1088] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1089] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1090] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1091] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1092] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1093] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1094] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1095] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1096] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1097] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1098] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1099] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1100] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1101] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1102] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1103] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1104] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1105] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1106] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1107] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1108] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1109] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1110] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1111] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1112] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1113] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1114] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1115] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1116] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1117] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1118] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1119] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1120] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1121] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1122] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1123] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1124] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1125] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1126] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1127] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1128] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1129] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1130] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1131] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_TwinTowers_1132] = new MyCueParameters(false, null, false);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_WhiteWolvesBioResearch_1044] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1045] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1046] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1047] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1048] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1049] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1050] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1051] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1052] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1053] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1054] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1055] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1056] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1057] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1058] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1059] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1060] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1061] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1062] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1063] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1064] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1065] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1066] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1067] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1068] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1069] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1070] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1071] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1072] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1073] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1074] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1075] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1076] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1077] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1078] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1079] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1080] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1081] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1082] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1083] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1084] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1085] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1086] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1087] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1088] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1089] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1090] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1091] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1092] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1093] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1094] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1095] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1096] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1097] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1098] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1099] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1100] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1101] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1102] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1103] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1104] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1105] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1106] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1107] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1108] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1109] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1110] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1111] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1112] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1113] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1114] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1115] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1116] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1117] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1118] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1119] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1120] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1121] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1122] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1123] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1124] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1125] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1126] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1127] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1128] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1129] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1130] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1131] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1132] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1133] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1134] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1135] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1136] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1137] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1138] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1139] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1140] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1141] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1142] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1143] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1144] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1145] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1146] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1147] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RimeConvince_1148] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1045] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1046] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1047] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1048] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1049] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1050] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1051] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1052] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1053] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1054] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1055] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1056] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1057] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1058] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1059] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Rift_1060] = new MyCueParameters(false, null, false);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1000] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1001] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1002] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1003] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1004] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1005] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1006] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1007] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1008] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1009] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1010] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1011] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1012] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1013] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1014] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1015] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1016] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1017] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1018] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1019] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1020] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1021] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1022] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1023] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1024] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1025] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1026] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1027] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1037] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1038] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1039] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1040] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1041] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1042] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1043] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1044] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1045] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1046] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1047] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1048] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1049] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1050] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1051] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1052] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1053] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1054] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1055] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1056] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1057] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1058] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1059] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1060] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1061] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1062] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1063] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1064] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1065] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1066] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1067] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1068] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1069] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1070] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1071] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1072] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1073] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1074] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1075] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1076] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1077] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1078] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1079] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1080] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1081] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1082] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1083] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1084] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1085] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1086] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1087] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1088] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1089] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1090] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1091] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1092] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1093] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1094] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1095] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1096] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1097] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1098] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1099] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1100] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1101] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1102] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1103] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1104] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1105] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1106] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1107] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1108] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1109] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1110] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianTransmitter_1111] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1004] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1013] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1016] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1023] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1047] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1069] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1070] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1074] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1079] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1079b] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1082] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1083] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1084] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1086] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1088] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1089] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1093] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1102] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1104] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1106] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1107] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1112] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1114] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1121] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1126] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1127] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1128] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1129] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1130] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1131] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1132] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1133] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1134] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1135] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1136] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1137] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1138] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1139] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1140] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1141] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1142] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1143] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1144] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1145] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1146] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1147] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1148] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1149] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1150] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1151] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1152] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1153] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1154] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1155] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1156] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1157] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1158] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1159] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1160] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1161] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1162] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1163] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1164] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1165] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1166] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1167] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1168] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1169] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1170] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1171] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1172] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1173] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1174] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1175] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1176] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1177] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1178] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1179] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1180] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1181] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1182] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1183] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1184] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1185] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1186] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1187] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1188] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1189] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1190] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1191] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1192] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1193] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1194] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1195] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1196] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1197] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1198] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1199] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1200] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1201] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1202] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1203] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1204] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1205] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1206] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1207] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1208] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1209] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1210] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1211] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1212] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1213] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1214] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonTransmitter_1215] = new MyCueParameters(false, null, true);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiant_1046] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantB_1032] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagA_1038] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1022_2] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ReichstagC_1065] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacPrison_1088] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ChineseEscape_1027] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_ResearchVessel_1056] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_Laika_1126] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonPlant_1126] = new MyCueParameters(false, null, true);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1126] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1127] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1128] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1129] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1130] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1131] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1132] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1133] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1134] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1135] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1136] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1137] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_FortValiantC_1138] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1038_2] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1067_2] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1126] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1127] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1128] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1129] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1130] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1131] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1132] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1133] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1134] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1135] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1136] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1137] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1138] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1139] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1140] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1141] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1142] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1143] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1144] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1145] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1146] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1147] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1148] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1149] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1150] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1151] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1152] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1153] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1154] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1155] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1156] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1157] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1158] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1159] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1160] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1161] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1162] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1163] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1164] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1165] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_AlienGate_1166] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1126] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1127] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1128] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1129] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1130] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1131] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1132] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1133] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1134] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1135] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1136] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1137] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1138] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1139] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1140] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1141] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1142] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1143] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1144] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1145] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1146] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1147] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1148] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1149] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1150] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1151] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1152] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1153] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1154] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1155] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1156] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1157] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1158] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1159] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_PirateBase_1160] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_BarthsMoonConvince_1111] = new MyCueParameters(false, null, true);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1126] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1127] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1128] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1129] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1130] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1131] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1132] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1133] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1134] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1135] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1136] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1137] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1138] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1139] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1140] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1141] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1142] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1143] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1144] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1145] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1146] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1147] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1148] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1149] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardConvince_1150] = new MyCueParameters(false, null, true);


            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1090] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1077] = new MyCueParameters(false, null, true);




            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1083] = new MyCueParameters(false, null, true);




            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1126] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1127] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1128] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1129] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1130] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1131] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1132] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1133] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_LastHope_1134] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_RussianWarehouse_1062] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1072] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1073] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1074] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1075] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1076] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1077] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1078] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1079] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1080] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1081] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1082] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1083] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1084] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1085] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1086] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1087] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1088] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1089] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1090] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1091] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1092] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1093] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1094] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1095] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1096] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1097] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1098] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1099] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1100] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1101] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1102] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1103] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1104] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1105] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1106] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1107] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1108] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1109] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1110] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1111] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1112] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1113] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1114] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1115] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1116] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1117] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1118] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1119] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1120] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1121] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1122] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1123] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1124] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1125] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1126] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1127] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1128] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1129] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1130] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1131] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1132] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1133] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1134] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1135] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1136] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardEacAmbush_1137] = new MyCueParameters(false, null, true);

            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1000] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1001] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1002] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1003] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1004] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1005] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1006] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1007] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1008] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1009] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1010] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1011] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1012] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1013] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1014] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1015] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1016] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1017] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1018] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1019] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1020] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1021] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1022] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1023] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1024] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1025] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1026] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1027] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1028] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1029] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1030] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1031] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1032] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1033] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1034] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1035] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1036] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1037] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1038] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1039] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1040] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1041] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1042] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1043] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1044] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1045] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1046] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1047] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1048] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1049] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1050] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1051] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1052] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1053] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1054] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1055] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1056] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1057] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1058] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1059] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1060] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1061] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1062] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1063] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1064] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1065] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1066] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1067] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1068] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1069] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1070] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1071] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_EacTransmitter_1072] = new MyCueParameters(false, null, true);


            m_cueParameters[(int)MySoundCuesEnum.Sht_Reich_JE_ForTheFuture] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Reich_JE_Vanquish] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Reich_JE_WeControl] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Reich_JE_YouAreTheWeakLink] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Reich_JE_YourFinalDay] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Sht_Russian_JE_DasVidanya] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Russian_JE_ForKremlin] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Russian_JE_ForMotherland] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Russian_JE_Harasho] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Russian_JE_ShootThem] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Russian_JE_TovaryshRemember] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Sht_Russian_JE_UnitedAndMighty] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.SfxTakeAllUniversal] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxTakeAllAmmo] = new MyCueParameters(false, null, false);
            //m_cueParameters[(int)MySoundCuesEnum.SfxTakeAllEnergy] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxTakeAllFuel] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxTakeAllMedkit] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxTakeAllOxygen] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxTakeAllRepair] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxAcquireWeaponOff] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxAcquireWeaponOn] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxAcquireDroneOff] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxAcquireDroneOn] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxAcquireCameraOff] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxAcquireCameraOn] = new MyCueParameters(false, null, false);

            m_cueParameters[(int)MySoundCuesEnum.HudAmmoNoWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudArmorNoWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.HudOxygenLeakingWarning] = new MyCueParameters(false, null, false, DISTANCE_250_METERS, true);
            m_cueParameters[(int)MySoundCuesEnum.VehLoopLargeShip] = new MyCueParameters(false, null, true, DISTANCE_1000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.VehLoopLargeShipMadelyn] = new MyCueParameters(false, null, true, DISTANCE_1000_METERS);

            m_cueParameters[(int)MySoundCuesEnum.SfxMeteorFly] = new MyCueParameters(false, null, true, DISTANCE_15000_METERS);
            m_cueParameters[(int)MySoundCuesEnum.SfxMeteorExplosion] = new MyCueParameters(true, null, false, DISTANCE_15000_METERS);

            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RacingFans] = new MyCueParameters(false, null, false, DISTANCE_500_METERS);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RedHeat2] = new MyCueParameters(false, null, false, DISTANCE_500_METERS);

            m_cueParameters[(int)MySoundCuesEnum.SfxVoxelCrack] = new MyCueParameters(true, null, false, false);
            m_cueParameters[(int)MySoundCuesEnum.SfxScanner] = new MyCueParameters(true, null, false);

            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Electrical01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Electrical02] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_EngineThrust] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanLargeDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanLargeDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanLargeNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanMediumDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanMediumDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanMediumNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanSmallDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanSmallDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_FanSmallNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenLargeDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenLargeDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenLargeNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenMediumDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenMediumDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenMediumNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenSmallDamaged] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenSmallDestroyed] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenSmallNormal] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenXend] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenXloop] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_GenXstart] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_PipeFlow01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_PrefabFire01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL06] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL07] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL09] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL10] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL11] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL13] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL14] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL15] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterAL16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH06] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH07] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH09] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH10] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH11] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH13] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH14] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH15] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterCH16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS05] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS06] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS07] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS08] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS09] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS10] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS11] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS12] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS13] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS14] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS15] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_RadioChatterRS16] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Spark01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamDischarge01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamDischarge02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamDischarge03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamDischarge04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamLoop01] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamLoop02] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamLoop03] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamLoop04] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_SteamLoop05] = new MyCueParameters(false, null, true);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Temple1] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Temple2] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Temple3] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapLarge01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapMed01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapMed02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapMed03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapMed04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapSmall01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapSmall02] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapSmall03] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_ThunderClapSmall04] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Amb2D_Welding01] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1028] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1029] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1030] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1031] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1032] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1033] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1034] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1035] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1036] = new MyCueParameters(false, null, false);
            m_cueParameters[(int)MySoundCuesEnum.Dlg_JunkyardReturn_1037] = new MyCueParameters(false, null, false);

            AddMusicCategory(MyMusicTransitionEnum.CalmAtmosphere, "MM01", MySoundCuesEnum.MusCalmAtmosphere_MM01);
            AddMusicCategory(MyMusicTransitionEnum.CalmAtmosphere, "MM02", MySoundCuesEnum.MusCalmAtmosphere_MM02);
            AddMusicCategory(MyMusicTransitionEnum.CalmAtmosphere, "MM_b", MySoundCuesEnum.MusCalmAtmosphere_MM_b);
            AddMusicCategory(MyMusicTransitionEnum.CalmAtmosphere, "KA01", MySoundCuesEnum.MusCalmAtmosphere_KA01);
            AddMusicCategory(MyMusicTransitionEnum.CalmAtmosphere, "KA02", MySoundCuesEnum.MusCalmAtmosphere_KA02);
            AddMusicCategory(MyMusicTransitionEnum.CalmAtmosphere, "KA03", MySoundCuesEnum.MusCalmAtmosphere_KA03);
            AddMusicCategory(MyMusicTransitionEnum.CalmAtmosphere, "KA05", MySoundCuesEnum.MusCalmAtmosphere_KA05);
            AddMusicCategory(MyMusicTransitionEnum.DesperateWithStress, "MM01", MySoundCuesEnum.MusDesperateWithStress_MM01);
            AddMusicCategory(MyMusicTransitionEnum.DesperateWithStress, "KA01", MySoundCuesEnum.MusDesperateWithStress_KA01);
            AddMusicCategory(MyMusicTransitionEnum.DesperateWithStress, "KA02", MySoundCuesEnum.MusDesperateWithStress_KA02);
            AddMusicCategory(MyMusicTransitionEnum.DesperateWithStress, "KA03", MySoundCuesEnum.MusDesperateWithStress_KA03);
            AddMusicCategory(MyMusicTransitionEnum.DesperateWithStress, "KA04", MySoundCuesEnum.MusDesperateWithStress_KA04);
            AddMusicCategory(MyMusicTransitionEnum.DesperateWithStress, "KA02b", MySoundCuesEnum.Mus_desperatestress_KA_2_longer_2b);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "MM01", MySoundCuesEnum.MusHeavyFight_MM01);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "MM02", MySoundCuesEnum.MusHeavyFight_MM02);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA01", MySoundCuesEnum.MusHeavyFight_KA01);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA02", MySoundCuesEnum.MusHeavyFight_KA02);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA03", MySoundCuesEnum.MusHeavyFight_KA03);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA04", MySoundCuesEnum.MusHeavyFight_KA04);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA05", MySoundCuesEnum.MusHeavyFight_KA05);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA06", MySoundCuesEnum.MusHeavyFight_KA06);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA07", MySoundCuesEnum.MusHeavyFight_KA07);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA15", MySoundCuesEnum.MusHeavyFight_KA15);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA16", MySoundCuesEnum.MusHeavyFight_KA16);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA17", MySoundCuesEnum.MusHeavyFight_KA17);
            AddMusicCategory(MyMusicTransitionEnum.HeavyFight, "KA19", MySoundCuesEnum.MusHeavyFight_KA19);
            AddMusicCategory(MyMusicTransitionEnum.Horror, "KA01", MySoundCuesEnum.MusHorror_KA01);
            AddMusicCategory(MyMusicTransitionEnum.Horror, "KA02", MySoundCuesEnum.MusHorror_KA02);
            AddMusicCategory(MyMusicTransitionEnum.Horror, "MM01", MySoundCuesEnum.MusHorror_MM01);
            AddMusicCategory(MyMusicTransitionEnum.Horror, "MM02", MySoundCuesEnum.MusHorror_MM02);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "MM01", MySoundCuesEnum.MusLightFight_MM01);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "MM02", MySoundCuesEnum.MusLightFight_MM02);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA03", MySoundCuesEnum.MusLightFight_KA03);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA04", MySoundCuesEnum.MusLightFight_KA04);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA05", MySoundCuesEnum.MusLightFight_KA05);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA07", MySoundCuesEnum.MusLightFight_KA07);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA08", MySoundCuesEnum.MusLightFight_KA08);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA10", MySoundCuesEnum.MusLightFight_KA10);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA11", MySoundCuesEnum.MusLightFight_KA11);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA12", MySoundCuesEnum.MusLightFight_KA12);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA27", MySoundCuesEnum.MusLightFight_KA27);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA26", MySoundCuesEnum.Mus_lightfight_KA_26);
            AddMusicCategory(MyMusicTransitionEnum.LightFight, "KA28", MySoundCuesEnum.Mus_lightfight_KA_28);
            AddMusicCategory(MyMusicTransitionEnum.MainMenu, "KAandMM", MySoundCuesEnum.MusMainMenu_KA_MM);
            AddMusicCategory(MyMusicTransitionEnum.Mystery, "MM01", MySoundCuesEnum.MusMystery_MM01);
            AddMusicCategory(MyMusicTransitionEnum.Mystery, "KA02", MySoundCuesEnum.MusMystery_KA02);
            AddMusicCategory(MyMusicTransitionEnum.Mystery, "KA01", MySoundCuesEnum.MusMystery_KA01);
            AddMusicCategory(MyMusicTransitionEnum.Mystery, "MM02", MySoundCuesEnum.MusMystery_MM02);
            AddMusicCategory(MyMusicTransitionEnum.SadnessOrDesperation, "MM01", MySoundCuesEnum.MusSadnessOrDesperation_MM01);
            AddMusicCategory(MyMusicTransitionEnum.SadnessOrDesperation, "MM02", MySoundCuesEnum.MusSadnessOrDesperation_MM02);
            AddMusicCategory(MyMusicTransitionEnum.SadnessOrDesperation, "KA02", MySoundCuesEnum.MusSadnessOrDesperation_KA02);
            AddMusicCategory(MyMusicTransitionEnum.SadnessOrDesperation, "KA01", MySoundCuesEnum.MusSadnessOrDesperation_KA01);
            AddMusicCategory(MyMusicTransitionEnum.Special, "MM01", MySoundCuesEnum.MusSpecial_MM01);
            AddMusicCategory(MyMusicTransitionEnum.Special, "MM02", MySoundCuesEnum.MusSpecial_MM02);
            AddMusicCategory(MyMusicTransitionEnum.Special, "MM03", MySoundCuesEnum.MusSpecial_MM03);
            AddMusicCategory(MyMusicTransitionEnum.Special, "MM04", MySoundCuesEnum.MusSpecial_MM04);
            AddMusicCategory(MyMusicTransitionEnum.Special, "MM05", MySoundCuesEnum.MusSpecial_MM05);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA01", MySoundCuesEnum.MusSpecial_KA01);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA02", MySoundCuesEnum.MusSpecial_KA02);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA03", MySoundCuesEnum.MusSpecial_KA03);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA04", MySoundCuesEnum.MusSpecial_KA04);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA05", MySoundCuesEnum.MusSpecial_KA05);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA06", MySoundCuesEnum.MusSpecial_KA06);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA08", MySoundCuesEnum.MusSpecial_KA08);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA09", MySoundCuesEnum.MusSpecial_KA09);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA10", MySoundCuesEnum.MusSpecial_KA10);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA11", MySoundCuesEnum.MusSpecial_KA11);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA12", MySoundCuesEnum.MusSpecial_KA12);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA13", MySoundCuesEnum.MusSpecial_KA13);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA14", MySoundCuesEnum.MusSpecial_KA14);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA15", MySoundCuesEnum.MusSpecial_KA15);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA16", MySoundCuesEnum.MusSpecial_KA16);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA04loop", MySoundCuesEnum.Mus_special_KA_4_loop);
            AddMusicCategory(MyMusicTransitionEnum.Special, "LostInTheDistance", MySoundCuesEnum.MusSpecial_LostInTheDistance);
            AddMusicCategory(MyMusicTransitionEnum.Special, "KA07", MySoundCuesEnum.MusSpecial_KA07);
            AddMusicCategory(MyMusicTransitionEnum.StealthAction, "MM01", MySoundCuesEnum.MusStealthAction_MM01);
            AddMusicCategory(MyMusicTransitionEnum.StealthAction, "MM02", MySoundCuesEnum.MusStealthAction_MM02);
            AddMusicCategory(MyMusicTransitionEnum.StealthAction, "KA01", MySoundCuesEnum.MusStealthAction_KA01);
            AddMusicCategory(MyMusicTransitionEnum.StealthAction, "KA03", MySoundCuesEnum.MusStealthAction_KA03);
            AddMusicCategory(MyMusicTransitionEnum.StressOrTimeRush, "MM01", MySoundCuesEnum.MusStressOrTimeRush_MM01);
            AddMusicCategory(MyMusicTransitionEnum.StressOrTimeRush, "MM02", MySoundCuesEnum.MusStressOrTimeRush_MM02);
            AddMusicCategory(MyMusicTransitionEnum.StressOrTimeRush, "KA01", MySoundCuesEnum.MusStressOrTimeRush_KA01);
            AddMusicCategory(MyMusicTransitionEnum.StressOrTimeRush, "KA02", MySoundCuesEnum.MusStressOrTimeRush_KA02);
            AddMusicCategory(MyMusicTransitionEnum.StressOrTimeRush, "KA03", MySoundCuesEnum.MusStressOrTimeRush_KA03);
            AddMusicCategory(MyMusicTransitionEnum.TensionBeforeAnAction, "MM01", MySoundCuesEnum.MusTensionBeforeAnAction_MM01);
            AddMusicCategory(MyMusicTransitionEnum.TensionBeforeAnAction, "MM02", MySoundCuesEnum.MusTensionBeforeAnAction_MM02);
            AddMusicCategory(MyMusicTransitionEnum.TensionBeforeAnAction, "KA01", MySoundCuesEnum.MusTensionBeforeAnAction_KA01);
            AddMusicCategory(MyMusicTransitionEnum.TensionBeforeAnAction, "KA02", MySoundCuesEnum.MusTensionBeforeAnAction_KA02);
            AddMusicCategory(MyMusicTransitionEnum.Victory, "MM01", MySoundCuesEnum.MusVictory_MM01);
            AddMusicCategory(MyMusicTransitionEnum.Victory, "MM02", MySoundCuesEnum.MusVictory_MM02);
            AddMusicCategory(MyMusicTransitionEnum.Victory, "KA01", MySoundCuesEnum.MusVictory_KA01);
            AddMusicCategory(MyMusicTransitionEnum.Victory, "KA02", MySoundCuesEnum.MusVictory_KA02);
            AddMusicCategory(MyMusicTransitionEnum.Victory, "KA06", MySoundCuesEnum.Mus_victory_KA_6);



            //Because of obfuscator
#if DEBUG
            //  This is just a check (assertion) if we didn't forget define something
            //for (int i = 0; i < m_cueParameters.Length; i++)
            foreach (MySoundCuesEnum soundCue in Enum.GetValues(typeof(MySoundCuesEnum)))
            {
                MyMwcLog.WriteLine("Validating cue parameters: " + (short)soundCue, SysUtils.LoggingOptions.VALIDATING_CUE_PARAMS);
                MyCommonDebugUtils.AssertRelease(m_cueParameters[(short)soundCue] != null, string.Format("Cue parameter {0}:{1} is missing", (short)soundCue, soundCue.ToString()));
            }
#endif

            MyMwcLog.DecreaseIndent(LoggingOptions.AUDIO);
            MyMwcLog.WriteLine("MyAudio.InitCueParameters() - END", LoggingOptions.AUDIO);
        }

        static void AddMusicCategory(MyMusicTransitionEnum musicTransition, string category, MySoundCuesEnum cueEnum)
        {
            if (m_musicTransitionCues[(int)musicTransition] == null)
            {
                m_musicTransitionCues[(int)musicTransition] = new Dictionary<string, MySoundCuesEnum>();
            }
            m_musicTransitionCues[(int)musicTransition].Add(category, cueEnum);
        }

        static void InitNonLoopableCuesLimitRemoveHelper()
        {
            MyMwcLog.WriteLine("InitNonLoopableCuesLimitRemoveHelper - START", LoggingOptions.AUDIO);

            //  Here we will remember every non-loopable cue we are playing. It will serve for limiting max count of same cues played too.
            if (MyAudioConstants.LIMIT_MAX_SAME_CUES == true)
            {
                m_nonLoopableCuesLimit = new HashSet<WeakReference>[GetNumberOfSounds()];
                //for (int i = 0; i < m_cueParameters.Length; i++)
                foreach (short i in Enum.GetValues(typeof(MySoundCuesEnum)))
                {
                    if (m_cueParameters[i].Value.Loopable == false)
                        m_nonLoopableCuesLimit[i] = new HashSet<WeakReference>();
                }
            }

            MyMwcLog.WriteLine("InitNonLoopableCuesLimitRemoveHelper - END", LoggingOptions.AUDIO);
        }

        public static int GetNumberOfSounds()
        {
            //return Enum.GetValues(typeof(MySoundCuesEnum)).Length;            
            return MyMwcUtils.GetMaxValueFromEnum<MySoundCuesEnum>() + 1;
        }

        // 0000216: MySounds - assert if all sounds cue exist in XACT project
        static void ValidateCues()
        {
            MyMwcLog.WriteLine("Validate cues - START", LoggingOptions.AUDIO);

            for (int i = 0; i < Enum.GetValues(typeof(MySoundCuesEnum)).Length; i++)
            {
                Cue cue = m_cueInfos[i].Prepare();
                MyCommonDebugUtils.AssertRelease(cue.IsValid(), "Cue \"" + MyEnumsToStrings.Sounds[i] + "\" does not exist in sound bank!");
                if (cue.IsValid())
                {
                    // All variables on all cues adds only 100 ms, rest of function does 3800 ms
                    CheckVar(MyCueVariableEnum.AmbVolume, cue);
                    CheckVar(MyCueVariableEnum.MusicProg1Progression, cue);
                    CheckVar(MyCueVariableEnum.Occluder, cue);
                    CheckVar(MyCueVariableEnum.Pitch, cue);
                    CheckVar(MyCueVariableEnum.RotatingSpeed, cue);
                    CheckVar(MyCueVariableEnum.Volume, cue);
                    CheckVar(MyCueVariableEnum.ShipAIdle, cue);
                    CheckVar(MyCueVariableEnum.ShipASpeed, cue);

                    cue.Destroy();
                }
            }

            MyMwcLog.WriteLine("Validate cues - END", LoggingOptions.AUDIO);
        }

        private static void CheckVar(MyCueVariableEnum variableEnum, Cue cue)
        {
            var index = MyXactVariables.GetVariableIndex(variableEnum);
            var cindex = cue.GetVariableIndex(MyXactVariables.GetVariableName(variableEnum));
            if (index == -1)
            {
                MyXactVariables.SetVariableIndex(variableEnum, cindex);
            }
            else
            {
                System.Diagnostics.Debug.Assert(index == cindex, "Variable index invalid");
            }
        }

        public static MySoundCuesEnum GetVersion2D(MySoundCuesEnum cueEnum3D)
        {
            MyCueParameters cueParams = m_cueParameters[(int)cueEnum3D].Value;
            if (cueParams.Wrapped2dEnum.HasValue)
            {
                return cueParams.Wrapped2dEnum.Value;
            }
            else
            {
                //  If we can't find 2D version for specified sound, we return 3D version so at least something is played
                return cueEnum3D;
            }
        }
    }
}
