﻿using System;
using System.Collections.Generic;
using System.Linq;
using MinerWars.AppCode.Game.Explosions;
using MinerWars.AppCode.Game.GUI.Helpers;
using MinerWars.AppCode.Game.Models;
using MinerWars.AppCode.Game.Utils;
using MinerWars.CommonLIB.AppCode.ObjectBuilders;
using MinerWars.CommonLIB.AppCode.ObjectBuilders.SubObjects;
using MinerWars.CommonLIB.AppCode.ObjectBuilders.SubObjects.Prefabs;
using MinerWars.CommonLIB.AppCode.Utils;
using SysUtils;
using SysUtils.Utils;


namespace MinerWars.AppCode.Game.Prefabs
{
    using Audio;
    using MinerWarsMath;

    public enum AxisTypesEnum : byte
    {
        X = 1,
        Y = 2,
        Z = 3
    }

    public enum DamageTypesEnum : byte
    {
        USE_OWN=0,
        SHARE_WITH_PARENT=1,
    }

    public enum BuildTypesEnum : byte
    {
        NONE = 0,
        BEAMS = 1,
        SHELLS = 2,
        MODULES = 3,
        CONNECTIONS = 4,
        DETAILS = 5,
        CUSTOM = 6, 
        LARGE_SHIPS = 7,        
    }

    public enum CategoryTypesEnum : ushort
    {
        LARGE = 1,
        MEDIUM = 2,
        SMALL = 3,
        FRAME = 4,
        PANELS = 5,
        CHAMBERS = 6,
        ARMORS = 7,
        FLIGHT = 8,
        SUPPLY = 9,
        LIFE_SUPPORT = 10,
        INDUSTRY = 11,
        WEAPONRY = 12,
        COMMUNICATIONS = 13,
        HANGARS = 14,
        MANNED_OBJECTS = 15,
        TUNNELS = 16,
        PIPES = 17,
        PASSAGES = 18,
        BILLBOARDS = 19,
        LIGHTS = 20,
        SIGNS = 21,
        OTHER = 22,
        SOUNDS = 23,
        PARTICLES = 24,
        TRAFFIC_SIGNS = 25,
        DOORS = 26,
        DOOR_CASES = 27,
        LARGE_SHIPS = 28,
        FOUNDATION_FACTORY = 29,
        SECURITY_CONTROL_HUB = 30,
        BANK_NODE = 31,
        GENERATOR = 32,
        SCANNER = 33,
        ALARM = 34,
        MSHIP = 35,
        CABLES = 36,
        CAMERA = 37,
        SHELVES = 38,
        BARRELS = 39,
    }

    public enum SubCategoryTypesEnum : ushort
    {
        LONG_TERM_THRUSTERS = 1,
        SHORT_TERM_THRUSTERS = 2,
        POWER_GENERATION = 3,
        POWER_STORAGE = 4,
        FUEL_GENERATION = 5,
        FUEL_STORAGE = 6,
        OXYGEN_GENERATION = 7,
        OXYGEN_STORAGE = 8,
        BIODOME = 9,
        DOCKS = 10,
        LOADING_UNLOADING = 11,
        ORE_STORAGE = 12,
        GAS_STORAGE = 13,
        REFINERY = 14,
        FACTORY = 15,
        WEAPON_MOUNT = 16,
        AMMUNITION_BOX = 17,
        LONG_DISTANCE_ANTENNA = 18,
        SHORT_DISTANCE_ANTENNA = 19,
        SMALL = 20,
        LARGE = 21,
        ESCAPE_PODS = 22,
        LIVING_QUARTERS = 23,
        BRIDGE = 24,
        PASSAGE = 25,
        STRAIGHT = 26,
        TURN = 27,
        JUNCTION = 28,
        OTHER = 29,        
        WIRES = 30,
        CONNECTION_BOXES = 31,
        CABLE_HOLDERS = 32,
    }

    [Flags]
    public enum PrefabTypesFlagEnum
    {
        Default = 1 << 0,
        Generator = 1 << 1,
        LargeShip = 1 << 2,
        Trader = 1 << 3,
        Hangar = 1 << 4,
        FoundationFactory = 1 << 5,
        Scanner = 1 << 6,
        Alarm = 1 << 7,
        Vendor = 1 << 8,
    }

    static class MyPrefabTypesFlagHelper
    {
        public static List<PrefabTypesFlagEnum> ParseToSingleFlags(PrefabTypesFlagEnum prefabTypesFlag)
        {
            List<PrefabTypesFlagEnum> prefabTypeSingleFlags = new List<PrefabTypesFlagEnum>();            
            foreach (PrefabTypesFlagEnum prefabTypeFlag in Enum.GetValues(typeof(PrefabTypesFlagEnum)))
            {
                if ((prefabTypesFlag & prefabTypeFlag) != 0)
                {
                    prefabTypeSingleFlags.Add(prefabTypeFlag);
                }
            }            
            return prefabTypeSingleFlags;
        }
    }

    static class MyPrefabConstants
    {
        /// <summary>
        /// Health ratio. Prefab starts smoking when damaged ratio is above this (and if dummy is defined).
        /// Also, some rotating prefabs may change sound when below this ratio.
        /// </summary>
        public const float DAMAGED_HEALTH = 0.5f;

        public const float SIZE_FOR_DOUBLE_MULTIPLIER = 300;
        public const float EXPLOSION_DELETE_MULTIPLIER_BY_SIZE = 2 / SIZE_FOR_DOUBLE_MULTIPLIER;
        public static Vector3 MIN_SIZE_INCREASE_FOR_EXPLOSION = new Vector3(8, 8, 8);

        /// <summary>
        /// The damage that a prefab deals to its surroundings when exploding
        /// </summary>
        public const float SMALL_PREFAB_DEAL_DAMAGE = 0f;
        public const float PANEL_PREFAB_DEAL_DAMAGE = 5f;
        public const float CHAMBER_PREFAB_DEAL_DAMAGE = 10f;
        public const float TUNNEL_PREFAB_DEAL_DAMAGE = 10f;
        public const float BARREL_PREFAB_DEAL_DAMAGE = 20f;
        public const float GENERATOR_PREFAB_DEAL_DAMAGE = 100f;
        public const float BIG_PREFAB_DEAL_DAMAGE = 150f;
        public const float HUGE_PREFAB_DEAL_DAMAGE = 500f;

        static private Dictionary<int, float> m_prefabDealthDamage = new Dictionary<int,float>();

        //static Dictionary<MyMwcObjectBuilder_Prefab_TypesEnum, MyPrefabConfiguration> m_prefabConfigurations;
        static readonly MyPrefabConfiguration[][] m_prefabConfigurations;

        static MyPrefabConstants()
        {
            SysUtils.Utils.MyMwcLog.WriteLine("MyPrefabConstants - START");
            
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.ALARM, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.ARMORS, PANEL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.BANK_NODE, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.BARRELS, BARREL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.BILLBOARDS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.CABLES, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.CAMERA, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.COMMUNICATIONS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.DOOR_CASES, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.DOORS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.FLIGHT, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.FOUNDATION_FACTORY, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.FRAME, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.GENERATOR, GENERATOR_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.HANGARS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.CHAMBERS, CHAMBER_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.INDUSTRY, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.LARGE, BIG_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.LARGE_SHIPS, HUGE_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.LIFE_SUPPORT, BIG_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.LIGHTS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.MANNED_OBJECTS, BIG_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.MEDIUM, CHAMBER_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.MSHIP, BIG_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.OTHER, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.PANELS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.PARTICLES, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.PASSAGES, TUNNEL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.PIPES, TUNNEL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.SCANNER, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.SECURITY_CONTROL_HUB, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.SHELVES, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.SIGNS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.SMALL, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.SOUNDS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.SUPPLY, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.TRAFFIC_SIGNS, SMALL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.TUNNELS, TUNNEL_PREFAB_DEAL_DAMAGE); 
            m_prefabDealthDamage.Add((int)CategoryTypesEnum.WEAPONRY, BIG_PREFAB_DEAL_DAMAGE); 


            m_prefabConfigurations = new MyPrefabConfiguration[MyMwcUtils.GetMaxValueFromEnum<MyMwcObjectBuilderTypeEnum>() + 1][];


            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P430_A01_PASSAGE_10M, new MyPrefabConfiguration(MyModelsEnum.p430_a01_passage_10m, MyModelsEnum.p430_a01_passage_10m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PASSAGES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p430_a01_passage_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P430_A02_PASSAGE_40M, new MyPrefabConfiguration(MyModelsEnum.p430_a02_passage_40m, MyModelsEnum.p430_a02_passage_40m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PASSAGES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p430_a02_passage_40m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P424_A01_PIPE_BASE, new MyPrefabConfiguration(MyModelsEnum.p424_a01_pipe_base, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p424_a01_pipe_base_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P423_A01_PIPE_JUNCTION, new MyPrefabConfiguration(MyModelsEnum.p423_a01_pipe_junction, MyModelsEnum.p423_a01_pipe_junction_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p423_a01_pipe_junction_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P422_A01_PIPE_TURN_90, new MyPrefabConfiguration(MyModelsEnum.p422_a01_pipe_turn_90, MyModelsEnum.p422_a01_pipe_turn_90_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p422_a01_pipe_turn_90_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P421_A01_PIPE_STRAIGHT_80M, new MyPrefabConfiguration(MyModelsEnum.p421_a01_pipe_straight_80m, MyModelsEnum.p421_a01_pipe_straight_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p421_a01_pipe_straight_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P421_A02_PIPE_STRAIGHT_40M, new MyPrefabConfiguration(MyModelsEnum.p421_a02_pipe_straight_40m, MyModelsEnum.p421_a02_pipe_straight_40m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p421_a02_pipe_straight_40m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P421_A03_PIPE_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p421_a03_pipe_straight_10m, MyModelsEnum.p421_a03_pipe_straight_10m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p421_a03_pipe_straight_10m_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_G01_JUNCTION_6AXES, new MyPrefabConfiguration(MyModelsEnum.p413_g01_junction_6axes, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_g01_junction_6axes_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_G02_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_g02_entrance_60m, MyModelsEnum.p414_g02_entrance_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p414_g02_entrance_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P410_G01_TURN_90_RIGHT_0M, new MyPrefabConfiguration(MyModelsEnum.p410_g01_turn_90_right_0m, MyModelsEnum.p410_g01_turn_90_right_0m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p410_g01_turn_90_right_0m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_G01_STRAIGHT_1, new MyPrefabConfiguration(MyModelsEnum.p411_g01_straight_1, MyModelsEnum.p411_g01_straight_1_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_g01_straight_1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_G02_STRAIGHT_2, new MyPrefabConfiguration(MyModelsEnum.p411_g02_straight_2, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_g02_straight_2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_G03_STRAIGHT_3, new MyPrefabConfiguration(MyModelsEnum.p411_g03_straight_3, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_g03_straight_3_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_G04_STRAIGHT_4, new MyPrefabConfiguration(MyModelsEnum.p411_g04_straight_4, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_g04_straight_4_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_F02_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_f02_entrance_60m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F21_TURN_S_UP, new MyPrefabConfiguration(MyModelsEnum.p412_f21_turn_s_up, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f21_turn_s_up_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F22_TURN_S_LEFT, new MyPrefabConfiguration(MyModelsEnum.p412_f22_turn_s_left, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f22_turn_s_left_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F23_TURN_S_RIGHT, new MyPrefabConfiguration(MyModelsEnum.p412_f23_turn_s_right, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f23_turn_s_right_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F24_TURN_S_DOWN, new MyPrefabConfiguration(MyModelsEnum.p412_f24_turn_s_down, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f24_turn_s_down_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F01_TURN_90_UP_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f01_turn_90_up_230m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F02_TURN_90_LEFT_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f02_turn_90_left_230m, MyModelsEnum.p412_f02_turn_90_left_230m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f02_turn_90_left_230m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F03_TURN_90_RIGHT_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f03_turn_90_right_230m, MyModelsEnum.p412_f03_turn_90_right_230m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f03_turn_90_right_230m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F04_TURN_90_DOWN_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f04_turn_90_down_230m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f04_turn_90_down_230m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_F01_STRAIGHT_1, new MyPrefabConfiguration(MyModelsEnum.p411_f01_straight_1, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_f01_straight_1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_F02_STRAIGHT_2, new MyPrefabConfiguration(MyModelsEnum.p411_f02_straight_2, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_f02_straight_2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_F03_STRAIGHT_3, new MyPrefabConfiguration(MyModelsEnum.p411_f03_straight_3, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_f03_straight_3_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_E01_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_e01_entrance_60m, MyModelsEnum.p414_e01_entrance_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p414_e01_entrance_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_E01_STRAIGHT_1, new MyPrefabConfiguration(MyModelsEnum.p411_e01_straight_1, MyModelsEnum.p411_e01_straight_1_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_e01_straight_1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_E02_STRAIGHT_2, new MyPrefabConfiguration(MyModelsEnum.p411_e02_straight_2, MyModelsEnum.p411_e02_straight_2_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_e02_straight_2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_E03_STRAIGHT_3, new MyPrefabConfiguration(MyModelsEnum.p411_e03_straight_3, MyModelsEnum.p411_e03_straight_3_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_e03_straight_3_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_E04_STRAIGHT_4, new MyPrefabConfiguration(MyModelsEnum.p411_e04_straight_4, MyModelsEnum.p411_e04_straight_4_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_e04_straight_4_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_E05_STRAIGHT_5, new MyPrefabConfiguration(MyModelsEnum.p411_e05_straight_5, MyModelsEnum.p411_e05_straight_5_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_e05_straight_5_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P415_D01_DOORCASE, new MyPrefabConfiguration(MyModelsEnum.p415_d01_doorcase, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.DOOR_CASES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p415_d01_doorcase_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P415_D02_DOOR1, new MyPrefabConfiguration(MyModelsEnum.p415_d02_door1, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P415_D03_DOOR2_A, new MyPrefabConfiguration(MyModelsEnum.p415_d03_door2_a, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p415_d03_door2_a_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P415_D03_DOOR2_B, new MyPrefabConfiguration(MyModelsEnum.p415_d03_door2_b, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p415_d03_door2_b_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_D01_JUNCTION_T_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p413_d01_junction_t_horizontal, MyModelsEnum.p413_d01_junction_t_horizontal_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_D03_JUNCTION_X_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p413_d03_junction_x_horizontal, MyModelsEnum.p413_d03_junction_x_horizontal_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_d03_junction_x_horizontal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_D01_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_d01_entrance_60m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p414_d01_entrance_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_D01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p411_d01_straight_10m, MyModelsEnum.p411_d01_straight_10m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_d01_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_D02_STRAIGHT_40M_WITH_HOLE, new MyPrefabConfiguration(MyModelsEnum.p411_d02_straight_40m_with_hole, MyModelsEnum.p411_d02_straight_40m_with_hole_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_d02_straight_40m_with_hole_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_D03_STRAIGHT_60M, new MyPrefabConfiguration(MyModelsEnum.p411_d03_straight_60m, MyModelsEnum.p411_d03_straight_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_d03_straight_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_D04_STRAIGHT_120M, new MyPrefabConfiguration(MyModelsEnum.p411_d04_straight_120m, MyModelsEnum.p411_d04_straight_120m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_d04_straight_120m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_D05_STRAIGHT_180M, new MyPrefabConfiguration(MyModelsEnum.p411_d05_straight_180m, MyModelsEnum.p411_d05_straight_180m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_d05_straight_180m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_C01_JUNCTION_T_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p413_c01_junction_t_horizontal, MyModelsEnum.p413_c01_junction_t_horizontal_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_c01_junction_t_horizontal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_C01_JUNCTION_X_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p413_c01_junction_x_horizontal, MyModelsEnum.p413_c01_junction_x_horizontal_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_c01_junction_x_horizontal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_C01_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_c01_entrance_60m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p414_c01_entrance_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_C01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p411_c01_straight_10m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_c01_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_C02_STRAIGHT_40M_WITH_HOLE, new MyPrefabConfiguration(MyModelsEnum.p411_c02_straight_40m_with_hole, MyModelsEnum.p411_c02_straight_40m_with_hole_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_c02_straight_40m_with_hole_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_C03_STRAIGHT_60M, new MyPrefabConfiguration(MyModelsEnum.p411_c03_straight_60m, MyModelsEnum.p411_c03_straight_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_c03_straight_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_C04_STRAIGHT_120M, new MyPrefabConfiguration(MyModelsEnum.p411_c04_straight_120m, MyModelsEnum.p411_c04_straight_120m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_c04_straight_120m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_C05_STRAIGHT_180M, new MyPrefabConfiguration(MyModelsEnum.p411_c05_straight_180m, MyModelsEnum.p411_c05_straight_180m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p411_c05_straight_180m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P415_B01_DOORCASE, new MyPrefabConfiguration(MyModelsEnum.p415_b01_doorcase, MyModelsEnum.p415_b01_doorcase_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.DOOR_CASES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p415_b01_doorcase_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P415_B02_DOOR, new MyPrefabConfiguration(MyModelsEnum.p415_b02_door, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, enabledInEditor: false));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_B01_JUNCTION_T_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p413_b01_junction_t_horizontal, MyModelsEnum.p413_b01_junction_t_horizontal_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_b01_junction_t_horizontal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_B02_JUNCTION_T_VERTICAL, new MyPrefabConfiguration(MyModelsEnum.p413_b02_junction_t_vertical, MyModelsEnum.p413_b02_junction_t_vertical_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_b02_junction_t_vertical_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_B02_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_b02_entrance_60m, MyModelsEnum.p414_b02_entrance_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B21_TURN_S_UP, new MyPrefabConfiguration(MyModelsEnum.p412_b21_turn_s_up, MyModelsEnum.p412_b21_turn_s_up_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_b21_turn_s_up_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B22_TURN_S_LEFT, new MyPrefabConfiguration(MyModelsEnum.p412_b22_turn_s_left, MyModelsEnum.p412_b22_turn_s_left_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B23_TURN_S_RIGHT, new MyPrefabConfiguration(MyModelsEnum.p412_b23_turn_s_right, MyModelsEnum.p412_b23_turn_s_right_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_b23_turn_s_right_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B24_TURN_S_DOWN, new MyPrefabConfiguration(MyModelsEnum.p412_b24_turn_s_down, MyModelsEnum.p412_b24_turn_s_down_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B11_TURN_90_UP_160M, new MyPrefabConfiguration(MyModelsEnum.p412_b11_turn_90_up_160m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B12_TURN_90_LEFT_160M, new MyPrefabConfiguration(MyModelsEnum.p412_b12_turn_90_left_160m, MyModelsEnum.p412_b12_turn_90_left_160m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B13_TURN_90_RIGHT_160M, new MyPrefabConfiguration(MyModelsEnum.p412_b13_turn_90_right_160m, MyModelsEnum.p412_b13_turn_90_right_160m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_b13_turn_90_right_160m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B14_TURN_90_DOWN_160M, new MyPrefabConfiguration(MyModelsEnum.p412_b14_turn_90_down_160m, MyModelsEnum.p412_b14_turn_90_down_160m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_b14_turn_90_down_160m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B01_TURN_90_UP_80M, new MyPrefabConfiguration(MyModelsEnum.p412_b01_turn_90_up_80m, MyModelsEnum.p412_b01_turn_90_up_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_b01_turn_90_up_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B02_TURN_90_LEFT_80M, new MyPrefabConfiguration(MyModelsEnum.p412_b02_turn_90_left_80m, MyModelsEnum.p412_b02_turn_90_left_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Left, collisionModelEnum: MyModelsEnum.p412_b02_turn_90_left_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B03_TURN_90_RIGHT_80M, new MyPrefabConfiguration(MyModelsEnum.p412_b03_turn_90_right_80m, MyModelsEnum.p412_b03_turn_90_right_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_b03_turn_90_right_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_B04_TURN_90_DOWN_80M, new MyPrefabConfiguration(MyModelsEnum.p412_b04_turn_90_down_80m, MyModelsEnum.p412_b04_turn_90_down_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_b04_turn_90_down_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_B01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p411_b01_straight_10m, MyModelsEnum.p411_b01_straight_10m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b01_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B02_STRAIGHT_30M_YELLOW, new MyPrefabConfiguration(MyModelsEnum.p411_b02_straight_30m_yellow, MyModelsEnum.p411_b02_straight_30m_yellow_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b02_straight_30m_yellow_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B03_STRAIGHT_320M, new MyPrefabConfiguration(MyModelsEnum.p411_b03_straight_320m, MyModelsEnum.p411_b03_straight_320m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b03_straight_320m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B04_STRAIGHT_80M_WITH_SIDE_GRATES, new MyPrefabConfiguration(MyModelsEnum.p411_b04_straight_80m_with_side_grates, MyModelsEnum.p411_b04_straight_80m_with_side_grates_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b04_straight_80m_with_side_grates_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B05_STRAIGHT_80M_WITH_SIDE_OPEN, new MyPrefabConfiguration(MyModelsEnum.p411_b05_straight_80m_with_side_open, MyModelsEnum.p411_b05_straight_80m_with_side_open_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b05_straight_80m_with_side_open_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B06_STRAIGHT_180M_CONCRETE, new MyPrefabConfiguration(MyModelsEnum.p411_b06_straight_180m_concrete, MyModelsEnum.p411_b06_straight_180m_concrete_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b06_straight_180m_concrete_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B06_STRAIGHT_200M, new MyPrefabConfiguration(MyModelsEnum.p411_b06_straight_200m, MyModelsEnum.p411_b06_straight_200m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b06_straight_200m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B07_STRAIGHT_180M_BLUE, new MyPrefabConfiguration(MyModelsEnum.p411_b07_straight_180m_blue, MyModelsEnum.p411_b07_straight_180m_blue_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b07_straight_180m_blue_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B09_STRAIGHT_30M_GRAY, new MyPrefabConfiguration(MyModelsEnum.p411_b09_straight_30m_gray, MyModelsEnum.p411_b09_straight_30m_gray_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b09_straight_30m_gray_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B11_STRAIGHT_220M, new MyPrefabConfiguration(MyModelsEnum.p411_b11_straight_220m, MyModelsEnum.p411_b11_straight_220m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b11_straight_220m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B12_STRAIGHT_160M_DARK_METAL, new MyPrefabConfiguration(MyModelsEnum.p411_b12_straight_160m_dark_metal, MyModelsEnum.p411_b12_straight_160m_dark_metal_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b12_straight_160m_dark_metal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.p411_B13_STRAIGHT_100M_TUBE_INSIDE, new MyPrefabConfiguration(MyModelsEnum.p411_b13_straight_100m_tube_inside, MyModelsEnum.p411_b13_straight_100m_tube_inside_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_b13_straight_100m_tube_inside_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_A01_JUNCTION_T_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p413_a01_junction_t_horizontal, MyModelsEnum.p413_a01_junction_t_horizontal_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p413_a01_junction_t_horizontal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_A02_JUNCTION_T_VERTICAL, new MyPrefabConfiguration(MyModelsEnum.p413_a02_junction_t_vertical, MyModelsEnum.p413_a02_junction_t_vertical_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p413_a02_junction_t_vertical_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_A01_ENTRANCE_30M, new MyPrefabConfiguration(MyModelsEnum.p414_a01_entrance_30m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_A02_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_a02_entrance_60m, MyModelsEnum.p414_a02_entrance_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p414_a02_entrance_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A21_TURN_S_UP, new MyPrefabConfiguration(MyModelsEnum.p412_a21_turn_s_up, MyModelsEnum.p412_a21_turn_s_up_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a21_turn_s_up_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A22_TURN_S_LEFT, new MyPrefabConfiguration(MyModelsEnum.p412_a22_turn_s_left, MyModelsEnum.p412_a22_turn_s_left_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a22_turn_s_left_COL, previewAngle: MyPreviewPointOfViewEnum.RearLeft));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A23_TURN_S_RIGHT, new MyPrefabConfiguration(MyModelsEnum.p412_a23_turn_s_right, MyModelsEnum.p412_a23_turn_s_right_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a23_turn_s_right_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A24_TURN_S_DOWN, new MyPrefabConfiguration(MyModelsEnum.p412_a24_turn_s_down, MyModelsEnum.p412_a24_turn_s_down_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a24_turn_s_down_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A11_TURN_90_UP_160M, new MyPrefabConfiguration(MyModelsEnum.p412_a11_turn_90_up_160m, MyModelsEnum.p412_a11_turn_90_up_160m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a11_turn_90_up_160m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A12_TURN_90_LEFT_160M, new MyPrefabConfiguration(MyModelsEnum.p412_a12_turn_90_left_160m, MyModelsEnum.p412_a12_turn_90_left_160m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a12_turn_90_left_160m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A13_TURN_90_RIGHT_160M, new MyPrefabConfiguration(MyModelsEnum.p412_a13_turn_90_right_160m, MyModelsEnum.p412_a13_turn_90_right_160m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a13_turn_90_right_160m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A14_TURN_90_DOWN_160M, new MyPrefabConfiguration(MyModelsEnum.p412_a14_turn_90_down_160m, MyModelsEnum.p412_a14_turn_90_down_160m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a14_turn_90_down_160m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A01_TURN_90_UP_80M, new MyPrefabConfiguration(MyModelsEnum.p412_a01_turn_90_up_80m, MyModelsEnum.p412_a01_turn_90_up_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a01_turn_90_up_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A02_TURN_90_LEFT_80M, new MyPrefabConfiguration(MyModelsEnum.p412_a02_turn_90_left_80m, MyModelsEnum.p412_a02_turn_90_left_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a02_turn_90_left_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A03_TURN_90_RIGHT_80M, new MyPrefabConfiguration(MyModelsEnum.p412_a03_turn_90_right_80m, MyModelsEnum.p412_a03_turn_90_right_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a03_turn_90_right_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_A04_TURN_90_DOWN_80M, new MyPrefabConfiguration(MyModelsEnum.p412_a04_turn_90_down_80m, MyModelsEnum.p412_a04_turn_90_down_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p412_a04_turn_90_down_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_A01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p411_a01_straight_10m, MyModelsEnum.p411_a01_straight_10m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_a01_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_A02_STRAIGHT_60M_WITH_HOLE, new MyPrefabConfiguration(MyModelsEnum.p411_a02_straight_60m_with_hole, MyModelsEnum.p411_a02_straight_60m_with_hole_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_a02_straight_60m_with_hole_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_A03_STRAIGHT_120M, new MyPrefabConfiguration(MyModelsEnum.p411_a03_straight_120m, MyModelsEnum.p411_a03_straight_120m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_a03_straight_120m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_A04_STRAIGHT_80M, new MyPrefabConfiguration(MyModelsEnum.p411_a04_straight_80m, MyModelsEnum.p411_a04_straight_80m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_a04_straight_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_A05_STRAIGHT_80M_WITH_EXTENSION, new MyPrefabConfiguration(MyModelsEnum.p411_a05_straight_80m_with_extension, MyModelsEnum.p411_a05_straight_80m_with_extension_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_a05_straight_80m_with_extension_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_D02_JUNCTION_T_VERTICAL, new MyPrefabConfiguration(MyModelsEnum.p413_d02_junction_t_vertical, MyModelsEnum.p413_d02_junction_t_vertical_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_d02_junction_t_vertical_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P413_D04_JUNCTION_X_VERTICAL, new MyPrefabConfiguration(MyModelsEnum.p413_d04_junction_x_vertical, MyModelsEnum.p413_d04_junction_x_vertical_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p413_d04_junction_x_vertical_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F11_TURN_90_UP_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f11_turn_90_up_230m, MyModelsEnum.p412_f11_turn_90_up_230m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f11_turn_90_up_230m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F12_TURN_90_LEFT_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f12_turn_90_left_230m, MyModelsEnum.p412_f12_turn_90_left_230m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f12_turn_90_left_230m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F13_TURN_90_RIGHT_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f13_turn_90_right_230m, MyModelsEnum.p412_f13_turn_90_right_230m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f13_turn_90_right_230m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P412_F14_TURN_90_DOWN_230M, new MyPrefabConfiguration(MyModelsEnum.p412_f14_turn_90_down_230m, MyModelsEnum.p412_f14_turn_90_down_230m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p412_f14_turn_90_down_230m_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_F04_STRAIGHT_4, new MyPrefabConfiguration(MyModelsEnum.p411_f04_straight_4, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_f04_straight_4_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_F05_STRAIGHT_5, new MyPrefabConfiguration(MyModelsEnum.p411_f05_straight_5, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_f05_straight_5_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_G05_STRAIGHT_5, new MyPrefabConfiguration(MyModelsEnum.p411_g05_straight_5, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_g05_straight_5_COL));

            
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_A02_DOOR_LEFT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_A02_DOOR_LEFT, MyModelsEnum.p415_a02_door_left, "door_left_open", "door_left_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, collisionModelEnum: MyModelsEnum.p415_a02_door_left_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_A02_DOOR_RIGHT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_A02_DOOR_LEFT, MyModelsEnum.p415_a02_door_right, "door_right_open", "door_right_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, collisionModelEnum: MyModelsEnum.p415_a02_door_right_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematic, (int)MyMwcObjectBuilder_PrefabKinematic_TypesEnum.P415_A01_DOORCASE, 
                new MyPrefabConfigurationKinematic(
                MyModelsEnum.p415_a01_doorcase, MyModelsEnum.p415_a01_doorcase_LOD1,
                new List<MyPrefabConfigurationKinematicPart>() {
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_A02_DOOR_LEFT), 
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_A02_DOOR_RIGHT)
                },
                MySoundCuesEnum.MovDoor1AOpen, null, MySoundCuesEnum.MovDoor1AClose,//soundhere
                1.6f, 1.4f,
                BuildTypesEnum.CONNECTIONS,
                CategoryTypesEnum.DOOR_CASES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p415_a01_doorcase_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C02_DOOR1_LEFT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C02_DOOR1_LEFT, MyModelsEnum.p415_c02_door1_left, "door_left_open", "door_left_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, collisionModelEnum: MyModelsEnum.p415_c02_door1_left_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C02_DOOR1_RIGHT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C02_DOOR1_LEFT, MyModelsEnum.p415_c02_door1_right, "door_right_open", "door_right_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, collisionModelEnum: MyModelsEnum.p415_c02_door1_right_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematic, (int)MyMwcObjectBuilder_PrefabKinematic_TypesEnum.P415_C01_DOOR1, new MyPrefabConfigurationKinematic(
                MyModelsEnum.p415_c01_doorcase, MyModelsEnum.p415_c01_doorcase_LOD1,
                new List<MyPrefabConfigurationKinematicPart>() {
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C02_DOOR1_LEFT), 
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C02_DOOR1_RIGHT) 
                },
                MySoundCuesEnum.MovDoor1BOpen, null, MySoundCuesEnum.MovDoor1BClose,//soundhere
                1.6f, 1.4f,
                BuildTypesEnum.CONNECTIONS,
                CategoryTypesEnum.DOOR_CASES, null, MyMaterialType.METAL, MyModelsEnum.p415_c01_doorcase_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A, MyModelsEnum.p415_c03_door2_a, "door_left_open", "door_left_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B, MyModelsEnum.p415_c03_door2_b, "door_right_open", "door_right_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_LEFT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_LEFT, MyModelsEnum.p415_c03_door2_b_left, "door_left_open", "door_left_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, MyModelsEnum.p415_c03_door2_b_left_LOD1, MyModelsEnum.p415_c03_door2_b_left_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_RIGHT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_RIGHT, MyModelsEnum.p415_c03_door2_b_right, "door_right_open", "door_right_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, MyModelsEnum.p415_c03_door2_b_right_LOD1, MyModelsEnum.p415_c03_door2_b_right_COL));


            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematic, (int)MyMwcObjectBuilder_PrefabKinematic_TypesEnum.P415_C01_DOOR2, new MyPrefabConfigurationKinematic(
                MyModelsEnum.p415_c01_doorcase, null,
                new List<MyPrefabConfigurationKinematicPart>() {
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_LEFT), 
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_RIGHT) 
                },
                MySoundCuesEnum.MovDoor2AOpen, null, MySoundCuesEnum.MovDoor2AClose,//soundhere
                2.5f, 2.7f,
                BuildTypesEnum.CONNECTIONS,
                CategoryTypesEnum.DOOR_CASES, null, MyMaterialType.METAL, MyModelsEnum.p415_c01_doorcase_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A_LEFT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A_LEFT, MyModelsEnum.p415_c03_door2_a_left, "door_left_open", "door_left_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, MyModelsEnum.p415_c03_door2_a_left_LOD1, MyModelsEnum.p415_c03_door2_a_left_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A_RIGHT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A_RIGHT, MyModelsEnum.p415_c03_door2_a_right, "door_right_open", "door_right_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN, MyModelsEnum.p415_c03_door2_a_right_LOD1, MyModelsEnum.p415_c03_door2_a_right_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematic, (int)MyMwcObjectBuilder_PrefabKinematic_TypesEnum.P415_C01_DOOR3, new MyPrefabConfigurationKinematic(
                MyModelsEnum.p415_c01_doorcase, null,
                new List<MyPrefabConfigurationKinematicPart>() {
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A_LEFT), 
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_A_RIGHT) 
                },
                MySoundCuesEnum.MovDoor2BOpen, null, MySoundCuesEnum.MovDoor2BClose,//soundhere
                3.0f, 4.0f,
                BuildTypesEnum.CONNECTIONS,
                CategoryTypesEnum.DOOR_CASES, null, MyMaterialType.METAL, MyModelsEnum.p415_c01_doorcase_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematic, (int)MyMwcObjectBuilder_PrefabKinematic_TypesEnum.P415_C01_DOOR4, new MyPrefabConfigurationKinematic(
                MyModelsEnum.p415_c01_doorcase, null,
                new List<MyPrefabConfigurationKinematicPart>() {
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_LEFT), 
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P415_C03_DOOR2_B_RIGHT) 
                },
                MySoundCuesEnum.MovDoor3AOpen, null, MySoundCuesEnum.MovDoor3AClose,//soundhere
                3.5f, 3.0f,
                BuildTypesEnum.CONNECTIONS,
                CategoryTypesEnum.DOOR_CASES, null, MyMaterialType.METAL, MyModelsEnum.p415_c01_doorcase_COL));

            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P411_A05_STRAIGHT_80M_WITH_EXTENSION, new MyPrefabConfiguration(MyModelsEnum.p411_a05_straight_80m_with_extension, MyModelsEnum.p411_a05_straight_80m_with_extension_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, null, MyModelsEnum.p411_a05_straight_80m_with_extension_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1, new MyPrefabConfiguration(MyModelsEnum.p341_a01_open_dock_variation1, null, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p341_a01_open_dock_variation1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1_DOORLEFT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1_DOORLEFT, MyModelsEnum.p341_a01_open_dock_variation1_doorleft, "door_left_open", "door_left_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1_DOORRIGHT,
                new MyPrefabConfigurationKinematicPart((int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1_DOORRIGHT, MyModelsEnum.p341_a01_open_dock_variation1_doorright, "door_right_open", "door_right_close", BuildTypesEnum.NONE, CategoryTypesEnum.DOORS, null, MyMaterialType.METAL, DamageTypesEnum.USE_OWN));


            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematic, (int)MyMwcObjectBuilder_PrefabKinematic_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1, new MyPrefabConfigurationKinematic(
                MyModelsEnum.p341_a01_open_dock_variation1, MyModelsEnum.p341_a01_open_dock_variation1_LOD1,
                new List<MyPrefabConfigurationKinematicPart>() {
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1_DOORLEFT), 
                    (MyPrefabConfigurationKinematicPart)GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabKinematicPart, (int)MyMwcObjectBuilder_PrefabKinematicPart_TypesEnum.P341_A01_OPEN_DOCK_VARIATION1_DOORRIGHT) 
                },
                MySoundCuesEnum.MovDoor1AOpen, null, MySoundCuesEnum.MovDoor1AClose,//soundhere
                1.6f, 1.4f,
                BuildTypesEnum.MODULES,
                CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, MyModelsEnum.p341_a01_open_dock_variation1_COL));




            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P382_E01_BRIDGE5, new MyPrefabConfiguration(MyModelsEnum.p382_e01_bridge5, MyModelsEnum.p382_e01_bridge5_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P382_D01_BRIDGE4, new MyPrefabConfiguration(MyModelsEnum.p382_d01_bridge4, MyModelsEnum.p382_d01_bridge4_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p382_d01_bridge4_COL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P382_C01_BRIDGE3, new MyPrefabConfiguration(MyModelsEnum.p382_c01_bridge3, MyModelsEnum.p382_c01_bridge3_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p382_c01_bridge3_COL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P382_B01_BRIDGE2, new MyPrefabConfiguration(MyModelsEnum.p382_b01_bridge2, null, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P382_A01_BRIDGE1, new MyPrefabConfiguration(MyModelsEnum.p382_a01_bridge1, MyModelsEnum.p382_a01_bridge1_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 1f, collisionModelEnum: MyModelsEnum.p382_a01_bridge1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_C01_BUILDING4, new MyPrefabConfiguration(MyModelsEnum.p381_c01_building4, MyModelsEnum.p381_c01_building4_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 1f, collisionModelEnum: MyModelsEnum.p381_c01_building4_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_C01_BUILDING3, new MyPrefabConfiguration(MyModelsEnum.p381_c01_building3, MyModelsEnum.p381_c01_building3_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p381_c01_building3_COL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_B01_BUILDING2, new MyPrefabConfiguration(MyModelsEnum.p381_b01_building2, MyModelsEnum.p381_b01_building2_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p381_b01_building2_COL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_A01_BUILDING1, new MyPrefabConfiguration(MyModelsEnum.p381_a01_building1, MyModelsEnum.p381_a01_building1_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 1f, collisionModelEnum: MyModelsEnum.p381_a01_building1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabHangar, (int)MyMwcObjectBuilder_PrefabHangar_TypesEnum.VENDOR, new MyPrefabConfigurationHangar(MyModelsEnum.vendor, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, PrefabTypesFlagEnum.Vendor, collisionModel: MyModelsEnum.vendor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabHangar, (int)MyMwcObjectBuilder_PrefabHangar_TypesEnum.HANGAR, new MyPrefabConfigurationHangar(MyModelsEnum.hangar_screen, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModel: MyModelsEnum.hangar_screen_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P362_A01_SHORT_DISTANCE_ANTENNA, new MyPrefabConfiguration(MyModelsEnum.p362_a01_short_distance_antenna, MyModelsEnum.p362_a01_short_distance_antenna_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.COMMUNICATIONS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p362_a01_short_distance_antenna_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P361_A01_LONG_DISTANCE_ANTENNA, new MyPrefabConfiguration(MyModelsEnum.p361_a01_long_distance_antenna, MyModelsEnum.p361_a01_long_distance_antenna_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.COMMUNICATIONS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p361_a01_long_distance_antenna_COL, explosionParticleEffectScale: 1.6f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P351_A01_WEAPON_MOUNT, new MyPrefabConfiguration(MyModelsEnum.p351_a01_weapon_mount, MyModelsEnum.p351_a01_weapon_mount_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p351_a01_weapon_mount_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P345_A01_REFINERY, new MyPrefabConfiguration(MyModelsEnum.p345_a01_refinery, MyModelsEnum.p345_a01_refinery_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p345_a01_refinery_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P344_A01_CONTAINER_ARM_FILLED, new MyPrefabConfiguration(MyModelsEnum.p344_a01_container_arm_filled, MyModelsEnum.p344_a01_container_arm_filled_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p344_a01_container_arm_filled_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P344_A02_CONTAINER_ARM_EMPTY, new MyPrefabConfiguration(MyModelsEnum.p344_a02_container_arm_empty, MyModelsEnum.p344_a02_container_arm_empty_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p344_a02_container_arm_empty_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P343_A01_ORE_STORAGE, new MyPrefabConfiguration(MyModelsEnum.p343_a01_ore_storage, MyModelsEnum.p343_a01_ore_storage_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p343_a01_ore_storage_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P342_A01_LOADING_BAY, new MyPrefabConfiguration(MyModelsEnum.p342_a01_loading_bay, MyModelsEnum.p342_a01_loading_bay_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p342_a01_loading_bay_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P341_B01_OPEN_DOCK_VARIATION1, new MyPrefabConfiguration(MyModelsEnum.p341_b01_open_dock_variation1, MyModelsEnum.p341_b01_open_dock_variation1_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p341_b01_open_dock_variation1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P341_B02_OPEN_DOCK_VARIATION2, new MyPrefabConfiguration(MyModelsEnum.p341_b02_open_dock_variation2, MyModelsEnum.p341_b02_open_dock_variation2_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p341_b02_open_dock_variation2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P341_A02_OPEN_DOCK_VARIATION2, new MyPrefabConfiguration(MyModelsEnum.p341_a02_open_dock_variation2, MyModelsEnum.p341_a02_open_dock_variation2_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p341_a02_open_dock_variation2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P333_A01_HYDROPONIC_BUILDING, new MyPrefabConfiguration(MyModelsEnum.p333_a01_hydroponic_building, MyModelsEnum.p333_a01_hydroponic_building_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.LIFE_SUPPORT, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p333_a01_hydroponic_building_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P332_A01_OXYGEN_STORAGE, new MyPrefabConfiguration(MyModelsEnum.p332_a01_oxygen_storage, null, BuildTypesEnum.MODULES, CategoryTypesEnum.LIFE_SUPPORT, null, MyMaterialType.METAL, null, MyModelsEnum.p332_a01_oxygen_storage_COL, explosionType: MyExplosionTypeEnum.SMALL_EXPLOSION));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P331_A01_OXYGEN_GENERATOR, new MyPrefabConfiguration(MyModelsEnum.p331_a01_oxygen_generator, MyModelsEnum.p331_a01_oxygen_generator_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.LIFE_SUPPORT, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p331_a01_oxygen_generator_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P324B01_FUEL_STORAGE_B, new MyPrefabConfiguration(MyModelsEnum.p324b01_fuel_storage_b, MyModelsEnum.p324b01_fuel_storage_b_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p324b01_fuel_storage_b_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P324A01_FUEL_STORAGE_A, new MyPrefabConfiguration(MyModelsEnum.p324a01_fuel_storage_a, MyModelsEnum.p324a01_fuel_storage_a_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p324a01_fuel_storage_a_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P323A01_FUEL_GENERATOR, new MyPrefabConfiguration(MyModelsEnum.p323a01_fuel_generator, MyModelsEnum.p323a01_fuel_generator_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p323a01_fuel_generator_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P322A01_BATTERY, new MyPrefabConfiguration(MyModelsEnum.p322a01_battery, MyModelsEnum.p322a01_battery_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.EMP_EXPLOSION, collisionModelEnum: MyModelsEnum.p322a01_battery_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P321B01_NUCLEAR_REACTOR, new MyPrefabConfiguration(MyModelsEnum.p321b01_nuclear_reactor, MyModelsEnum.p321b01_nuclear_reactor_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.NUCLEAR_EXPLOSION));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P321A01_SOLAR_PANEL, new MyPrefabConfiguration(MyModelsEnum.p321a01_solar_panel, MyModelsEnum.p321a01_solar_panel_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p321a01_solar_panel_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P321_D01_BIG_SOLAR_PANEL, new MyPrefabConfiguration(MyModelsEnum.p321d01_big_solar_panel, MyModelsEnum.p321d01_big_solar_panel_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P312A01_SHORT_TERM_THRUSTER_LATITUDE, new MyPrefabConfiguration(MyModelsEnum.p312a01_short_term_thruster_latitude, MyModelsEnum.p312a01_short_term_thruster_latitude_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.PLASMA_EXPLOSION, collisionModelEnum: MyModelsEnum.p312a01_short_term_thruster_latitude_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P312A02_SHORT_TERM_THRUSTER_LATERAL, new MyPrefabConfiguration(MyModelsEnum.p312a02_short_term_thruster_lateral, MyModelsEnum.p312a02_short_term_thruster_lateral_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.PLASMA_EXPLOSION, collisionModelEnum: MyModelsEnum.p312a02_short_term_thruster_lateral_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P311A01_LONG_TERM_THRUSTER, new MyPrefabConfiguration(MyModelsEnum.p311a01_long_term_thruster, MyModelsEnum.p311a01_long_term_thruster_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.PLASMA_EXPLOSION));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A01_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a01_armor, MyModelsEnum.p231a01_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle:MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a01_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A02_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a02_armor, MyModelsEnum.p231a02_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a02_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A03_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a03_armor, MyModelsEnum.p231a03_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeftLeft, collisionModelEnum: MyModelsEnum.p231a03_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A04_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a04_armor, MyModelsEnum.p231a04_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeftLeft, collisionModelEnum: MyModelsEnum.p231a04_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A05_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a05_armor, MyModelsEnum.p231a05_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a05_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A06_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a06_armor, MyModelsEnum.p231a06_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a06_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A07_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a07_armor, MyModelsEnum.p231a07_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a07_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A08_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a08_armor, MyModelsEnum.p231a08_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a08_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A09_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a09_armor, MyModelsEnum.p231a09_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a09_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A10_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a10_armor, MyModelsEnum.p231a10_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a10_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A11_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a11_armor, MyModelsEnum.p231a11_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.FrontLeft, collisionModelEnum: MyModelsEnum.p231a11_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A12_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a12_armor, MyModelsEnum.p231a12_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.FrontLeft, collisionModelEnum: MyModelsEnum.p231a12_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A13_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a13_armor, MyModelsEnum.p231a13_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.FrontLeft, collisionModelEnum: MyModelsEnum.p231a13_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A14_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a14_armor, MyModelsEnum.p231a14_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top, collisionModelEnum: MyModelsEnum.p231a14_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A15_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a15_armor, MyModelsEnum.p231a15_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.FrontLeft, collisionModelEnum: MyModelsEnum.p231a15_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A16_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a16_armor, MyModelsEnum.p231a16_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a16_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A17_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a17_armor, MyModelsEnum.p231a17_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a17_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231A18_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231a18_armor, MyModelsEnum.p231a18_armor_LOD01, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.p231a18_armor_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221A01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221a01_chamber_v1, MyModelsEnum.p221a01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221a01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221B01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221b01_chamber_v1, MyModelsEnum.p221b01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221b01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221C01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221c01_chamber_v1, MyModelsEnum.p221c01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221c01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221D01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221d01_chamber_v1, MyModelsEnum.p221d01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221d01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221E01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221e01_chamber_v1, MyModelsEnum.p221e01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221e01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221F01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221f01_chamber_v1, MyModelsEnum.p221f01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221f01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221G01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221g01_chamber_v1, MyModelsEnum.p221g01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221g01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221H01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221h01_chamber_v1, MyModelsEnum.p221h01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221h01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221J01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221j01_chamber_v1, MyModelsEnum.p221j01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221j01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221K01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221k01_chamber_v1, MyModelsEnum.p221k01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, null));


            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211H01_PANEL_535MX130M, new MyPrefabConfiguration(MyModelsEnum.p211h01_panel_535mx130m, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211h01_panel_535mx130m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211G01_PANEL_120MX60M, new MyPrefabConfiguration(MyModelsEnum.p211g01_panel_120mx60m, MyModelsEnum.p211g01_panel_120mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211g01_panel_120mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211G02_PANEL_60MX60M, new MyPrefabConfiguration(MyModelsEnum.p211g02_panel_60mx60m, MyModelsEnum.p211g02_panel_60mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211g02_panel_60mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211G03_PANEL_60MX30M, new MyPrefabConfiguration(MyModelsEnum.p211g03_panel_60mx30m, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211F01_PANEL_120MX60M, new MyPrefabConfiguration(MyModelsEnum.p211f01_panel_120mx60m, MyModelsEnum.p211f01_panel_120mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211f01_panel_120mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211F02_PANEL_60MX60M, new MyPrefabConfiguration(MyModelsEnum.p211f02_panel_60mx60m, MyModelsEnum.p211f02_panel_60mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211f02_panel_60mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211F03_PANEL_60MX30M, new MyPrefabConfiguration(MyModelsEnum.p211f03_panel_60mx30m, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211f03_panel_60mx30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211E01_PANEL_120MX60M, new MyPrefabConfiguration(MyModelsEnum.p211e01_panel_120mx60m, MyModelsEnum.p211e01_panel_120mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211e01_panel_120mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211E02_PANEL_60MX60M, new MyPrefabConfiguration(MyModelsEnum.p211e02_panel_60mx60m, MyModelsEnum.p211e02_panel_60mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211e02_panel_60mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211E03_PANEL_60MX30M, new MyPrefabConfiguration(MyModelsEnum.p211e03_panel_60mx30m, MyModelsEnum.p211e03_panel_60mx30m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211e03_panel_60mx30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211D01_PANEL_120MX60M, new MyPrefabConfiguration(MyModelsEnum.p211d01_panel_120mx60m, MyModelsEnum.p211d01_panel_120mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211d01_panel_120mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211D02_PANEL_60MX60M, new MyPrefabConfiguration(MyModelsEnum.p211d02_panel_60mx60m, MyModelsEnum.p211d02_panel_60mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211d02_panel_60mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211D03_PANEL_60MX30M, new MyPrefabConfiguration(MyModelsEnum.p211d03_panel_60mx30m, MyModelsEnum.p211d03_panel_60mx30m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211d03_panel_60mx30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211C01_PANEL_120MX60M, new MyPrefabConfiguration(MyModelsEnum.p211c01_panel_120mx60m, MyModelsEnum.p211c01_panel_120mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211C02_PANEL_60MX60M, new MyPrefabConfiguration(MyModelsEnum.p211c02_panel_60mx60m, MyModelsEnum.p211c02_panel_60mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211c02_panel_60mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211C03_PANEL_60MX30M, new MyPrefabConfiguration(MyModelsEnum.p211c03_panel_60mx30m, MyModelsEnum.p211c03_panel_60mx30m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211B01_PANEL_120MX60M, new MyPrefabConfiguration(MyModelsEnum.p211b01_panel_120mx60m, MyModelsEnum.p211b01_panel_120mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211b01_panel_120mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211B02_PANEL_60MX60M, new MyPrefabConfiguration(MyModelsEnum.p211b02_panel_60mx60m, MyModelsEnum.p211b02_panel_60mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211b02_panel_60mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211B03_PANEL_60MX30M, new MyPrefabConfiguration(MyModelsEnum.p211b03_panel_60mx30m, MyModelsEnum.p211b03_panel_60mx30m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211b03_panel_60mx30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211A01_PANEL_120MX60M, new MyPrefabConfiguration(MyModelsEnum.p211a01_panel_120mx60m, MyModelsEnum.p211a01_panel_120mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211a01_panel_120mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211A02_PANEL_60MX60M, new MyPrefabConfiguration(MyModelsEnum.p211a02_panel_60mx60m, MyModelsEnum.p211a02_panel_60mx60m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211a02_panel_60mx60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P211A03_PANEL_60MX30M, new MyPrefabConfiguration(MyModelsEnum.p211a03_panel_60mx30m, MyModelsEnum.p211a03_panel_60mx30m_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p211a03_panel_60mx30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142B01_CAGE_EMPTY, new MyPrefabConfiguration(MyModelsEnum.p142b01_cage_empty, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142B02_CAGE_HALFCUT, new MyPrefabConfiguration(MyModelsEnum.p142b02_cage_halfcut, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142B03_CAGE_WITH_CORNERS, new MyPrefabConfiguration(MyModelsEnum.p142b03_cage_with_corners, MyModelsEnum.p142b03_cage_with_corners_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p142b03_cage_with_corners_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142B11_CAGE_PILLAR, new MyPrefabConfiguration(MyModelsEnum.p142b11_cage_pillar, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p142b11_cage_pillar_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142B12_CAGE_EDGE, new MyPrefabConfiguration(MyModelsEnum.p142b12_cage_edge, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142A01_CAGE_EMPTY, new MyPrefabConfiguration(MyModelsEnum.p142a01_cage_empty, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p142a01_cage_empty_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142A02_CAGE_HALFCUT, new MyPrefabConfiguration(MyModelsEnum.p142a02_cage_halfcut, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p142a02_cage_halfcut_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142A03_CAGE_WITH_CORNERS, new MyPrefabConfiguration(MyModelsEnum.p142a03_cage_with_corners, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p142a03_cage_with_corners_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142A11_CAGE_PILLAR, new MyPrefabConfiguration(MyModelsEnum.p142a11_cage_pillar, MyModelsEnum.p142a11_cage_pillar_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p142a11_cage_pillar_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P142A12_CAGE_EDGE, new MyPrefabConfiguration(MyModelsEnum.p142a12_cage_edge, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141B01_THICK_FRAME_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p141b01_thick_frame_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141B02_THICK_FRAME_STRAIGHT_60M, new MyPrefabConfiguration(MyModelsEnum.p141b02_thick_frame_straight_60m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p141b02_thick_frame_straight_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141B11_THICK_FRAME_EDGE, new MyPrefabConfiguration(MyModelsEnum.p141b11_thick_frame_edge, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141B12_THICK_FRAME_CORNER, new MyPrefabConfiguration(MyModelsEnum.p141b12_thick_frame_corner, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141B31_THICK_FRAME_JOINT, new MyPrefabConfiguration(MyModelsEnum.p141b31_thick_frame_joint, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p141b31_thick_frame_joint_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141A01_THICK_FRAME_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p141a01_thick_frame_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p141a01_thick_frame_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141A02_THICK_FRAME_STRAIGHT_60M, new MyPrefabConfiguration(MyModelsEnum.p141a02_thick_frame_straight_60m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p141a02_thick_frame_straight_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141A11_THICK_FRAME_EDGE, new MyPrefabConfiguration(MyModelsEnum.p141a11_thick_frame_edge, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p141a11_thick_frame_edge_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141A12_THICK_FRAME_CORNER, new MyPrefabConfiguration(MyModelsEnum.p141a12_thick_frame_corner, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P141A31_THICK_FRAME_JOINT, new MyPrefabConfiguration(MyModelsEnum.p141a31_thick_frame_joint, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.FRAME, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p141a31_thick_frame_joint_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120J01_J_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130j01_j_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130j01_j_straight_30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130J02_J_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130j02_j_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130j02_j_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120I01_I_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130i01_i_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130I02_I_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130i02_i_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130i02_i_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120H01_H_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130h01_h_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130h01_h_straight_30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130H02_H_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130h02_h_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130h02_h_straight_10m_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120G01_G_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130g01_g_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130G02_G_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130g02_g_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120F01_F_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130f01_f_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130F02_F_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130f02_f_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120E01_E_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130e01_e_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130E02_E_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130e02_e_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120D01_D_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130d01_d_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130D02_D_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130d02_d_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120C01_C_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130c01_c_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130C02_C_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130c02_c_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120B01_B_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130b01_b_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130B02_B_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130b02_b_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A01_A_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130a01_a_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130A02_A_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130a02_a_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120D01_D_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p120d01_d_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120d01_d_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120D02_D_STRAIGHT_40M, new MyPrefabConfiguration(MyModelsEnum.p120d02_d_straight_40m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120d02_d_straight_40m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120C01_C_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p120c01_c_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120c01_c_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120C02_C_STRAIGHT_40M, new MyPrefabConfiguration(MyModelsEnum.p120c02_c_straight_40m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120c02_c_straight_40m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120B01_B_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p120b01_b_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120b01_b_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120B02_B_STRAIGHT_40M, new MyPrefabConfiguration(MyModelsEnum.p120b02_b_straight_40m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120b02_b_straight_40m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A01_STRONG_LATTICE_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p120a01_strong_lattice_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a01_strong_lattice_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A02_STRONG_LATTICE_STRAIGHT_60M, new MyPrefabConfiguration(MyModelsEnum.p120a02_strong_lattice_straight_60m, MyModelsEnum.p120a02_strong_lattice_straight_60m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a02_strong_lattice_straight_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A03_STRONG_LATTICE_STRAIGHT_120M, new MyPrefabConfiguration(MyModelsEnum.p120a03_strong_lattice_straight_120m, MyModelsEnum.p120a03_strong_lattice_straight_120m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a03_strong_lattice_straight_120m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A21_STRONG_LATTICE_JUNCTION_T_STRONG, new MyPrefabConfiguration(MyModelsEnum.p120a21_strong_lattice_junction_t_strong, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a21_strong_lattice_junction_t_strong_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A22_STRONG_LATTICE_JUNCTION_T_WEAK, new MyPrefabConfiguration(MyModelsEnum.p120a22_strong_lattice_junction_t_weak, MyModelsEnum.p120a22_strong_lattice_junction_t_weak_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a22_strong_lattice_junction_t_weak_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A23_STRONG_LATTICE_JUNCTION_T_ROTATED, new MyPrefabConfiguration(MyModelsEnum.p120a23_strong_lattice_junction_t_rotated, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a23_strong_lattice_junction_t_rotated_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A51_STRONG_TO_WEAK_LATTICE_2TO1, new MyPrefabConfiguration(MyModelsEnum.p120a51_strong_to_weak_lattice_2to1, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a51_strong_to_weak_lattice_2to1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A52_STRONG_TO_WEAK_LATTICE_1TO2, new MyPrefabConfiguration(MyModelsEnum.p120a52_strong_to_weak_lattice_1to2, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120A61_WEAK_LATTICE_JUNCTION_T_ROTATED, new MyPrefabConfiguration(MyModelsEnum.p120a61_weak_lattice_junction_t_rotated, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120a61_weak_lattice_junction_t_rotated_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B01_LATTICE_BEAM_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p110b01_lattice_beam_straight_10m, MyModelsEnum.p110b01_lattice_beam_straight_10m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b01_lattice_beam_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B02_LATTICE_BEAM_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p110b02_lattice_beam_straight_30m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b02_lattice_beam_straight_30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B03_LATTICE_BEAM_STRAIGHT_60M, new MyPrefabConfiguration(MyModelsEnum.p110b03_lattice_beam_straight_60m, MyModelsEnum.p110b03_lattice_beam_straight_60m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b03_lattice_beam_straight_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B04_LATTICE_BEAM_STRAIGHT_60M_WITH_PANELS, new MyPrefabConfiguration(MyModelsEnum.p110b04_lattice_beam_straight_60m_with_panels, MyModelsEnum.p110b04_lattice_beam_straight_60m_with_panels_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b04_lattice_beam_straight_60m_with_panels_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B21_LATTICE_BEAM_JUNCTION_T_STRONG, new MyPrefabConfiguration(MyModelsEnum.p110b21_lattice_beam_junction_t_strong, MyModelsEnum.p110b21_lattice_beam_junction_t_strong_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b21_lattice_beam_junction_t_strong_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B22_LATTICE_BEAM_JUNCTION_T_WEAK, new MyPrefabConfiguration(MyModelsEnum.p110b22_lattice_beam_junction_t_weak, MyModelsEnum.p110b22_lattice_beam_junction_t_weak_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b22_lattice_beam_junction_t_weak_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B31_LATTICE_BEAM_JOINT_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p110b31_lattice_beam_joint_horizontal, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b31_lattice_beam_joint_horizontal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B32_LATTICE_BEAM_JOINT_VERTICAL, new MyPrefabConfiguration(MyModelsEnum.p110b32_lattice_beam_joint_vertical, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b32_lattice_beam_joint_vertical_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A01_SOLID_BEAM_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p110a01_solid_beam_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a01_solid_beam_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A02_SOLID_BEAM_STRAIGHT_20M, new MyPrefabConfiguration(MyModelsEnum.p110a02_solid_beam_straight_20m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a02_solid_beam_straight_20m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A03_SOLID_BEAM_STRAIGHT_40M_WITH_HOLE, new MyPrefabConfiguration(MyModelsEnum.p110a03_solid_beam_straight_40m_with_hole, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a03_solid_beam_straight_40m_with_hole_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A04_SOLID_BEAM_STRAIGHT_40M_LATTICE, new MyPrefabConfiguration(MyModelsEnum.p110a04_solid_beam_straight_40m_lattice, MyModelsEnum.p110a04_solid_beam_straight_40m_lattice_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a04_solid_beam_straight_40m_lattice_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A05_SOLID_BEAM_STRAIGHT_80M, new MyPrefabConfiguration(MyModelsEnum.p110a05_solid_beam_straight_80m, MyModelsEnum.p110a05_solid_beam_straight_80m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a05_solid_beam_straight_80m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A11_SOLID_BEAM_JUNCTION_X_STRONG, new MyPrefabConfiguration(MyModelsEnum.p110a11_solid_beam_junction_x_strong, MyModelsEnum.p110a11_solid_beam_junction_x_strong_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a11_solid_beam_junction_x_strong_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A12_SOLID_BEAM_JUNCTION_X_WEAK, new MyPrefabConfiguration(MyModelsEnum.p110a12_solid_beam_junction_x_weak, MyModelsEnum.p110a12_solid_beam_junction_x_weak_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a12_solid_beam_junction_x_weak_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A13_SOLID_BEAM_JUNCTION_X_ROTATED, new MyPrefabConfiguration(MyModelsEnum.p110a13_solid_beam_junction_x_rotated, MyModelsEnum.p110a13_solid_beam_junction_x_rotated_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a13_solid_beam_junction_x_rotated_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A21_SOLID_BEAM_JUNCTION_T_STRONG, new MyPrefabConfiguration(MyModelsEnum.p110a21_solid_beam_junction_t_strong, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a21_solid_beam_junction_t_strong_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A22_SOLID_BEAM_JUNCTION_T_WEAK, new MyPrefabConfiguration(MyModelsEnum.p110a22_solid_beam_junction_t_weak, MyModelsEnum.p110a22_solid_beam_junction_t_weak_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a22_solid_beam_junction_t_weak_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A23_SOLID_BEAM_JUNCTION_T_ROTATED, new MyPrefabConfiguration(MyModelsEnum.p110a23_solid_beam_junction_t_rotated, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a23_solid_beam_junction_t_rotated_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A31_SOLID_BEAM_JOINT_HORIZONTAL, new MyPrefabConfiguration(MyModelsEnum.p110a31_solid_beam_joint_horizontal, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a31_solid_beam_joint_horizontal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A32_SOLID_BEAM_JOINT_VERTICAL, new MyPrefabConfiguration(MyModelsEnum.p110a32_solid_beam_joint_vertical, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a32_solid_beam_joint_vertical_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A33_SOLID_BEAM_JOINT_LONGITUDINAL, new MyPrefabConfiguration(MyModelsEnum.p110a33_solid_beam_joint_longitudinal, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a33_solid_beam_joint_longitudinal_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A41_SOLID_BEAM_SUPERJOINT, new MyPrefabConfiguration(MyModelsEnum.p110a41_solid_beam_superjoint, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110a41_solid_beam_superjoint_COL));
            //@ Lights
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLight, (int)MyMwcObjectBuilder_PrefabLight_TypesEnum.DEFAULT_LIGHT_0, new MyPrefabConfigurationLight(MyModelsEnum.default_light_0, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.LIGHTS, null, MyMaterialType.METAL, MyLightPrefabTypeEnum.POINT_LIGHT, MyModelsEnum.default_light_0_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLight, (int)MyMwcObjectBuilder_PrefabLight_TypesEnum.P521_A01_LIGHT1, new MyPrefabConfigurationLight(MyModelsEnum.p521_a01_light1, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.LIGHTS, null, MyMaterialType.METAL, MyLightPrefabTypeEnum.POINT_LIGHT, MyModelsEnum.p521_a01_light1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLight, (int)MyMwcObjectBuilder_PrefabLight_TypesEnum.P521_A02_LIGHT2, new MyPrefabConfigurationLight(MyModelsEnum.p521_a02_light2, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.LIGHTS, null, MyMaterialType.METAL, MyLightPrefabTypeEnum.POINT_LIGHT, MyModelsEnum.p521_a02_light2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLight, (int)MyMwcObjectBuilder_PrefabLight_TypesEnum.P521_A03_LIGHT3, new MyPrefabConfigurationLight(MyModelsEnum.p521_a03_light3, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.LIGHTS, null, MyMaterialType.METAL, MyLightPrefabTypeEnum.POINT_LIGHT, MyModelsEnum.p521_a03_light3_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLight, (int)MyMwcObjectBuilder_PrefabLight_TypesEnum.P521_A04_LIGHT4, new MyPrefabConfigurationLight(MyModelsEnum.p521_a04_light4, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.LIGHTS, null, MyMaterialType.METAL, MyLightPrefabTypeEnum.POINT_LIGHT, MyModelsEnum.p521_a04_light4_COL));
            //@ Particles
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabParticles, (int)MyMwcObjectBuilder_PrefabParticles_TypesEnum.DEFAULT_PARTICLE_PREFAB_0, new MyPrefabConfigurationParticles(MyModelsEnum.default_light_0, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.PARTICLES, null, MyMaterialType.METAL, MyParticleEffectsIDEnum.Prefab_LeakingSteamGrey));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabParticles, (int)MyMwcObjectBuilder_PrefabParticles_TypesEnum.P551_A01_PARTICLES, new MyPrefabConfigurationParticles(MyModelsEnum.p551_a01_particles, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.PARTICLES, null, MyMaterialType.METAL, MyParticleEffectsIDEnum.Prefab_LeakingSteamWhite));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabParticles, (int)MyMwcObjectBuilder_PrefabParticles_TypesEnum.P551_B01_PARTICLES, new MyPrefabConfigurationParticles(MyModelsEnum.p551_b01_particles, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.PARTICLES, null, MyMaterialType.METAL, MyParticleEffectsIDEnum.Prefab_LeakingSteamGrey));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabParticles, (int)MyMwcObjectBuilder_PrefabParticles_TypesEnum.P551_C01_PARTICLES, new MyPrefabConfigurationParticles(MyModelsEnum.p551_c01_particles, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.PARTICLES, null, MyMaterialType.METAL, MyParticleEffectsIDEnum.Prefab_LeakingSteamBlack));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabParticles, (int)MyMwcObjectBuilder_PrefabParticles_TypesEnum.P551_D01_PARTICLES, new MyPrefabConfigurationParticles(MyModelsEnum.p551_d01_particles, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.PARTICLES, null, MyMaterialType.METAL, MyParticleEffectsIDEnum.Prefab_DustyArea));
            //@ Sounds
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSound, (int)MyMwcObjectBuilder_PrefabSound_TypesEnum.DEFAULT_SOUND_PREFAB_0, new MyPrefabConfigurationSound(MyModelsEnum.default_light_0, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SOUNDS, null, MyMaterialType.METAL, MySoundCuesEnum.VehShipaEngineHigh3d));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSound, (int)MyMwcObjectBuilder_PrefabSound_TypesEnum.P561_A01_SOUND, new MyPrefabConfigurationSound(MyModelsEnum.p561_a01_sound, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SOUNDS, null, MyMaterialType.METAL, MySoundCuesEnum.VehShipaEngineHigh3d));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSound, (int)MyMwcObjectBuilder_PrefabSound_TypesEnum.P561_B01_SOUND, new MyPrefabConfigurationSound(MyModelsEnum.p561_b01_sound, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SOUNDS, null, MyMaterialType.METAL, MySoundCuesEnum.VehShipaEngineHigh3d));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSound, (int)MyMwcObjectBuilder_PrefabSound_TypesEnum.P561_C01_SOUND, new MyPrefabConfigurationSound(MyModelsEnum.p561_c01_sound, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SOUNDS, null, MyMaterialType.METAL, MySoundCuesEnum.VehShipaEngineHigh3d));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSound, (int)MyMwcObjectBuilder_PrefabSound_TypesEnum.P561_D01_SOUND, new MyPrefabConfigurationSound(MyModelsEnum.p561_d01_sound, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SOUNDS, null, MyMaterialType.METAL, MySoundCuesEnum.VehShipaEngineHigh3d));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSound, (int)MyMwcObjectBuilder_PrefabSound_TypesEnum.MOTHERSHIP_SOUND, new MyPrefabConfigurationSound(MyModelsEnum.p561_a01_sound, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SOUNDS, null, MyMaterialType.METAL, MySoundCuesEnum.VehLoopLargeShip));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSound, (int)MyMwcObjectBuilder_PrefabSound_TypesEnum.MADELINE_MOTHERSHIP_SOUND, new MyPrefabConfigurationSound(MyModelsEnum.p561_a01_sound, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SOUNDS, null, MyMaterialType.METAL, MySoundCuesEnum.VehLoopLargeShipMadelyn));
            //@ Weapons
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_AUTOCANNON, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_AUTOCANNON, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MACHINEGUN, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_CIWS, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_CIWS, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MACHINEGUN, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MISSILE_BASIC4, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MISSILE_BASIC4, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MISSILE_BASIC6, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MISSILE_BASIC6, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MISSILE_BASIC9, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A01_LARGESHIP_MISSILE_BASIC9, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A02_LARGESHIP_MISSILE_GUIDED4, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A02_LARGESHIP_MISSILE_GUIDED4, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A02_LARGESHIP_MISSILE_GUIDED6, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A02_LARGESHIP_MISSILE_GUIDED6, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeWeapon, (int)MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A02_LARGESHIP_MISSILE_GUIDED9, new MyPrefabConfigurationLargeWeapon(BuildTypesEnum.MODULES, CategoryTypesEnum.WEAPONRY, null, MyMaterialType.METAL, MyMwcObjectBuilder_PrefabLargeWeapon_TypesEnum.P352_A02_LARGESHIP_MISSILE_GUIDED9, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            //@ Billboards:
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A01_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a01_billboard, MyModelsEnum.p511_a01_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a01_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A02_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a02_billboard, MyModelsEnum.p511_a02_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a02_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A03_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a03_billboard, MyModelsEnum.p511_a03_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a03_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A04_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a04_billboard, MyModelsEnum.p511_a04_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a04_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A05_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a05_billboard, MyModelsEnum.p511_a05_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a05_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A06_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a06_billboard, MyModelsEnum.p511_a06_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a06_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A07_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a07_billboard, MyModelsEnum.p511_a07_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a07_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A08_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a08_billboard, MyModelsEnum.p511_a08_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a08_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A09_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a09_billboard, MyModelsEnum.p511_a09_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a09_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A10_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a10_billboard, MyModelsEnum.p511_a10_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a10_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A11_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a11_billboard, MyModelsEnum.p511_a11_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a11_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A12_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a12_billboard, MyModelsEnum.p511_a12_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a12_billboard_COL));            
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A14_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a14_billboard, MyModelsEnum.p511_a14_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A15_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a15_billboard, MyModelsEnum.p511_a15_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a15_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A16_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a16_billboard, MyModelsEnum.p511_a16_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a16_billboard_COL));

            //@ Signs:
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A01_SIGN1, new MyPrefabConfiguration(MyModelsEnum.p531_a01_sign1, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A02_SIGN2, new MyPrefabConfiguration(MyModelsEnum.p531_a02_sign2, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A03_SIGN3, new MyPrefabConfiguration(MyModelsEnum.p531_a03_sign3, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A04_SIGN4, new MyPrefabConfiguration(MyModelsEnum.p531_a04_sign4, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A05_SIGN5, new MyPrefabConfiguration(MyModelsEnum.p531_a05_sign5, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A06_SIGN6, new MyPrefabConfiguration(MyModelsEnum.p531_a06_sign6, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A07_SIGN7, new MyPrefabConfiguration(MyModelsEnum.p531_a07_sign7, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A08_SIGN8, new MyPrefabConfiguration(MyModelsEnum.p531_a08_sign8, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A09_SIGN9, new MyPrefabConfiguration(MyModelsEnum.p531_a09_sign9, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A10_SIGN10, new MyPrefabConfiguration(MyModelsEnum.p531_a10_sign10, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A11_SIGN11, new MyPrefabConfiguration(MyModelsEnum.p531_a11_sign11, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_A12_SIGN12, new MyPrefabConfiguration(MyModelsEnum.p531_a12_sign12, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Top));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_ADMINISTRATIVE_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_administrative_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_ARMORY, new MyPrefabConfiguration(MyModelsEnum.p531_c_armory, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_ARROW_L, new MyPrefabConfiguration(MyModelsEnum.p531_c_arrow_L, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_ARROW_R, new MyPrefabConfiguration(MyModelsEnum.p531_c_arrow_R, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_ARROW_STR, new MyPrefabConfiguration(MyModelsEnum.p531_c_arrow_str, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_CARGO_BAY, new MyPrefabConfiguration(MyModelsEnum.p531_c_cargo_bay, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_COMMAND_CENTER, new MyPrefabConfiguration(MyModelsEnum.p531_c_command_center, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_COMMERCIAL_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_commercial_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_COMMUNICATIONS, new MyPrefabConfiguration(MyModelsEnum.p531_c_communications, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_DEFENSES, new MyPrefabConfiguration(MyModelsEnum.p531_c_defenses, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_DOCKS, new MyPrefabConfiguration(MyModelsEnum.p531_c_docks, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front, collisionModelEnum: MyModelsEnum.p531_c_docks_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_EMERGENCY_EXIT, new MyPrefabConfiguration(MyModelsEnum.p531_c_emergency_exit, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_ENGINEERING_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_engineering_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front, collisionModelEnum: MyModelsEnum.p531_c_engineering_area_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_EXIT, new MyPrefabConfiguration(MyModelsEnum.p531_c_exit, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_EXPERIMENTAL_LABS, new MyPrefabConfiguration(MyModelsEnum.p531_c_experimental_labs, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_FOUNDRY, new MyPrefabConfiguration(MyModelsEnum.p531_c_foundry, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_HABITATS, new MyPrefabConfiguration(MyModelsEnum.p531_c_habitats, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front, collisionModelEnum: MyModelsEnum.p531_c_habitats_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_HANGARS, new MyPrefabConfiguration(MyModelsEnum.p531_c_hangars, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front, collisionModelEnum: MyModelsEnum.p531_c_hangars_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_INDUSTRIAL_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_industrial_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_LANDING_BAY, new MyPrefabConfiguration(MyModelsEnum.p531_c_landing_bay, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_MAINTENANCE, new MyPrefabConfiguration(MyModelsEnum.p531_c_maintenance, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front, collisionModelEnum: MyModelsEnum.p531_c_maintenance_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_MILITARY_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_military_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_MINES, new MyPrefabConfiguration(MyModelsEnum.p531_c_mines, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_ORE_PROCESSING, new MyPrefabConfiguration(MyModelsEnum.p531_c_ore_processing, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_OUTER_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_outer_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_PRISON, new MyPrefabConfiguration(MyModelsEnum.p531_c_prison, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_PUBLIC_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_public_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_REACTOR, new MyPrefabConfiguration(MyModelsEnum.p531_c_reactor, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front, collisionModelEnum: MyModelsEnum.p531_c_reactor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_RESEARCH, new MyPrefabConfiguration(MyModelsEnum.p531_c_research, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_RESTRICTED_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_restricted_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_SECURITY, new MyPrefabConfiguration(MyModelsEnum.p531_c_security, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_SIGN, new MyPrefabConfiguration(MyModelsEnum.p531_c_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_STORAGE, new MyPrefabConfiguration(MyModelsEnum.p531_c_storage, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front, collisionModelEnum: MyModelsEnum.p531_c_storage_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_TECHNICAL_AREA, new MyPrefabConfiguration(MyModelsEnum.p531_c_technical_area, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_C_TRADE_PORT, new MyPrefabConfiguration(MyModelsEnum.p531_c_trade_port, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Front));


            // @ chambers:
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221_A02_CHAMBER_V2, new MyPrefabConfiguration(MyModelsEnum.p221a02_chamber_v2, MyModelsEnum.p221a02_chamber_v2_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221a02_chamber_v2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221_B02_CHAMBER_V2, new MyPrefabConfiguration(MyModelsEnum.p221b02_chamber_v2, MyModelsEnum.p221b02_chamber_v2_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221b02_chamber_v2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221_C02_CHAMBER_V2, new MyPrefabConfiguration(MyModelsEnum.p221c02_chamber_v2, MyModelsEnum.p221c02_chamber_v2_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221c02_chamber_v2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221_D02_CHAMBER_V2, new MyPrefabConfiguration(MyModelsEnum.p221d02_chamber_v2, MyModelsEnum.p221d02_chamber_v2_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, null, MyModelsEnum.p221d02_chamber_v2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221_E02_CHAMBER_V2, new MyPrefabConfiguration(MyModelsEnum.p221e02_chamber_v2, MyModelsEnum.p221e02_chamber_v2_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221e02_chamber_v2_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_A01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130a01_a_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130a01_a_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_A02_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130a02_a_straight_30m, MyModelsEnum.p130a02_a_straight_30m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130a02_a_straight_30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_B01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130b01_b_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130b01_b_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_B02_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130b02_b_straight_30m, MyModelsEnum.p130b02_b_straight_30m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130b02_b_straight_30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_C01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130c01_c_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130c01_c_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_C02_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130c02_c_straight_30m, MyModelsEnum.p130c02_c_straight_30m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130c02_c_straight_30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_D01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130d01_d_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130d01_d_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_D02_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130d02_d_straight_30m, MyModelsEnum.p130d02_d_straight_30m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130d02_d_straight_30m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_E01_STRAIGHT_10M, new MyPrefabConfiguration(MyModelsEnum.p130e01_e_straight_10m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130e01_e_straight_10m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130_E02_STRAIGHT_30M, new MyPrefabConfiguration(MyModelsEnum.p130e02_e_straight_30m, MyModelsEnum.p130e02_e_straight_30m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130e02_e_straight_30m_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P321_B01_SOLAR_PANEL, new MyPrefabConfiguration(MyModelsEnum.p321b01_solar_panel, MyModelsEnum.p321b01_solar_panel_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p321b01_solar_panel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P321_C01_SOLAR_PANEL, new MyPrefabConfiguration(MyModelsEnum.p321c01_solar_panel, MyModelsEnum.p321c01_solar_panel_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p321c01_solar_panel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P321_D01_BIG_SOLAR_PANEL, new MyPrefabConfiguration(MyModelsEnum.p321d01_big_solar_panel, MyModelsEnum.p321d01_big_solar_panel_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_A01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_a01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_a01_traffic_sign_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_B01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_b01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_BOX01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_box01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_box01_traffic_sign_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_BOX02_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_box02_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_C01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_c01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_c01_traffic_sign_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_D01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_d01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_E01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_e01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_F01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_f01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_f01_traffic_sign_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_G01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_g01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_H01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_h01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_I01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_i01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_i01_traffic_sign_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_J01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_j01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_j01_traffic_sign_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_K01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_k01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_k01_traffic_sign_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P571_L01_TRAFFIC_SIGN, new MyPrefabConfiguration(MyModelsEnum.p571_l01_traffic_sign, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.TRAFFIC_SIGNS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.Bottom, collisionModelEnum: MyModelsEnum.p571_l01_traffic_sign_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_F01_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_f01_entrance_60m, MyModelsEnum.p414_f01_entrance_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p414_f01_entrance_60m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P414_G01_ENTRANCE_60M, new MyPrefabConfiguration(MyModelsEnum.p414_g01_entrance_60m, MyModelsEnum.p414_g01_entrance_60m_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.TUNNELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p414_g01_entrance_60m_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P385_A01_TEMPLE_900M, new MyPrefabConfiguration(MyModelsEnum.p385_a01_temple_900m, MyModelsEnum.p385_a01_temple_900m_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 1f, collisionModelEnum: MyModelsEnum.p385_a01_temple_900m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P383_A01_CHURCH, new MyPrefabConfiguration(MyModelsEnum.p383_a01_church, MyModelsEnum.p383_a01_church_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 1f, collisionModelEnum: MyModelsEnum.p383_a01_church_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P334_A01_FOOD_GROW, new MyPrefabConfiguration(MyModelsEnum.p334_a01_food_grow, MyModelsEnum.p334_a01_food_grow_LOD01, BuildTypesEnum.MODULES, CategoryTypesEnum.LIFE_SUPPORT, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p334_a01_food_grow_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P345_A01_BIO_EXP, new MyPrefabConfiguration(MyModelsEnum.p345_a01_bio_exp, MyModelsEnum.p345_a01_bio_exp_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p345_a01_bio_exp_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P345_A01_BIO_MACH_EXP, new MyPrefabConfiguration(MyModelsEnum.p345_a01_bio_mach_exp, MyModelsEnum.p345_a01_bio_mach_exp_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p345_a01_bio_mach_exp_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P345_A01_RECYCLE, new MyPrefabConfiguration(MyModelsEnum.p345_a01_recycle, MyModelsEnum.p345_a01_recycle_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, explosionParticleEffectScale: 1.5f, collisionModelEnum: MyModelsEnum.p345_a01_recycle_COL));

            //@ large ships
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.LARGESHIP_KAI, new MyPrefabConfigurationLargeShip(MyModelsEnum.Kai, MyModelsEnum.Kai_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.LARGE_SHIPS, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.None, explosionType: MyExplosionTypeEnum.LARGE_SHIP_EXPLOSION, particleScale: 0.3f, collisionModelEnum: MyModelsEnum.Kai_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.LARGESHIP_SAYA, new MyPrefabConfigurationLargeShip(MyModelsEnum.MotherShipSaya, MyModelsEnum.MotherShipSaya_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.LARGE_SHIPS, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.None, explosionType: MyExplosionTypeEnum.LARGE_SHIP_EXPLOSION, particleScale: 0.3f, collisionModelEnum: MyModelsEnum.MotherShipSaya_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.LARGESHIP_ARDANT, new MyPrefabConfigurationLargeShip(MyModelsEnum.Ardant, MyModelsEnum.Ardant_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.LARGE_SHIPS, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.None, explosionType: MyExplosionTypeEnum.LARGE_SHIP_EXPLOSION, particleScale: 0.3f, collisionModelEnum: MyModelsEnum.Ardant_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.FOURTH_REICH_MOTHERSHIP, new MyPrefabConfigurationLargeShip(MyModelsEnum.FourthReichMothership, MyModelsEnum.FourthReichMothership_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.LARGE_SHIPS, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.FourthReich, explosionType: MyExplosionTypeEnum.LARGE_SHIP_EXPLOSION, particleScale: 0.3f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.FOURTH_REICH_MOTHERSHIP_B, new MyPrefabConfigurationLargeShip(MyModelsEnum.FourthReichMothership_B, MyModelsEnum.FourthReichMothership_B_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.LARGE_SHIPS, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.FourthReich, explosionType: MyExplosionTypeEnum.LARGE_SHIP_EXPLOSION, particleScale: 0.3f, collisionModelEnum: MyModelsEnum.FourthReichMothership_B_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.RUS_MOTHERSHIP, new MyPrefabConfigurationLargeShip(MyModelsEnum.RusMothership, MyModelsEnum.RusMothership_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.LARGE_SHIPS, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Russian, collisionModelEnum: MyModelsEnum.RusMothership_COL, explosionType: MyExplosionTypeEnum.LARGE_SHIP_EXPLOSION, particleScale: 0.3f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.RUSSIAN_MOTHERSHIP_HUMMER, new MyPrefabConfigurationLargeShip(MyModelsEnum.Russian_Mothership_Hummer, MyModelsEnum.Russian_Mothership_Hummer_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.LARGE_SHIPS, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Russian, collisionModelEnum: MyModelsEnum.Russian_Mothership_Hummer_COL, explosionType: MyExplosionTypeEnum.LARGE_SHIP_EXPLOSION, particleScale: 0.3f));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_BODY, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_body, MyModelsEnum.mship_body_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_ENGINE, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_engine, MyModelsEnum.mship_engine_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_BACK_LARGE_LEFT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_back_large_left, MyModelsEnum.mship_shield_back_large_left_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_BACK_LARGE_RIGHT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_back_large_right, MyModelsEnum.mship_shield_back_large_right_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_BACK_SMALL_LEFT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_back_small_left, MyModelsEnum.mship_shield_back_small_left_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_BACK_SMALL_RIGHT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_back_small_right, MyModelsEnum.mship_shield_back_small_right_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_FRONT_LARGE_LEFT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_front_large_left, MyModelsEnum.mship_shield_front_large_left_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_FRONT_LARGE_RIGHT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_front_large_right, MyModelsEnum.mship_shield_front_large_right_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_FRONT_SMALL_LEFT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_front_small_left, MyModelsEnum.mship_shield_front_small_left_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_FRONT_SMALL_RIGHT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_front_small_right, MyModelsEnum.mship_shield_front_small_right_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_FRONT_SMALL02_LEFT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_front_small02_left, MyModelsEnum.mship_shield_front_small02_left_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabLargeShip, (int)MyMwcObjectBuilder_PrefabLargeShip_TypesEnum.MSHIP_SHIELD_FRONT_SMALL02_RIGHT, new MyPrefabConfigurationLargeShip(MyModelsEnum.mship_shield_front_small02_right, MyModelsEnum.mship_shield_front_small02_right_LOD1, BuildTypesEnum.LARGE_SHIPS, CategoryTypesEnum.MSHIP, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.Euroamerican));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.SimpleObject, new MyPrefabConfiguration(MyModelsEnum.SimpleObject, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.AsteroidPrefabTest, new MyPrefabConfiguration(MyModelsEnum.AsteroidPrefabTest, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, MyMwcObjectBuilder_Prefab_AppearanceEnum.None));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P541_ESCAPE_POD, new MyPrefabConfiguration(MyModelsEnum.p541_escape_pod, MyModelsEnum.p541_escape_pod_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_escape_pod_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P541_ESCAPE_POD_BASE, new MyPrefabConfiguration(MyModelsEnum.p541_escape_pod_base, MyModelsEnum.p541_escape_pod_base_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_escape_pod_base_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P541_VENTILATOR_BODY, new MyPrefabConfiguration(MyModelsEnum.p541_ventilator_body_standalone, MyModelsEnum.p541_ventilator_body_standalone_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_ventilator_body_standalone_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P541_VENTILATOR_PROPELLER, new MyPrefabConfiguration(MyModelsEnum.p541_ventilator_propeller_standalone, MyModelsEnum.p541_ventilator_propeller_standalone_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_ventilator_propeller_standalone_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P541_VENTILATOR, new MyPrefabConfiguration(MyModelsEnum.p541_ventilator_body, MyModelsEnum.p541_ventilator_body_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, loopRotatingCue: MySoundCuesEnum.Amb3D_FanMediumNormal, loopRotatingDamagedCue: MySoundCuesEnum.Amb3D_FanMediumDamaged, collisionModelEnum: MyModelsEnum.p541_ventilator_body_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P349_A_TOWER, new MyPrefabConfiguration(MyModelsEnum.p349_a_tower, MyModelsEnum.p349_a_tower_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p349_a_tower_COL, explosionParticleEffectScale: 1));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P349_B_TOWER, new MyPrefabConfiguration(MyModelsEnum.p349_b_tower, MyModelsEnum.p349_b_tower_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p349_b_tower_COL, explosionParticleEffectScale: 1));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P349_C_TOWER, new MyPrefabConfiguration(MyModelsEnum.p349_c_tower, MyModelsEnum.p349_c_tower_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, null, MyModelsEnum.p349_c_tower_COL, explosionParticleEffectScale: 1));

            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabHangar, (int)MyMwcObjectBuilder_PrefabHangar_TypesEnum.P361_A02_HANGAR_PANEL, new MyPrefabConfigurationHangar(MyModelsEnum.p361_a02_hangar_panel, null, BuildTypesEnum.MODULES, CategoryTypesEnum.HANGARS, null, MyMaterialType.METAL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_B_FACTION, new MyPrefabConfiguration(MyModelsEnum.p531_b_faction, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p531_b_faction_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_B_FACTION_HOLO, new MyPrefabConfiguration(MyModelsEnum.p531_b_faction_holo, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p531_b_faction_holo_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.ARMOR_HULL, new MyPrefabConfiguration(MyModelsEnum.armor_hull, MyModelsEnum.armor_hull_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, previewAngle: MyPreviewPointOfViewEnum.RearLeft, collisionModelEnum: MyModelsEnum.armor_hull_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabFoundationFactory, (int)MyMwcObjectBuilder_PrefabFoundationFactory_TypesEnum.DEFAULT, new MyPrefabConfigurationFoundationFactory(MyModelsEnum.FoundationFactory, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.FOUNDATION_FACTORY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.FoundationFactory_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212A01_PANEL_LARGE, new MyPrefabConfiguration(MyModelsEnum.p212a01_panel_large, MyModelsEnum.p212a01_panel_large_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212a01_panel_large_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212A01_PANEL_SMALL, new MyPrefabConfiguration(MyModelsEnum.p212a01_panel_small, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p212a01_panel_small_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212A01_PANEL_MEDIUM, new MyPrefabConfiguration(MyModelsEnum.p212a01_panel_medium, MyModelsEnum.p212a01_panel_medium_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212a01_panel_medium_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212B02_PANEL_MEDIUM, new MyPrefabConfiguration(MyModelsEnum.p212b02_panel_medium, MyModelsEnum.p212b02_panel_medium_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212b02_panel_medium_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212B02_PANEL_SMALL, new MyPrefabConfiguration(MyModelsEnum.p212b02_panel_small, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p212b02_panel_small_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212C03_PANEL_MEDIUM, new MyPrefabConfiguration(MyModelsEnum.p212c03_panel_medium, MyModelsEnum.p212c03_panel_medium_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212c03_panel_medium_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212C03_PANEL_SMALL, new MyPrefabConfiguration(MyModelsEnum.p212c03_panel_small, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p212c03_panel_small_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212D04_PANEL_MEDIUM, new MyPrefabConfiguration(MyModelsEnum.p212d04_panel_medium, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212d04_panel_medium_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212D04_PANEL_SMALL, new MyPrefabConfiguration(MyModelsEnum.p212d04_panel_small, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p212d04_panel_small_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212E05_PANEL_MEDIUM, new MyPrefabConfiguration(MyModelsEnum.p212e05_panel_medium, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212e05_panel_medium_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212E05_PANEL_SMALL, new MyPrefabConfiguration(MyModelsEnum.p212e05_panel_small, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p212e05_panel_small_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSecurityControlHUB, (int)MyMwcObjectBuilder_PrefabSecurityControlHUB_TypesEnum.DEFAULT, new MyPrefabConfigurationSecurityControlHUB(MyModelsEnum.p541_security_hub, MyModelsEnum.p541_security_hub_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.SECURITY_CONTROL_HUB, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_security_hub_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSecurityControlHUB, (int)MyMwcObjectBuilder_PrefabSecurityControlHUB_TypesEnum.P541_SCREEN_A, new MyPrefabConfigurationSecurityControlHUB(MyModelsEnum.p541_screen_A, MyModelsEnum.p541_screen_A_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.SECURITY_CONTROL_HUB, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_screen_A_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSecurityControlHUB, (int)MyMwcObjectBuilder_PrefabSecurityControlHUB_TypesEnum.P541_SCREEN_B, new MyPrefabConfigurationSecurityControlHUB(MyModelsEnum.p541_screen_B, MyModelsEnum.p541_screen_B_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.SECURITY_CONTROL_HUB, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_screen_B_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabSecurityControlHUB, (int)MyMwcObjectBuilder_PrefabSecurityControlHUB_TypesEnum.P541_TERMINAL_A, new MyPrefabConfigurationSecurityControlHUB(MyModelsEnum.p541_terminal_A, MyModelsEnum.p541_terminal_A_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.SECURITY_CONTROL_HUB, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p541_terminal_A_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabBankNode, (int)MyMwcObjectBuilder_PrefabBankNode_TypesEnum.DEFAULT, new MyPrefabConfigurationBankNode(MyModelsEnum.Bank_node, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.BANK_NODE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Bank_node_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabBankNode, (int)MyMwcObjectBuilder_PrefabBankNode_TypesEnum.DEFAULT, new MyPrefabConfigurationBankNode(MyModelsEnum.Bank_node, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.BANK_NODE, null, MyMaterialType.METAL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabGenerator, (int)MyMwcObjectBuilder_PrefabGenerator_TypesEnum.P321C01_INERTIA_GENERATOR, new MyPrefabConfigurationGenerator(MyModelsEnum.p321c01_inertia_generator, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.GENERATOR, null, MyMaterialType.METAL, 1500f, MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.8f, modelCol: MyModelsEnum.p321c01_inertia_generator_COL, explosionParticleEffectScale: 2.1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabGenerator, (int)MyMwcObjectBuilder_PrefabGenerator_TypesEnum.P321C03_CENTRIFUGE, new MyPrefabConfigurationGenerator(MyModelsEnum.p321c03_centrifuge, MyModelsEnum.p321c03_centrifuge_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.GENERATOR, null, MyMaterialType.METAL, 1500f, MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.5f, modelCol: MyModelsEnum.p321c03_centrifuge_COL, explosionParticleEffectScale: 1.7f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabGenerator, (int)MyMwcObjectBuilder_PrefabGenerator_TypesEnum.P321C04_BOX_GENERATOR, new MyPrefabConfigurationGenerator(MyModelsEnum.p321c04_box_generator, MyModelsEnum.p321c04_box_generator_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.GENERATOR, null, MyMaterialType.METAL, 1500f, MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.8f, modelCol: MyModelsEnum.p321c04_box_generator_COL, explosionParticleEffectScale: 2.5f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabGenerator, (int)MyMwcObjectBuilder_PrefabGenerator_TypesEnum.P321C05_CENTRIFUGE_BIG, new MyPrefabConfigurationGenerator(MyModelsEnum.p321c05_centrifuge_big, MyModelsEnum.p321c05_centrifuge_big_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.GENERATOR, null, MyMaterialType.METAL, 1500f, MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.5f, modelCol: MyModelsEnum.p321c05_centrifuge_big_COL, explosionParticleEffectScale: 1.7f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabGenerator, (int)MyMwcObjectBuilder_PrefabGenerator_TypesEnum.P321C02_GENERATOR_WALL_BIG, new MyPrefabConfigurationGenerator(MyModelsEnum.p321c02_generator_wall_big, MyModelsEnum.p321c02_generator_wall_big_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.GENERATOR, null, MyMaterialType.METAL, 1500f, MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.8f, modelCol: MyModelsEnum.p321c02_generator_wall_big_COL, explosionParticleEffectScale: 1.7f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabGenerator, (int)MyMwcObjectBuilder_PrefabGenerator_TypesEnum.P321C06_INERTIA_GENERATOR_B, new MyPrefabConfigurationGenerator(MyModelsEnum.p321c06_inertia_generator_B, MyModelsEnum.p321c06_inertia_generator_B_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.GENERATOR, null, MyMaterialType.METAL, 1500f, MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.8f, modelCol: MyModelsEnum.p321c06_inertia_generator_B_COL, explosionParticleEffectScale: 1.7f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabGenerator, (int)MyMwcObjectBuilder_PrefabGenerator_TypesEnum.P321C07_GENERATOR, new MyPrefabConfigurationGenerator(MyModelsEnum.p321c07_generator, MyModelsEnum.p321c07_generator_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.GENERATOR, null, MyMaterialType.METAL, 1500f, MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.8f, MyModelsEnum.p321c07_generator_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P341_C01_CLOSED_DOCK_V1, new MyPrefabConfiguration(MyModelsEnum.p341_c01_closed_dock_v1, MyModelsEnum.p341_c01_closed_dock_v1_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p341_c01_closed_dock_v1_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212B02_PANEL_LARGE, new MyPrefabConfiguration(MyModelsEnum.p212b02_panel_large, MyModelsEnum.p212b02_panel_large_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212b02_panel_large_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212C03_PANEL_LARGE, new MyPrefabConfiguration(MyModelsEnum.p212c03_panel_large, MyModelsEnum.p212c03_panel_large_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212c03_panel_large_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212D04_PANEL_LARGE, new MyPrefabConfiguration(MyModelsEnum.p212d04_panel_large, MyModelsEnum.p212d04_panel_large_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212d04_panel_large_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212E05_PANEL_LARGE, new MyPrefabConfiguration(MyModelsEnum.p212e05_panel_large, MyModelsEnum.p212e05_panel_large_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212e05_panel_large_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P212F01_PANEL_LARGE, new MyPrefabConfiguration(MyModelsEnum.p212f01_panel_large, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.PANELS, null, MyMaterialType.METAL, null, MyModelsEnum.p212f01_panel_large_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_D03_HOSPITAL, new MyPrefabConfiguration(MyModelsEnum.p381_d03_hospital, MyModelsEnum.p381_d03_hospital_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 2.5f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_D05_FOOD_GROW, new MyPrefabConfiguration(MyModelsEnum.p381_d05_food_grow, MyModelsEnum.p381_d05_food_grow_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.LIFE_SUPPORT, null, MyMaterialType.METAL));            

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.CABLE_CORNER_25M, new MyPrefabConfiguration(MyModelsEnum.Cable_corner_25m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.CABLES, SubCategoryTypesEnum.WIRES, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Cable_corner_25m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.CABLE_S_45M, new MyPrefabConfiguration(MyModelsEnum.Cable_S_45m, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.CABLES, SubCategoryTypesEnum.WIRES, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Cable_S_45m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.CABLE_STRAIGHT_180, new MyPrefabConfiguration(MyModelsEnum.Cable_straight_180, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.CABLES, SubCategoryTypesEnum.WIRES, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Cable_straight_180_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.CABLE_STRAIGHT_45, new MyPrefabConfiguration(MyModelsEnum.Cable_straight_45, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.CABLES, SubCategoryTypesEnum.WIRES, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Cable_straight_45_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.CABLE_STRAIGHT_90, new MyPrefabConfiguration(MyModelsEnum.Cable_straight_90, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.CABLES, SubCategoryTypesEnum.WIRES, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Cable_straight_90_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.CONNECTION_BOX, new MyPrefabConfiguration(MyModelsEnum.Connection_box, null, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.CABLES, SubCategoryTypesEnum.CONNECTION_BOXES, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Connection_box_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P361_B01_LONG_DISTANCE_ANTENNA_BIG, new MyPrefabConfiguration(MyModelsEnum.p361_b01_long_distance_antenna_big, MyModelsEnum.p361_b01_long_distance_antenna_big_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.COMMUNICATIONS, SubCategoryTypesEnum.LONG_DISTANCE_ANTENNA, MyMaterialType.METAL, rotatingVelocity: MyPrefabConfiguration.DEFAULT_ROTATING_VELOCITY * 0.3f, collisionModelEnum: MyModelsEnum.p361_b01_long_distance_antenna_big_COL, explosionParticleEffectScale: 2.2f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.FOURTH_REICH_WRECK, new MyPrefabConfiguration(MyModelsEnum.fourth_reich_wreck, MyModelsEnum.fourth_reich_wreck_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.fourth_reich_wreck_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P344_A03_CONTAINER, new MyPrefabConfiguration(MyModelsEnum.p344_a03_container, MyModelsEnum.p344_a03_container_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p344_a03_container_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231B01_ARMOR, new MyPrefabConfiguration(MyModelsEnum.p231b01_armor, MyModelsEnum.p231b01_armor_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p231b01_armor_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231B01_ARMOR_CORNER, new MyPrefabConfiguration(MyModelsEnum.p231b01_armor_corner, MyModelsEnum.p231b01_armor_corner_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p231b01_armor_corner_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231B01_ARMOR_EDGE, new MyPrefabConfiguration(MyModelsEnum.p231b01_armor_edge, MyModelsEnum.p231b01_armor_edge_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p231b01_armor_edge_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P231B01_ARMOR_HOLE, new MyPrefabConfiguration(MyModelsEnum.p231b01_armor_hole, MyModelsEnum.p231b01_armor_hole_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.ARMORS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p231b01_armor_hole_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P150A03_SHELF_1, new MyPrefabConfiguration(MyModelsEnum.p150a03_shelf_1, MyModelsEnum.p150a03_shelf_1_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SHELVES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p150a03_shelf_1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P150A02_SHELF_1X2, new MyPrefabConfiguration(MyModelsEnum.p150a02_shelf_1X2, MyModelsEnum.p150a02_shelf_1X2_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SHELVES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p150a02_shelf_1X2_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P150A01_SHELF_1X3, new MyPrefabConfiguration(MyModelsEnum.p150a01_shelf_1X3, MyModelsEnum.p150a01_shelf_1X3_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SHELVES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p150a01_shelf_1X3_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P611_ASTEROID_PART_A, new MyPrefabConfiguration(MyModelsEnum.p611_asteroid_part_A, MyModelsEnum.p611_asteroid_part_A_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P611_ASTEROID_PART_B, new MyPrefabConfiguration(MyModelsEnum.p611_asteroid_part_B, MyModelsEnum.p611_asteroid_part_B_LOD1, BuildTypesEnum.CUSTOM, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P363_A01_BIG_ANTENNA_300M, new MyPrefabConfiguration(MyModelsEnum.p363_a01_big_antenna_300m, MyModelsEnum.p363_a01_big_antenna_300m_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.COMMUNICATIONS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p363_a01_big_antenna_300m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130D02_D_STRAIGHT_300M, new MyPrefabConfiguration(MyModelsEnum.p130d02_d_straight_300m, MyModelsEnum.p130d02_d_straight_300m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130d02_d_straight_300m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P130J01_J_STRAIGHT_300M, new MyPrefabConfiguration(MyModelsEnum.p130j01_j_straight_300m, MyModelsEnum.p130j01_j_straight_300m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.SMALL, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p130j01_j_straight_300m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P120C02_C_STRAIGHT_400M, new MyPrefabConfiguration(MyModelsEnum.p120c02_c_straight_400m, MyModelsEnum.p120c02_c_straight_400m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.MEDIUM, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p120c02_c_straight_400m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B03_LATTICE_BEAM_STRAIGHT_420M, new MyPrefabConfiguration(MyModelsEnum.p110b03_lattice_beam_straight_420m, MyModelsEnum.p110b03_lattice_beam_straight_420m_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b03_lattice_beam_straight_420m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110B04_LATTICE_BEAM_STRAIGHT_420M_WITH_PANELS, new MyPrefabConfiguration(MyModelsEnum.p110b04_lattice_beam_straight_420m_with_panels, MyModelsEnum.p110b04_lattice_beam_straight_420m_with_panels_LOD1, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p110b04_lattice_beam_straight_420m_with_panels_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P363_A01_BIG_ANTENNA_1500M, new MyPrefabConfiguration(MyModelsEnum.p363_a01_big_antenna_1500m, MyModelsEnum.p363_a01_big_antenna_1500m_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.COMMUNICATIONS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p363_a01_big_antenna_1500m_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P4221_A01_COOLING_DEVICE_WALL_340X400, new MyPrefabConfiguration(MyModelsEnum.p4221_a01_cooling_device_wall_340x400, MyModelsEnum.p4221_a01_cooling_device_wall_340x400_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p4221_a01_cooling_device_wall_340x400_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P4222_A01_PIPES_CONNECTOR, new MyPrefabConfiguration(MyModelsEnum.p4222_a01_pipes_connector, MyModelsEnum.p4222_a01_pipes_connector_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p4222_a01_pipes_connector_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P4223_A01_OPEN_PIPE, new MyPrefabConfiguration(MyModelsEnum.p4223_a01_open_pipe, MyModelsEnum.p4223_a01_open_pipe_LOD1, BuildTypesEnum.CONNECTIONS, CategoryTypesEnum.PIPES, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p4223_a01_open_pipe_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P311B01_LONG_TERM_THRUSTER, new MyPrefabConfiguration(MyModelsEnum.p311b01_long_term_thruster, MyModelsEnum.p311b01_long_term_thruster_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221K01_CHAMBER_V2, new MyPrefabConfiguration(MyModelsEnum.p221k01_chamber_v2, MyModelsEnum.p221k01_chamber_v2_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL));

            const float barrelExplosionMultiplier = 2;
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_BARREL_BIOHAZARD, new MyPrefabConfiguration(MyModelsEnum.p581_a01_barrel_biohazard, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.BIOCHEM_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_BARREL_BIOHAZARD_2, new MyPrefabConfiguration(MyModelsEnum.p581_a01_barrel_biohazard_2, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.BIOCHEM_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_NUKE_BARREL, new MyPrefabConfiguration(MyModelsEnum.p581_a01_nuke_barrel, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.NUCLEAR_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_RED_BARREL, new MyPrefabConfiguration(MyModelsEnum.p581_a01_red_barrel, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_SIMPLE_BARREL, new MyPrefabConfiguration(MyModelsEnum.p581_a01_simple_barrel, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_SIMPLE_BARREL_2, new MyPrefabConfiguration(MyModelsEnum.p581_a01_simple_barrel_2, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.BARREL_PROP_A, new MyPrefabConfiguration(MyModelsEnum.Barrel_prop_A, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.BARREL_PROP_B, new MyPrefabConfiguration(MyModelsEnum.Barrel_prop_B, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.BIOCHEM_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.BARREL_PROP_C, new MyPrefabConfiguration(MyModelsEnum.Barrel_prop_C, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.BARREL_PROP_D, new MyPrefabConfiguration(MyModelsEnum.Barrel_prop_D, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            //AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.BARREL_PROP_E, new MyPrefabConfiguration(MyModelsEnum.Barrel_prop_E, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.CANNONBALL_CAPSULE_1, new MyPrefabConfiguration(MyModelsEnum.CannonBall_Capsule_1, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.CannonBall_Capsule_1_COL, explosionType: MyExplosionTypeEnum.BLASTER_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.GATTLING_AMMO_BELT, new MyPrefabConfiguration(MyModelsEnum.gattling_ammo_belt, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.gattling_ammo_belt_COL, explosionType: MyExplosionTypeEnum.BLASTER_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.MISSILE_PACK01, new MyPrefabConfiguration(MyModelsEnum.Missile_pack01, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Missile_pack01_COL, explosionType: MyExplosionTypeEnum.NUCLEAR_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.MISSILE_PACK02, new MyPrefabConfiguration(MyModelsEnum.Missile_pack02, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Missile_pack02_COL, explosionType: MyExplosionTypeEnum.NUCLEAR_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.MISSILE_PLAZMA01, new MyPrefabConfiguration(MyModelsEnum.Missile_plazma01, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Missile_plazma01_COL, explosionType: MyExplosionTypeEnum.PLASMA_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.MISSILE_STACK_BIOCHEM01, new MyPrefabConfiguration(MyModelsEnum.Missile_stack_biochem01, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.Missile_stack_biochem01_COL, explosionType: MyExplosionTypeEnum.BIOCHEM_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_O2_BARREL, new MyPrefabConfiguration(MyModelsEnum.p581_a01_o2_barrel, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.BIOCHEM_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_o2_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.NUCLEAR_WARHEAD_CLOSED, new MyPrefabConfiguration(MyModelsEnum.Nuclear_Warhead_closed, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.NUCLEAR_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, explosionParticleEffectScale: 3.0f, collisionModelEnum: MyModelsEnum.Nuclear_Warhead_closed_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.NUCLEAR_WARHEAD_OPEN, new MyPrefabConfiguration(MyModelsEnum.Nuclear_Warhead_open, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.NUCLEAR_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, explosionParticleEffectScale: 3.0f, collisionModelEnum: MyModelsEnum.Nuclear_Warhead_open_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_NUKE_BARREL_1, new MyPrefabConfiguration(MyModelsEnum.p581_a01_nuke_barrel_1, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.BIOCHEM_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P581_A01_SIMPLE_BARREL_3, new MyPrefabConfiguration(MyModelsEnum.p581_a01_simple_barrel_3, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BARRELS, null, MyMaterialType.METAL, explosionType: MyExplosionTypeEnum.BIOCHEM_EXPLOSION, minSizeForExplosion: 0f, explosionRadiusMultiplier: barrelExplosionMultiplier, collisionModelEnum: MyModelsEnum.p581_a01_universal_barrel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221L01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221L01_chamber_v1, MyModelsEnum.p221L01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221L01_chamber_v1_COL));

            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221M01_CHAMBER_BOTTOM_V1, new MyPrefabConfiguration(MyModelsEnum.p221m01_chamber_bottom_v1, MyModelsEnum.p221m01_chamber_bottom_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221m01_chamber_bottom_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221M01_CHAMBER_CENTER_V1, new MyPrefabConfiguration(MyModelsEnum.p221m01_chamber_center_v1, MyModelsEnum.p221m01_chamber_center_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221m01_chamber_center_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221M01_CHAMBER_TOP_V1, new MyPrefabConfiguration(MyModelsEnum.p221m01_chamber_top_v1, MyModelsEnum.p221m01_chamber_top_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221m01_chamber_top_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P321E01_SOLAR_PANEL, new MyPrefabConfiguration(MyModelsEnum.p321e01_solar_panel, MyModelsEnum.p321e01_solar_panel_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.SUPPLY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p321e01_solar_panel_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P110A06_SOLID_BEAM_STRAIGHT_420M, new MyPrefabConfiguration(MyModelsEnum.p110a06_solid_beam_straight_420m, null, BuildTypesEnum.BEAMS, CategoryTypesEnum.LARGE, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P311B01_CUT_THRUSTER, new MyPrefabConfiguration(MyModelsEnum.p311b01_cut_thruster, MyModelsEnum.p311b01_cut_thruster_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p311b01_cut_thruster_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P312B01_CUT_THRUSTER_LATERAL, new MyPrefabConfiguration(MyModelsEnum.p312b01_cut_thruster_lateral, MyModelsEnum.p312b01_cut_thruster_lateral_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p312b01_cut_thruster_lateral_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P312B02_CUT_THRUSTER_LATITUDE, new MyPrefabConfiguration(MyModelsEnum.p312b02_cut_thruster_latitude, MyModelsEnum.p312b02_cut_thruster_latitude_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.FLIGHT, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p312b02_cut_thruster_latitude_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.ALIEN_DETECTOR_UNIT, new MyPrefabConfiguration(MyModelsEnum.alien_detector_unit, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.alien_detector_unit_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_C01_BUILDING5, new MyPrefabConfiguration(MyModelsEnum.p381_c01_building5, MyModelsEnum.p381_c01_building5_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p381_c01_building5_COL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_C01_BUILDING6, new MyPrefabConfiguration(MyModelsEnum.p381_c01_building6, MyModelsEnum.p381_c01_building6_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p381_c01_building6_COL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P381_C01_BUILDING7, new MyPrefabConfiguration(MyModelsEnum.p381_c01_building7, MyModelsEnum.p381_c01_building7_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p381_c01_building7_COL, explosionParticleEffectScale: 1f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P221N01_CHAMBER_V1, new MyPrefabConfiguration(MyModelsEnum.p221n01_chamber_v1, MyModelsEnum.p221n01_chamber_v1_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p221n01_chamber_v1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P531_D_MEDIC_CROSS, new MyPrefabConfiguration(MyModelsEnum.p531_d_medic_cross, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.SIGNS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p531_d_medic_cross_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.ALIEN_ARTEFACT, new MyPrefabConfiguration(MyModelsEnum.alien_artefact, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.alien_artefact_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.BOMB, new MyPrefabConfiguration(MyModelsEnum.bomb, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.bomb_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.RAIL_GUN, new MyPrefabConfiguration(MyModelsEnum.rail_gun, MyModelsEnum.rail_gun_LOD1, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.rail_gun_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P345_A01_RECYCLE_SPHERE, new MyPrefabConfiguration(MyModelsEnum.p345_a01_recycle_sphere, MyModelsEnum.p345_a01_recycle_sphere_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.INDUSTRY, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p345_a01_recycle_sphere_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.PRISON, new MyPrefabConfiguration(MyModelsEnum.prison, null, BuildTypesEnum.SHELLS, CategoryTypesEnum.CHAMBERS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.prison_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A17_BILLBOARD_PORTRAIT_1, new MyPrefabConfiguration(MyModelsEnum.p511_a17_billboard_portrait_1, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a17_billboard_portrait_1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A17_BILLBOARD_PORTRAIT_2, new MyPrefabConfiguration(MyModelsEnum.p511_a17_billboard_portrait_2, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a17_billboard_portrait_1_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A18_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a18_billboard, MyModelsEnum.p511_a18_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a18_billboard_COL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P511_A19_BILLBOARD, new MyPrefabConfiguration(MyModelsEnum.p511_a19_billboard, MyModelsEnum.p511_a19_billboard_LOD1, BuildTypesEnum.DETAILS, CategoryTypesEnum.BILLBOARDS, null, MyMaterialType.METAL, collisionModelEnum: MyModelsEnum.p511_a19_billboard_COL));

			//Scanners
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabScanner, (int)MyMwcObjectBuilder_PrefabScanner_TypesEnum.Plane, new MyPrefabConfigurationScanner(MyModelsEnum.ScannerPlane, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.SCANNER, null, MyMaterialType.METAL));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabScanner, (int)MyMwcObjectBuilder_PrefabScanner_TypesEnum.Rays, new MyPrefabConfigurationScanner(MyModelsEnum.ScannerRays, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.SCANNER, null, MyMaterialType.METAL));

            // Camera
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabCamera, (int)MyMwcObjectBuilder_PrefabCamera_TypesEnum.DEFAULT, new MyPrefabConfigurationCamera(MyModelsEnum.Cam, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.CAMERA, null, MyMaterialType.METAL));

            //Aliens
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.P391_ALIEN_GATE, new MyPrefabConfiguration(MyModelsEnum.Alien_gate, MyModelsEnum.Alien_gate_LOD1, BuildTypesEnum.MODULES, CategoryTypesEnum.MANNED_OBJECTS, null, MyMaterialType.METAL, explosionParticleEffectScale: 2f));
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.Prefab, (ushort)MyMwcObjectBuilder_Prefab_TypesEnum.DEAD_PILOT, new MyPrefabConfiguration(MyModelsEnum.Debris32_pilot, null, BuildTypesEnum.DETAILS, CategoryTypesEnum.OTHER, null, MyMaterialType.METAL));

            //Alarms
            AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum.PrefabAlarm, (int)MyMwcObjectBuilder_PrefabAlarm_TypesEnum.DEFAULT, new MyPrefabConfigurationAlarm(MyModelsEnum.Alarm_off, null, MyModelsEnum.Alarm, null, BuildTypesEnum.CUSTOM, CategoryTypesEnum.ALARM, null, MyMaterialType.METAL));

            SysUtils.Utils.MyMwcLog.WriteLine("All configurations added");

            Array a = MyGuiPrefabHelpers.MyMwcPrefabTypesEnumValues;
            if (a != null)
            {
                foreach (MyMwcObjectBuilderTypeEnum prefabType in MyGuiPrefabHelpers.MyMwcPrefabTypesEnumValues)
                {
                    foreach (int prefabId in MyMwcObjectBuilder_Base.GetObjectBuilderIDs(prefabType))
                    {
                        MyPrefabConfiguration prefabConfiguration = GetPrefabConfiguration(prefabType, prefabId);
                        MyMwcLog.WriteLine("Prefab configuration type: " + (ushort)prefabType + " id:" + prefabId, LoggingOptions.ENUM_CHECKING);
                        MyCommonDebugUtils.AssertDebug(prefabConfiguration != null);
                    }
                }


                foreach (MyMwcObjectBuilderTypeEnum prefabType in MyGuiPrefabHelpers.MyMwcPrefabTypesEnumValues)
                {
                    foreach (int prefabId in MyMwcObjectBuilder_Base.GetObjectBuilderIDs(prefabType))
                    {
                        MyPrefabConfiguration prefabConfiguration = GetPrefabConfiguration(prefabType, prefabId);
                        prefabConfiguration.ExplosionDamage = m_prefabDealthDamage[(int)prefabConfiguration.CategoryType];

                        if ((prefabConfiguration.CategoryType == CategoryTypesEnum.CHAMBERS) ||
                            (prefabConfiguration.CategoryType == CategoryTypesEnum.MANNED_OBJECTS))
                            prefabConfiguration.ExplosionType = MyExplosionTypeEnum.ASTEROID_EXPLOSION;

                        if ((prefabConfiguration.CategoryType == CategoryTypesEnum.TUNNELS) ||
                            (prefabConfiguration.CategoryType == CategoryTypesEnum.INDUSTRY) ||
                            (prefabConfiguration.CategoryType == CategoryTypesEnum.PASSAGES))
                            prefabConfiguration.ExplosionType = MyExplosionTypeEnum.MEDIUM_PREFAB_EXPLOSION;
                    }
                }

            }

            SysUtils.Utils.MyMwcLog.WriteLine("MyPrefabConstants - END");
        }

        // this method is there because we need load and check asserts when the game started
        public static void Check()
        {
            MyMwcLog.WriteLine("MyPrefabConstants.Check()");
        }

        static void AddPrefabConfiguration(MyMwcObjectBuilderTypeEnum prefabType, ushort prefabId, MyPrefabConfiguration prefabConfiguration)
        {
            ushort prefabTypeInt = (ushort)prefabType;
            if (m_prefabConfigurations[prefabTypeInt] == null)
            {
                m_prefabConfigurations[prefabTypeInt] = new MyPrefabConfiguration[MyMwcObjectBuilder_Base.GetObjectBuilderIDs(prefabType).Max() + 1];
            }
            m_prefabConfigurations[prefabTypeInt][prefabId] = prefabConfiguration;
        }

        public static MyPrefabConfiguration GetPrefabConfiguration(MyMwcObjectBuilderTypeEnum prefabType, int prefabId)
        {
            MyPrefabConfiguration prefabConfiguration = null;
            try
            {
                prefabConfiguration = m_prefabConfigurations[(ushort)prefabType][prefabId];
            }
            catch
            {
            }
            return prefabConfiguration;            
        }

        public static MyPrefabConfiguration GetPrefabConfiguration(MyMwcObjectBuilder_PrefabBase prefabObjectBuilder)
        {
            return GetPrefabConfiguration(prefabObjectBuilder.GetObjectBuilderType(), prefabObjectBuilder.GetObjectBuilderId().Value);
        }

        //static void AddPrefabConfiguration(MyMwcObjectBuilder_Prefab_TypesEnum prefabTypeEnum, MyPrefabConfiguration prefabConfiguration)
        //{
        //    m_prefabConfigurations.Add(prefabTypeEnum, prefabConfiguration);            
        //}

        //public static MyPrefabConfiguration GetPrefabConfiguration(MyMwcObjectBuilder_Prefab_TypesEnum prefabType)
        //{
        //    return m_prefabConfigurations[prefabType];
        //}

        public static IEnumerable<MyPrefabConfiguration> GetPrefabConfigurations()
        {
            foreach (var item in m_prefabConfigurations)
            {
                //yield return item.Value;
                foreach (MyPrefabConfiguration prefabConfiguration in item)
                {
                    yield return prefabConfiguration;
                }
            }
        }
    }
}
